/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <aul.h>
#include <ui-gadget-module.h>

#include "phone.h"
#include "phone-common.h"

#include "ct-list.h"

inline void ctui_ld_to_dd(ct_list_data *clist_d, ct_detail_data *cdetail_d)
{
	cdetail_d->win = clist_d->win;
	cdetail_d->navi = clist_d->navi;
	cdetail_d->base_grp = clist_d->base_grp;
	cdetail_d->base_ab = clist_d->base_ab;
	cdetail_d->ug = clist_d->ug;
	cdetail_d->prev_navi_item = clist_d->navi_item;
	cdetail_d->prev_view_data = clist_d;
}

void ctui_get_number_type_str(contacts_record_h record, int number_type, char *dest, int dest_len)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char* type_str;
	char *custom = NULL;

	if (number_type & CONTACTS_NUMBER_TYPE_CUSTOM) {
		err = contacts_record_get_str_p(record, _contacts_number.label, &custom);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		if (NULL == custom || '\0' == custom[0])
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_OTHER);
		else {
			snprintf(dest, dest_len, "%s", custom);
			return;
		}
	}
	else if (number_type & CONTACTS_NUMBER_TYPE_CELL) {
		if (number_type & CONTACTS_NUMBER_TYPE_HOME)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_HOME2);
		else if (number_type & CONTACTS_NUMBER_TYPE_WORK)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_WORK2);
		else
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_MOBILE);
	}
	else if (number_type & CONTACTS_NUMBER_TYPE_VOICE) {
		if (number_type & CONTACTS_NUMBER_TYPE_HOME)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_HOME);
		else if (number_type & CONTACTS_NUMBER_TYPE_WORK)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_WORK);
		else
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_TELEPHONE);
	}
	else if (number_type & CONTACTS_NUMBER_TYPE_FAX) {
		if (number_type & CONTACTS_NUMBER_TYPE_HOME)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_FAX_HOME);
		else if (number_type & CONTACTS_NUMBER_TYPE_WORK)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_FAX_WORK);
		else
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_FAX);
	}
	else if (number_type & CONTACTS_NUMBER_TYPE_PAGER)
		type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_PAGER);
	else if (number_type & CONTACTS_NUMBER_TYPE_CAR)
		type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_CAR);
	else if (number_type & CONTACTS_NUMBER_TYPE_ASSISTANT)
		type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_ASSISTANT);
	else {
		if (number_type == CONTACTS_NUMBER_TYPE_HOME)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_HOME);
		else if (number_type == CONTACTS_NUMBER_TYPE_WORK)
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_WORK);
		else
			type_str = T_(CT_GET_TEXT_NUM_TYPE, CTTEXT_TYPE_OTHER);
	}

	snprintf(dest, dest_len, "%s", type_str);
}

const char* ctui_get_default_contact_img(int view_type)
{
	switch (view_type) {
	case CT_VIEW_DETAIL:
		return CTUI_IMG_DEFAULT_PHOTO_BIG;

	case CT_VIEW_INPUT:
		return CTUI_IMG_PHOTO_EDIT;

	case CT_VIEW_ETC:
	default:
		return CTUI_IMG_DEFAULT_PHOTO_SMALL;
	}
}

Evas_Object* ctui_create_btn(Evas_Object *parent, const char *label)
{
	Evas_Object *btn;

	btn = elm_button_add(parent);
	p_retvm_if(NULL == btn, NULL, "elm_button_add() return NULL");

	elm_object_text_set(btn, label);
	evas_object_propagate_events_set(btn, EINA_FALSE);
	evas_object_show(btn);

	return btn;
}

void ctui_destroy_child_ug(void *data)
{
	void **child = data;
	*child = NULL;
}

void* ctui_msg_ug(ui_gadget_h parent, const char *number, const int person_id,
		const char *attach, void(*destroy_cb)(void*), void *cb_param)
{
	struct ug_cbs cbs = {0};
	ui_gadget_h ug;
	struct ug_priv_data *priv_data;
	service_h service;

	priv_data = calloc(1, sizeof(struct ug_priv_data));
	p_retvm_if(NULL == priv_data, NULL, "calloc is failed");
	priv_data->destroy_cb = destroy_cb;
	priv_data->cb_param = cb_param;

	service_create(&service);

	if (number && person_id) {
		char temp[PH_TEXT_MAX_LEN+1] = {0,};
		snprintf(temp, sizeof(temp), "%s/%d", number, person_id);
		service_add_extra_data(service, "number/index", temp);
	}
	else if (number) {
		service_add_extra_data(service, "TO", number);
	}

	if (attach) {
		service_add_extra_data(service, "ATTACHFILE", attach);
		service_add_extra_data(service, "SUBJECT", "Contacts name card");
	}

	cbs.layout_cb = ug_common_layout_cb;
	cbs.result_cb = NULL;
	cbs.destroy_cb = ug_common_destroy_cb;
	cbs.priv = priv_data;

	ug = ug_create(parent, MSG_COMPOSER_UG, UG_MODE_FULLVIEW, service, &cbs);
	p_warn_if(NULL == ug, "ug_create() Failed");

	service_destroy(service);
	return ug;
}

void ctui_entry_limit_size_set(Evas_Object *entry)
{
	static Elm_Entry_Filter_Limit_Size filter;
	p_retm_if(NULL == entry, "parameter(entry) is NULL");

	filter.max_char_count = CT_EDITFIELD_MAX_CHAR;
	elm_entry_markup_filter_append(entry, elm_entry_filter_limit_size, &filter);
}

void ctui_genlist_append_separator_10px(Evas_Object *genlist)
{
	static Elm_Genlist_Item_Class itc = { .item_style="dialogue/separator/10" };

	Elm_Object_Item *item;
	item = elm_genlist_item_append(genlist, &itc, NULL, NULL,
			ELM_GENLIST_ITEM_NONE, NULL, NULL);
	elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
}

void ctui_genlist_append_separator_20px(Evas_Object *genlist)
{
	static Elm_Genlist_Item_Class itc = { .item_style="dialogue/separator/20" };

	Elm_Object_Item *item;
	item = elm_genlist_item_append(genlist, &itc, NULL, NULL,
			ELM_GENLIST_ITEM_NONE, NULL, NULL);
	elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
}

void ctui_navi_btn_disabled_set(Evas_Object *navi, Eina_Bool disabled)
{
	Evas_Object *done_button, *delete_item, *share_button;

	p_retm_if(NULL == navi, "parameter(navi) is NULL");

	done_button = evas_object_data_get(navi, "done_button");
	delete_item = evas_object_data_get(navi, "delete_button");
	share_button = evas_object_data_get(navi, "share_button");

	if(done_button)
		elm_object_disabled_set(done_button, disabled);
	if(delete_item)
		elm_object_disabled_set(delete_item, disabled);
	if(share_button)
		elm_object_disabled_set(share_button, disabled);
}

char* ctui_entry_markup_to_utf8(const char *s)
{
	int i;
	char *ret;

	ret = elm_entry_markup_to_utf8(s);
	p_retvm_if(NULL == ret, NULL, "elm_entry_markup_to_utf8() return NULL");

	for (i = 0; i < strlen(ret); i++)
		if (' ' != ret[i]) return ret;

	ret[0] = '\0';
	return ret;
}

void ctui_entry_entry_set(Evas_Object *entry, const char *s)
{
	char *s_markup;
	s_markup = elm_entry_utf8_to_markup(s);
	elm_entry_entry_set(entry, s_markup);
	free(s_markup);
}


enum LIST_DETAIL_ITC_TYPE{
	LIST_DETAIL_ITC_ADDRBOOK,
	LIST_DETAIL_ITC_ADDRBOOK_EDIT,
	LIST_DETAIL_ITC_ADDRBOOK_NAME,
	LIST_DETAIL_ITC_ADDRBOOK_NAME_EDIT,
	LIST_DETAIL_ITC_MAX,
};

static Elm_Genlist_Item_Class list_detail_itcs[] = {
	{.item_style="2text.1icon.9"},
	{.item_style="2text.2icon.5"},
	{.item_style="1text.1icon.2"},
	{.item_style="1text.2icon.2"},
};

static void list_detail_addrbook_gl_del(void *data, Evas_Object *obj)
{
	ct_ab_popup_data *info = data;
	p_retm_if(NULL == info, "data is NULL");
	free(info->name);
	free(info);
}

static char* list_detail_addrbook_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	ct_ab_popup_data *ab_popup_data = data;
	p_retvm_if(NULL == ab_popup_data, NULL, "parameter(ab_popup_data) is NULL");

	if (0 == strcmp(part, "elm.text") || 0 == strcmp(part, "elm.text.2")) {
		if (0 == ab_popup_data->id)
			return strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
		else
			return SAFE_STRDUP(ab_popup_data->name);
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		char buf[PH_TEXT_MAX_LEN+1];
		buf[0] = '\0';
		ctui_get_account_name_by_address_book_id(ab_popup_data->id, buf, sizeof(buf));
		if (*buf)
			return strdup(buf);
	}
	return NULL;
}

static Evas_Object* list_detail_addrbook_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	int selected_id;
	Evas_Object *radio;
	Evas_Object *radio_main;

	ct_ab_popup_data *ab_popup_data = data;
	p_retvm_if(NULL == ab_popup_data, NULL, "parameter(ab_popup_data) is NULL");

	if (0 == strcmp(part, "elm.icon.1") || ab_popup_data->all_contacts_item == true) {
		radio = elm_radio_add(obj);
		p_retvm_if(NULL == radio, NULL, "elm_radio_add() return NULL");

		selected_id = (int)evas_object_data_get(obj, "selected_id");
		if (-1 == ab_popup_data->id) {
			evas_object_data_set(obj, "radio_main", radio);
		}
		else {
			radio_main = evas_object_data_get(obj, "radio_main");
			p_retvm_if(NULL == radio_main, NULL, "evas_object_data_get() return NULL");
			elm_radio_group_add(radio, radio_main);
		}

		elm_radio_state_value_set(radio, ab_popup_data->id);
		if (selected_id == ab_popup_data->id)
			elm_radio_value_set(radio, selected_id);

		evas_object_size_hint_weight_set(radio, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(radio, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_propagate_events_set(radio, EINA_FALSE);

		return radio;
	}
	else if (0 == strcmp(part, "elm.icon") || 0 == strcmp(part, "elm.icon.2")) {
		char *icon_path;
		Evas_Object *icon;
		icon_path = ctui_get_addressbook_icon_path(ab_popup_data->id);

		if (icon_path && *icon_path) {
			icon = elm_icon_add(obj);
			p_retvm_if(NULL == icon, NULL, "elm_icon_add() return NULL");

			elm_icon_file_set(icon, icon_path, NULL);
			evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			free(icon_path);
			return icon;
		}
	}
	return NULL;
}


static inline void list_detail_create_itcs()
{
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK].func.text_get = list_detail_addrbook_gl_label_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK].func.content_get = list_detail_addrbook_gl_icon_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK].func.del = list_detail_addrbook_gl_del;

	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_EDIT].func.text_get = list_detail_addrbook_gl_label_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_EDIT].func.content_get = list_detail_addrbook_gl_icon_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_EDIT].func.del = list_detail_addrbook_gl_del;

	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME].func.text_get = list_detail_addrbook_gl_label_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME].func.content_get = list_detail_addrbook_gl_icon_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME].func.del = list_detail_addrbook_gl_del;

	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME_EDIT].func.text_get = list_detail_addrbook_gl_label_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME_EDIT].func.content_get = list_detail_addrbook_gl_icon_get;
	list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME_EDIT].func.del = list_detail_addrbook_gl_del;
}


Evas_Object* ctui_addrbook_popup(Evas_Object *parent, Evas_Smart_Cb item_sel_cb, void *sel_cb_data, ct_addrbook_popup_type ab_type)
{
	PH_FN_CALL;
	int addressbook_id;
	int addressbook_mode;
	char *addressbook_name;
	Evas_Object *genlist;
	Evas_Object *popup;
	Elm_Object_Item *item;
	ct_ab_popup_data *ab_popup_data;
	Elm_Genlist_Item_Class *itc, *itc_name, *itc_all_contacts;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list = NULL;

	list_detail_create_itcs();

	popup = elm_popup_add(parent);
	p_retvm_if(NULL == popup, NULL, "elm_popup_add() return NULL");
	elm_object_style_set(popup, "liststyle");

	genlist = elm_genlist_add(popup);
	p_retvm_if(NULL == genlist, NULL, "elm_genlist_add() return NULL");
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	if (CT_AB_POPUP_TYPE_LIST_VIEW == ab_type) {
		ct_list_data *clist_d = sel_cb_data;

		itc = &list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_EDIT];
		itc_name = &list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME_EDIT];
		itc_all_contacts = &list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME];

		if (CT_GET_LIST_PERSON_OF_ADDRBOOK_ID == clist_d->list_op)
			evas_object_data_set(genlist, "selected_id", (void *)clist_d->base_ab);
		else
			evas_object_data_set(genlist, "selected_id", (void *)-1);

		/* All contacts */
		ab_popup_data = calloc(1, sizeof(ct_ab_popup_data));
		p_retvm_if(NULL == ab_popup_data, NULL, "calloc() return NULL");
		ab_popup_data->id = -1;
		ab_popup_data->name = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_ALL_CONTACTS));
		ab_popup_data->all_contacts_item = true;

		item = elm_genlist_item_append(genlist, itc_all_contacts, ab_popup_data,
				NULL, ELM_GENLIST_ITEM_NONE, item_sel_cb, sel_cb_data);
		ab_popup_data->item = item;
	}
	else {
		itc = &list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK];
		itc_name = &list_detail_itcs[LIST_DETAIL_ITC_ADDRBOOK_NAME];
	}

	err = contacts_db_get_all_records(_contacts_address_book._uri, 0, 0, &list);
	while (CONTACTS_ERROR_NONE == err) {
		phone_cts_get_int_from_list(list, _contacts_address_book.mode, &addressbook_mode);
		if (addressbook_mode == CONTACTS_ADDRESS_BOOK_MODE_READONLY && CT_AB_POPUP_TYPE_LIST_VIEW != ab_type) {
			err = contacts_list_next(list);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
			continue;
		}

		ab_popup_data = calloc(1, sizeof(ct_ab_popup_data));
		if (NULL == ab_popup_data) {
			ERR("calloc() return NULL");
			continue;
		}

		err = phone_cts_get_int_from_list(list, _contacts_address_book.id, &addressbook_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list() Failed(%d)", err);
		err = phone_cts_get_str_from_list(list, _contacts_address_book.name, &addressbook_name);
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list() Failed(%d)", err);

		ab_popup_data->id = addressbook_id;
		ab_popup_data->mode = addressbook_mode;
		ab_popup_data->name = addressbook_name;

		if (0 == ab_popup_data->id){
				item = elm_genlist_item_append(genlist, itc_name, ab_popup_data, NULL,
					ELM_GENLIST_ITEM_NONE, item_sel_cb, sel_cb_data);
		}
		else{
				item = elm_genlist_item_append(genlist, itc, ab_popup_data, NULL,
					ELM_GENLIST_ITEM_NONE, item_sel_cb, sel_cb_data);
		}

		ab_popup_data->item = item;

		err = contacts_list_next(list);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
	}

	err = contacts_list_destroy(list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err,"contacts_list_destroy() Failed(%d)", err);

	elm_object_content_set(popup, genlist);
	evas_object_show(popup);
	return popup;
}

void ctui_list_ug_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ug_destroy_me(data);
}



