/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#ifndef __PHONE_APP_H__
#define __PHONE_APP_H__

#include <Elementary.h>

#ifndef API
#define API __attribute__ ((visibility("default")))
#endif


#if !defined(PACKAGE)
#  define PACKAGE "phone"
#endif

#define PHONE_PKG "org.tizen.phone"

#define _EDJ(obj) (Evas_Object *)elm_layout_edje_get(obj)

#define PH_DLOG_OUT
#define PH_DEBUGGING

#ifdef PH_DLOG_OUT
#define LOG_TAG "PHONE"
#include <dlog.h>
#define DLOG(prio, fmt, arg...) \
	do { SLOG(prio, LOG_TAG, fmt, ##arg); } while (0)
#define INFO(fmt, arg...) SLOGI(fmt, ##arg)
#define ERR(fmt, arg...) SLOGE("%s[%d]:" fmt, __FUNCTION__, __LINE__, ##arg)
#define DBG(fmt, arg...) SLOGD("%s[%d]:" fmt, __FUNCTION__, __LINE__, ##arg)
#else //PH_DLOG_OUT
#define PRT(prio, fmt, arg...) \
	do { fprintf((prio?stderr:stdout), "[Phone]" fmt"\n", ##arg); } while (0)
#define INFO(fmt, arg...) PRT(0, fmt, ##arg)
#define ERR(fmt, arg...) PRT(1, "%s :" fmt, __FUNCTION__, ##arg)
#define DBG(fmt, arg...) \
	do { \
		printf("\x1b[105;37m[Phone]\x1b[0m(%s)" fmt "\n", __FUNCTION__, ##arg); \
	} while (0)
#endif //PH_DLOG_OUT

#ifdef PH_DEBUGGING
#define PH_FN_CALL DBG(">>>>>>>> called")
#define PH_FN_END DBG("<<<<<<<< ended")
#define PH_DBG(fmt, arg...) DBG(fmt, ##arg)
#else /* PH_DEBUGGING */
#define PH_FN_CALL
#define PH_FN_END
#define PH_DBG(...)
#endif /* PH_DEBUGGING */

#define p_warn_if(expr, fmt, arg...) do { \
	if (expr) { \
		ERR(fmt, ##arg); \
	} \
} while (0)
#define p_ret_if(expr) do { \
	if (expr) { \
		ERR("(%s)", #expr); \
		return; \
	} \
} while (0)
#define p_retv_if(expr, val) do { \
	if (expr) { \
		ERR("(%s)", #expr); \
		return (val); \
	} \
} while (0)
#define p_retm_if(expr, fmt, arg...) do { \
	if (expr) { \
		ERR(fmt, ##arg); \
		return; \
	} \
} while (0)
#define p_retvm_if(expr, val, fmt, arg...) do { \
	if (expr) { \
		ERR(fmt, ##arg); \
		return (val); \
	} \
} while (0)


#define SAFE_STR(src) (src)?src:""
#define STR_BLANK(str, src) (str && src)?" ":"", SAFE_STR(src)
#define SAFE_STRDUP(src) (src)?strdup(src):NULL
#define FREEandSTRDUP(dest, src) \
	do{ \
		free(dest); dest=NULL;\
		if (src) dest = strdup(src); \
	}while(false)

#endif //__PHONE_APP_H__
