/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <fcntl.h>
#include <vconf.h>
#include <Ecore_X.h>
#include <ui-gadget-module.h>
#include <account.h>
#include <unicode/udatpg.h>
#include <unicode/ucal.h>
#include <unicode/ustring.h>

#include "phone.h"
#include "ct-list.h"
#include "ct-detail.h"
#include "ct-detail-view.h"
#include "ct-detail-utils.h"
#include "ct-image-viewer.h"


int ct_detail_parse_bundle(service_h service, ct_detail_data* cdetail_d, int *ret)
{
	int type = CT_UG_REQUEST_DETAIL;
	char* val = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	p_retvm_if(NULL == ret, -1, "parameter(ret) is NULL");
	*ret = 0;

	if (service) {
		service_get_extra_data(service, CT_UG_BUNDLE_TYPE, &val);
		if (val) type = atoi(val);
		PH_DBG("type = %d", type);
		free(val);

		switch (type) {
		case CT_UG_REQUEST_DETAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ID, &val);

			if (val) {
				cdetail_d->person_id = atoi(val);
				free(val);
				break;
			}

			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_PERSON_ID, &val);
			if (val) {
				cdetail_d->person_id = atoi(val);
				free(val);
				break;
			}

			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_VCARD, &val);
			if (val) {
				if(ct_detail_load_vcard_contact(cdetail_d, val) < 0)
					PH_DBG("ct_detail_load_vcard_contact fail");
				free(val);
			}
			break;

		case CT_UG_REQUEST_EDIT:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ID, &val);

			if (val)
				cdetail_d->person_id = atoi(val);
			free(val);

			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_PERSON_ID, &val);
			if (val)
				cdetail_d->person_id = atoi(val);
			free(val);

			if (cdetail_d->person_id) { // From OSP
				Eina_List *list_contact = ctui_person_get_writable_contact_list(cdetail_d->person_id);
				if (list_contact) {
					cdetail_d->contact = list_contact->data;
					eina_list_free(list_contact);
				}
			}

			//number
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_NUM, &val);
			if (val) {
				contacts_record_h record_number;

				if (NULL == cdetail_d->contact && cdetail_d->person_id) {
					Eina_List *list_contact_id = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if (list_contact_id) {
						int contact_id = (int)list_contact_id->data;
						eina_list_free(list_contact_id);
						err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record(%d) Failed(%d)", contact_id, err);
					}
				}

				err = contacts_record_create( _contacts_number._uri, &record_number );
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

				err = contacts_record_set_int( record_number,_contacts_number.type, CONTACTS_NUMBER_TYPE_CELL);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_record_set_str( record_number,_contacts_number.number, (const char*)val );
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

				err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.number, record_number);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
			}

			//email
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &val);
			if (val) {
				contacts_record_h record_email;

				if (NULL == cdetail_d->contact && cdetail_d->person_id) {
					Eina_List *list_contact_id = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if (list_contact_id) {
						int contact_id = (int)list_contact_id->data;
						eina_list_free(list_contact_id);
						err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record(%d) Failed(%d)", contact_id, err);
					}
				}

				contacts_record_create( _contacts_email._uri, &record_email);
				contacts_record_set_int( record_email,_contacts_email.type, CONTACTS_EMAIL_TYPE_HOME);
				contacts_record_set_str( record_email,_contacts_email.email, (const char*)val );

				contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.email, record_email);
			}
			//url
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_WEB, &val);
			if (val) {
				contacts_record_h record_website;

				if (NULL == cdetail_d->contact && cdetail_d->person_id) {
					Eina_List *list_contact_id = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if (list_contact_id) {
						int contact_id = (int)list_contact_id->data;
						eina_list_free(list_contact_id);
						err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record(%d) Failed(%d)", contact_id, err);
					}
				}
				contacts_record_create( _contacts_url._uri, &record_website);
				contacts_record_set_int( record_website,_contacts_url.type, CONTACTS_EMAIL_TYPE_HOME);
				contacts_record_set_str( record_website,_contacts_url.url, (const char*)val );

				contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.url, record_website);
			}
			break;
		case CT_UG_REQUEST_ADD:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_VCARD, &val);
			if (val) {
				contacts_list_h list;
				err = contacts_vcard_parse_to_contacts(val, &list);
				if (CONTACTS_ERROR_NONE == err) {
					contacts_record_h record;
					err = contacts_list_get_current_record_p(list, &record);
					if (CONTACTS_ERROR_NONE != err) {
						err = contacts_record_clone(record, &(cdetail_d->contact));
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_clone() Failed(%d)", err);
					}
				}
				err = contacts_list_destroy(list, true);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

				free(val);
				break;
			}

			//number
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_NUM, &val);
			if (val) {
				contacts_record_h record_number = NULL;
				if (NULL == cdetail_d->contact) {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);
				}

				err = contacts_record_create(_contacts_number._uri, &record_number);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

				err = contacts_record_set_int(record_number, _contacts_number.type, CONTACTS_NUMBER_TYPE_HOME | CONTACTS_NUMBER_TYPE_CELL);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_record_set_str(record_number, _contacts_number.number, val);
				contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.number, record_number);

				free(val);
			}

			//email
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &val);
			if (val) {
				contacts_record_h record_email = NULL;
				if (NULL == cdetail_d->contact) {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);
				}

				err = contacts_record_create(_contacts_email._uri, &record_email);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

				err = contacts_record_set_int(record_email, _contacts_email.type, CONTACTS_EMAIL_TYPE_HOME);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_record_set_str(record_email, _contacts_email.email, val);
				contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.email, record_email);

				free(val);
			}

			//url
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_WEB, &val);
			if (val) {
				contacts_record_h record_url = NULL;
				if (NULL == cdetail_d->contact) {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);
				}
				err = contacts_record_create(_contacts_url._uri, &record_url);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

				err = contacts_record_set_int(record_url, _contacts_url.type, CONTACTS_URL_TYPE_HOME);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_record_set_str(record_url, _contacts_url.url, val);
				contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.url, record_url);

				free(val);
			}
			break;
		case CT_UG_REQUEST_ADD_WITH_NUM:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_NUM, &val);
			if (val) {
				contacts_record_h record_number = NULL;
				if (cdetail_d->person_id) {
					int contact_id = 0;
					Eina_List *contact_id_list;

					contact_id_list = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if(NULL == contact_id_list) {
						ERR("ctui_person_get_writable_contact_list() return NULL");
						break;
					}

					contact_id = (int)contact_id_list->data;
					eina_list_free(contact_id_list);

					err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
					if (CONTACTS_ERROR_NONE != err) {
						ERR("contacts_db_get_record() Failed(%d)", err);
						break;
					}

					err = contacts_record_create(_contacts_number._uri, &record_number);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_number, _contacts_number.type, CONTACTS_NUMBER_TYPE_HOME | CONTACTS_NUMBER_TYPE_VOICE);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_number, _contacts_number.number, val);
					contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.number, record_number);
				}
				else {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_create(_contacts_number._uri, &record_number);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_number, _contacts_number.type, CONTACTS_NUMBER_TYPE_HOME | CONTACTS_NUMBER_TYPE_VOICE);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_number, _contacts_number.number, val);
					contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.number, record_number);
				}
			}
			free(val);
			break;
		case CT_UG_REQUEST_ADD_WITH_EMAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &val);
			if (val) {
				contacts_record_h record_email;
				if (cdetail_d->person_id) {
					int contact_id = 0;
					Eina_List *contact_id_list;

					PH_DBG("cdetail_d->person_id=%d", cdetail_d->person_id);

					contact_id_list = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if(NULL == contact_id_list) {
						ERR("ctui_person_get_writable_contact_list() return NULL");
						break;
					}

					contact_id = (int)contact_id_list->data;
					eina_list_free(contact_id_list);

					err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
					if (CONTACTS_ERROR_NONE != err) {
						ERR("contacts_db_get_record() Failed(%d)", err);
						break;
					}

					err = contacts_record_create(_contacts_email._uri, &record_email);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_email, _contacts_email.type, CONTACTS_EMAIL_TYPE_HOME);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_email, _contacts_email.email, val);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

					err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.email, record_email);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
				}
				else {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_create(_contacts_email._uri, &record_email);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_email, _contacts_email.type, CONTACTS_EMAIL_TYPE_HOME);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_email, _contacts_email.email, val);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

					err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.email, record_email);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
				}
			}
			free(val);
			break;
		case CT_UG_REQUEST_ADD_WITH_WEB:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &val);
			if (val) {
			contacts_record_h record_url;
				if (cdetail_d->person_id) {
					int contact_id = 0;
					Eina_List *contact_id_list;

					contact_id_list = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if(NULL == contact_id_list) {
						ERR("ctui_person_get_writable_contact_list() return NULL");
						break;
					}

					contact_id = (int)contact_id_list->data;
					eina_list_free(contact_id_list);

					err = contacts_db_get_record(_contacts_contact._uri, contact_id, &(cdetail_d->contact));
					if (CONTACTS_ERROR_NONE != err) {
						ERR("contacts_db_get_record() Failed(%d)", err);
						break;
					}

					err = contacts_record_create(_contacts_url._uri, &record_url);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_url, _contacts_url.type, CONTACTS_URL_TYPE_HOME);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_url, _contacts_url.url, val);
					contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.url, record_url);
				}
				else {
					err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_create(_contacts_url._uri, &record_url);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

					err = contacts_record_set_int(record_url, _contacts_url.type, CONTACTS_URL_TYPE_HOME);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

					err = contacts_record_set_str(record_url, _contacts_url.url, val);
					contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.url, record_url);
				}
			}
			free(val);
			break;
		}
	}

	// FIXME : improve performance
	if (cdetail_d->person_id) {
		cdetail_d->list_contacts = ctui_person_get_contact_list(cdetail_d->person_id);
		if (NULL == cdetail_d->list_contacts)
			*ret = -2;
	}

	return type;
}

static inline bool detail_is_usable_char(char c)
{
	switch (c) {
	case '\\':
	case '/':
	case ':':
	case '*':
	case '?':
	case '\"':
	case '<':
	case '>':
	case '|':
	case ';':
		return false;
	default:
		return true;
	}
}

static inline bool detail_is_blank_str(char *display)
{
	int i;
	for (i = 0 ; i < strlen(display); i++)
		if (' ' != display[i]) return false;
	return true;
}

static void detail_get_vcard_filepath(ct_detail_data *cdetail_d, char* path_buf, int buf_size)
{
	char display[PH_TEXT_MAX_LEN];
	char temp[PH_TEXT_MAX_LEN];
	char *display_name = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	temp[0] = '\0';
	if (cdetail_d->person)
		err = contacts_record_get_str_p(cdetail_d->person, _contacts_person.display_name, &display_name);
	else if (cdetail_d->contact)
		err = contacts_record_get_str_p(cdetail_d->contact, _contacts_contact.display_name, &display_name);

	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

	if (display_name && *display_name)
		snprintf(temp, sizeof(temp), "%s", display_name);

	display[0] = '\0';
	if (*temp) {
		int i;
		int idx = 0;
		for (i = 0 ; i < strlen(temp); i++) {
			if (detail_is_usable_char(temp[i]))
				display[idx++] = temp[i];
		}
		display[idx] = '\0';
	}

	if ('\0' == display[0] || detail_is_blank_str(display)) {
		snprintf(display, sizeof(display), "%s", T_(CT_GET_TEXT_ERR, CTTEXT_NONAME));
	}
	snprintf(path_buf, buf_size, "%s/%s.vcf", UGDATAIDR, display);
}

static int detail_make_vcard(ct_detail_data *cdetail_d, char *path_buf, int buf_size)
{
	int ret, fd;
	char* vcard = NULL;

	detail_get_vcard_filepath(cdetail_d, path_buf, buf_size);
	p_retvm_if(NULL == path_buf, FALSE, "detail_get_vcard_filepath() return NULL");
	p_retvm_if(NULL == cdetail_d->list_contacts, FALSE, "contact_list is NULL");

	if (cdetail_d->person)
		ret = contacts_vcard_make_from_person(cdetail_d->person, &vcard);
	else
		ret = contacts_vcard_make_from_contact(cdetail_d->contact, &vcard);

	p_retvm_if(NULL == vcard, FALSE, "contacts_vcard_make_from_person() Failed(%d)", ret);

	fd = open(path_buf, O_WRONLY|O_CREAT|O_TRUNC, 0644);
	if (fd < 0) {
		ERR("Open(%s) Failed", path_buf);
		free(vcard);
		return FALSE;
	}

	if (-1 == write(fd, vcard, strlen(vcard)))
		ERR("write()... Failed(%s)", strerror(errno));
	close(fd);

	free(vcard);
	return TRUE;
}

void ct_detail_delete_namecards(ct_detail_data *cdetail_d)
{
	int ret;
	int fd;
	char file_path[FILENAME_MAX];

	detail_get_vcard_filepath(cdetail_d, file_path, sizeof(file_path));

	fd = open(file_path, O_RDONLY);
	if (fd < 0) return;

	close(fd);

	ret = unlink(file_path);
	p_warn_if(ret < 0, "unlink(%s) failed(%d)", file_path, ret);
}

static void __detail_destroy_child_ug(void *data)
{
	ct_detail_data *cdetail_d = (ct_detail_data*)data;
	p_retm_if(NULL == cdetail_d, "ct_detail_data is NULL");
	cdetail_d->child_ug = NULL;

}


static void detail_destroy_child_ug(ui_gadget_h ug, void *priv)
{
	ct_detail_data *cdetail_d = priv;
	p_retm_if(NULL == ug, "ug is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == priv, "priv is NULL");
	cdetail_d->child_ug = NULL;

}

void ct_detail_make_vcard(ct_detail_data *cdetail_d)
{
	int ret;
	char file_path[FILENAME_MAX] = {0,};

	ret = detail_make_vcard(cdetail_d, file_path, sizeof(file_path));
	if(TRUE != ret)
		cdetail_d->vcard_path = NULL;
	else
		cdetail_d->vcard_path = strdup(file_path);
	PH_DBG("vcf path %s", file_path);
}

void ct_detail_send_via_msg(void *data, Evas_Object *obj, void *event_info)
{
	int ret;
	char file_path[FILENAME_MAX];
	ct_detail_data *cdetail_d = data;

	ret = detail_make_vcard(cdetail_d, file_path, sizeof(file_path));
	if (TRUE != ret) {
		PH_DBG("detail_make_vcard() Failed(%d)", ret);
		phone_show_popup(cdetail_d->win, T_(CT_GET_TEXT_ERR, CTTEXT_UNABLE_TO_SEND_NAME_CARDS), 1.5);
		return;
	}
	cdetail_d->child_ug = ctui_msg_ug(cdetail_d->ug, NULL, 0, file_path,
			__detail_destroy_child_ug, cdetail_d);
	ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);

}

void ct_detail_send_via_email(void *data, Evas_Object *obj, void *event_info)
{
	int ret;
	char file_path[FILENAME_MAX];
	service_h service;
	Ecore_X_Window win_id;
	ct_detail_data *cdetail_d = data;

	ret = detail_make_vcard(cdetail_d, file_path, sizeof(file_path));
	if (TRUE != ret) {
		PH_DBG("detail_make_vcard() Failed(%d)", ret);
		phone_show_popup(cdetail_d->win, T_(CT_GET_TEXT_ERR, CTTEXT_UNABLE_TO_SEND_NAME_CARDS), 1.5);
		return;
	}

	win_id = elm_win_xwindow_get(cdetail_d->win);
	service_create(&service);
	service_set_operation(service, SERVICE_OPERATION_SEND);
	service_set_uri(service, file_path);
	service_set_app_id(service, EMAIL_COMPOSER_PKG);
	service_add_extra_data(service, SERVICE_DATA_SUBJECT, "Contacts name card");

	service_set_window(service, win_id);
	service_send_launch_request(service, NULL, NULL);
	service_destroy(service);

	ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);
}

void ct_detail_send_via_bluetooth(void *data, Evas_Object *obj, void *event_info)
{
	int ret;
	service_h service;
	ct_detail_data *cdetail_d = data;
	char file_path[FILENAME_MAX];
	struct ug_cbs cbs = {0};

	ret = detail_make_vcard(cdetail_d, file_path, sizeof(file_path));
	if (TRUE != ret) {
		PH_DBG("detail_make_vcard() Failed(%d)", ret);
		phone_show_popup(cdetail_d->win, T_(CT_GET_TEXT_ERR, CTTEXT_UNABLE_TO_SEND_NAME_CARDS), 1.5);
		return;
	}

	service_create(&service);
	service_add_extra_data(service, "launch-type", "send");
	service_add_extra_data(service, "filecount", "1");
	service_add_extra_data(service, "files", file_path);

	cbs.layout_cb = ug_common_layout_cb;
	cbs.result_cb = NULL;
	cbs.destroy_cb = detail_destroy_child_ug;
	cbs.priv = data;

	cdetail_d->child_ug = ug_create(cdetail_d->ug, BLUETOOTH_UG, UG_MODE_FULLVIEW, service, &cbs);

	service_destroy(service);
	ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);
}


static void detail_get_displayname(contacts_record_h record_name, char *display, int display_len)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e name_display_order;
	char *first, *last, *middle;
	char temp[CT_EDITFIELD_MAX_CHAR*2+2] = {0,};

	err = contacts_record_get_str_p(record_name, _contacts_name.first, &first);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() first Failed(%d)", err);

	err = contacts_record_get_str_p(record_name, _contacts_name.addition, &middle);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() addition Failed(%d)", err);

	err = contacts_record_get_str_p(record_name, _contacts_name.last, &last);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() last Failed(%d)", err);

	if (STRLEN(middle)>0) {
		if (STRLEN(first)>0)
			snprintf(temp, CT_EDITFIELD_MAX_CHAR*2+1, "%s %s", first, middle);
		else
			snprintf(temp, CT_EDITFIELD_MAX_CHAR, "%s", middle);
	}
	else{
		if (STRLEN(first)>0)
			snprintf(temp, CT_EDITFIELD_MAX_CHAR, "%s", first);
	}

	contacts_setting_get_name_display_order(&name_display_order);
	if( CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST == name_display_order){
		if (STRLEN(last) > 0 ) {
			if (STRLEN(temp)>0)
				snprintf(display, display_len-1, "%s %s", temp, last);
			else
				snprintf(display, display_len-1, "%s", last);
		}
		else{
			if (STRLEN(temp)>0)
				snprintf(display, display_len-1, "%s", temp);
		}
	}
	else{
		if (STRLEN(last) > 0 ) {
			if (STRLEN(temp)>0)
				snprintf(display, display_len-1, "%s, %s", last, temp );
			else
				snprintf(display, display_len-1, "%s", last);
		}
		else{
			if (STRLEN(temp)>0)
				snprintf(display, display_len-1, "%s", temp);
		}
	}
}

static void detail_get_company_str(contacts_record_h record_company, char *dest, int dest_len)
{
	int ret;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	char *company, *job, *department;
	char temp[dest_len];

	p_ret_if(NULL == dest);

	err = contacts_record_get_str_p(record_company, _contacts_company.name, (char**)&company);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
	err = contacts_record_get_str_p(record_company, _contacts_company.department, (char**)&department);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
	err = contacts_record_get_str_p(record_company, _contacts_company.job_title, (char**)&job );
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

	if (company) {
		if(strlen(company)>CT_EDITFIELD_MAX_CHAR){
			company[CT_EDITFIELD_MAX_CHAR] = '\0';
		}
		if (*dest) {
			ret = snprintf(temp, dest_len, "%s, %s", dest, company);
			if (0 < ret)
				memcpy(dest, temp, ret+1);
		}
		else {
			snprintf(dest, dest_len, "%s", company);
		}
	}

	if (department) {
		if(strlen(department)>CT_EDITFIELD_MAX_CHAR){
			department[CT_EDITFIELD_MAX_CHAR] = '\0';
		}
		if (*dest) {
			ret = snprintf(temp, dest_len, "%s, %s", dest, department);
			if (0 < ret)
				memcpy(dest, temp, ret+1);
		}
		else {
			snprintf(dest, dest_len, "%s", department);
		}
	}

	if (job) {
		if(strlen(job)>CT_EDITFIELD_MAX_CHAR){
			job[CT_EDITFIELD_MAX_CHAR] = '\0';
		}
		if (*dest) {
			ret = snprintf(temp, dest_len, "%s, %s", dest, job);
			if (0 < ret)
				memcpy(dest, temp, ret+1);
		}
		else {
			snprintf(dest, dest_len, "%s", job);
		}
	}
}

void ct_detail_person_get_company_str(Eina_List *contact_list, char *dest, int dest_len)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_company = NULL;
	Eina_List *l;
	p_retm_if(NULL == contact_list, "contact_list is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		if (NULL == record_contact)
			continue;
		err = contacts_record_get_child_record_at_p(record_contact, _contacts_contact.company, 0, &record_company);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
		if (CONTACTS_ERROR_NONE == err) {
			detail_get_company_str(record_company, dest, dest_len);
			break;
		}
	}
}

static void detail_get_extra_name_str(contacts_record_h record_name, contacts_record_h record_nickname, char *dest, int dest_len)
{
	int ret;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	char *suffix;
	char *nickname;
	char temp[CT_EDITFIELD_MAX_CHAR+1] = {0,};

	p_ret_if(NULL == dest);

	err = contacts_record_get_str_p(record_name, _contacts_name.suffix, (char**)&suffix);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() suffix Failed(%d)", err);

	err = contacts_record_get_str_p(record_nickname, _contacts_nickname.name, (char**)&nickname);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() name Failed(%d)", err);

	if (suffix && *suffix) {
		if(strlen(suffix)>CT_EDITFIELD_MAX_CHAR){
			suffix[CT_EDITFIELD_MAX_CHAR] = '\0';
		}
		if (*dest) {
			ret = snprintf(temp, sizeof(temp)-1, "%s, %s", dest, suffix);
			if (0 < ret)
				memcpy(dest, temp, ret+1);
		}
		else {
			snprintf(dest, dest_len-1, "%s", suffix);
		}
	}

	if (nickname && *nickname) {
		if(strlen(nickname)>CT_EDITFIELD_MAX_CHAR){
			nickname[CT_EDITFIELD_MAX_CHAR] = '\0';
		}
		if (*dest) {
			ret = snprintf(temp, sizeof(temp)-1, "%s, %s", dest, nickname);
			if (0 < ret)
				memcpy(dest, temp, ret+1);
		}
		else {
			snprintf(dest, dest_len-1, "%s", nickname);
		}
	}
}

void ct_detail_person_get_extra_name(Eina_List *contact_list, contacts_record_h record_person, char *dest, int dest_len)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_name = NULL;
	contacts_record_h record_nickname = NULL;
	Eina_List *l;

	p_retm_if(NULL == contact_list, "contact_list is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		if (NULL == record_contact)
			continue;

		err = contacts_record_get_child_record_at_p(record_contact, _contacts_contact.name, 0, &record_name);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() name Failed(%d)", err);
		err = contacts_record_get_child_record_at_p(record_contact, _contacts_contact.nickname, 0, &record_nickname);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() nickname Failed(%d)", err);

		if (CONTACTS_ERROR_NONE == err) {
			detail_get_extra_name_str(record_name, record_nickname, dest, dest_len);
			break;
		}
	}
}

char* ct_detail_gl_baseinfo_label_get(void *data, Evas_Object *obj, const char *part)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char *display_name = NULL;
	char display[PH_TEXT_MAX_LEN*4+1] ={0,} ;
	ct_detail_data *cdetail_d = data;
	p_retvm_if(NULL == cdetail_d, NULL, "parameter(cdetail_d) is NULL");

	display[0] = '\0';
	if (0 == strcmp(part, "elm.text.name")) {
		if (cdetail_d->person) {
			err = contacts_record_get_str_p(cdetail_d->person, _contacts_person.display_name, (char**)&display_name);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			if (display_name && *display_name)
				return strdup(display_name);
		}
		else if (cdetail_d->contact) {
			err = contacts_record_get_str_p(cdetail_d->contact, _contacts_contact.display_name, (char**)&display_name);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
			if ( STRLEN(display_name)>0 ) {
				snprintf(display, sizeof(display), "%s", display_name);
				return strdup(display);
			}
			else{
				contacts_record_h record_name;

				err = contacts_record_get_child_record_at_p(cdetail_d->contact, _contacts_contact.name, 0, &record_name);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);

				detail_get_displayname(record_name, display, sizeof(display));

				if (STRLEN(display) > 0)
					return strdup(display);
			}
		}
	}
	else if (0 == strcmp(part, "elm.text.subname")) {
		if (cdetail_d->list_contacts) {
			ct_detail_person_get_extra_name(cdetail_d->list_contacts, cdetail_d->person, display, sizeof(display));
			if (strlen(display)>0)
				return strdup(display);

			ct_detail_person_get_company_str(cdetail_d->list_contacts, display, sizeof(display));
			if (strlen(display)>0)
				return strdup(display);
		}
	}
	else if (0 == strcmp(part, "elm.text.subname2")) {
		if (cdetail_d->list_contacts) {
			ct_detail_person_get_company_str(cdetail_d->list_contacts, display, sizeof(display));
			if (strlen(display)>0)
				return strdup(display);
		}
	}
	return NULL;
}


static bool detail_set_favorite(contacts_record_h record_person, bool is_set)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_record_set_bool(record_person, _contacts_person.is_favorite, is_set);
	p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_set_bool() Failed(%d)", err);

	return true;
}

static void detail_favorite_mouseup_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	bool is_favorite;
	bool success;
	int x,y,w,h;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char* result_msg;
	ct_detail_data *cdetail_d = data;
	Evas_Event_Mouse_Up *ev = event_info;

	p_retm_if(NULL == cdetail_d, "evas_object_data_get() return NULL");
	p_retm_if(NULL == cdetail_d->person, "person is NULL");

	if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD)
		return;

	evas_object_geometry_get(obj, &x, &y, &w, &h);
	if (ev->output.y < y || y + h < ev->output.y || ev->output.x < x || x + w < ev->output.x)
		return;

	err = contacts_record_get_bool(cdetail_d->person, _contacts_person.is_favorite, &is_favorite);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_bool() Failed(%d)", err);

	is_favorite = !is_favorite;
	success = detail_set_favorite(cdetail_d->person, is_favorite);

	if (is_favorite && success)
		result_msg = S_(CT_SYS_POP_ADDED);
	else if (!is_favorite && success)
		result_msg = S_(PH_SYS_POP_REMOVED);
	else
		result_msg = S_(PH_SYS_POP_FAILED);

	phone_show_tickernoti(result_msg);
	err = contacts_db_update_record(cdetail_d->person);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);

	elm_genlist_item_update(cdetail_d->base_item);
}

static void detail_person_changed_cb(void *data)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_detail_data *cdetail_d = data;
	p_retm_if(NULL == cdetail_d, "paramter(ct_detail_data) is NULL");
	p_retm_if(NULL == cdetail_d->person, "paramter(cdetail_d->person) is NULL");
	p_retm_if(0 == cdetail_d->person_id, "paramter(cdetail_d->person_id) is 0");

	err = contacts_record_destroy(cdetail_d->person, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

	err = contacts_db_get_record(_contacts_person._uri, cdetail_d->person_id, &(cdetail_d->person));
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_db_get_record(%d) Failed(%d)", cdetail_d->person_id, err);
	}

	elm_genlist_item_update(cdetail_d->base_item);
}

static void detail_image_mouseup_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	int x,y,w,h;

	Evas_Event_Mouse_Up *ev = event_info;
	ct_detail_data *cdetail_d = data;

	if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD)
		return;

	evas_object_geometry_get(obj, &x, &y, &w, &h);
	if (ev->output.y < y || y + h < ev->output.y || ev->output.x < x || x + w < ev->output.x)
		return;

	p_ret_if(NULL != cdetail_d->vcard_path);
	p_retm_if(NULL == cdetail_d->person && NULL == cdetail_d->list_contacts, "parameter(person, list_contacts) is NULL");
	ct_image_viewer(cdetail_d->navi, cdetail_d->person, cdetail_d->list_contacts, detail_person_changed_cb, cdetail_d);
}

Evas_Object *ct_detail_gl_baseinfo_icon_get(void *data, Evas_Object *obj,
		const char *part)
{
	const char *value_data = NULL;
	Evas_Object *img;
	Evas_Object *icon;
	bool is_favorite = false;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_detail_data *cdetail_d = data;
	p_retvm_if(cdetail_d->person == NULL && cdetail_d->contact == NULL, NULL, "parameter(person/contact) is NULL");

	if (0 == strcmp(part, "elm.icon.image")) {

		if (cdetail_d->person)
			err = contacts_record_get_str_p(cdetail_d->person, _contacts_person.image_thumbnail_path, (char**)&value_data);
		else
			err = contacts_record_get_str_p(cdetail_d->contact, _contacts_contact.image_thumbnail_path, (char**)&value_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		img = elm_image_add(obj);
		p_retvm_if(NULL == img, NULL, "elm_image_add() return NULL");
		evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_HORIZONTAL, 1, 1);
		elm_image_file_set(img, value_data, NULL);

		evas_object_event_callback_add(img, EVAS_CALLBACK_MOUSE_UP, detail_image_mouseup_cb, cdetail_d);
		return img;
	}
	else if (0 == strcmp(part, "elm.icon.favorite")) {

		icon = elm_icon_add(obj);
		p_retvm_if(NULL == icon, NULL, "elm_icon_add() return NULL");

		evas_object_event_callback_add(icon, EVAS_CALLBACK_MOUSE_UP, detail_favorite_mouseup_cb, cdetail_d);
		elm_icon_resizable_set(icon, EINA_FALSE, EINA_FALSE);

		if (cdetail_d->person)
			err = contacts_record_get_bool(cdetail_d->person, _contacts_person.is_favorite, &is_favorite);
		else{
			if(cdetail_d->vcard_path) return NULL;
			err = contacts_record_get_bool(cdetail_d->contact, _contacts_contact.is_favorite, &is_favorite);
		}
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_bool() Failed(%d)", err);

		elm_icon_file_set(icon, CTUI_IMG_ICON_FAVORITE, NULL);
		if (!is_favorite)
			evas_object_color_set(icon, 185, 185, 185, 255);
		else
			evas_object_color_set(icon, 59, 115, 182, 255);

		return icon;
	}
	return NULL;
}


static void __detail_voice_call_pressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	p_retm_if(NULL == data, "data is NULL");
	elm_icon_file_set(data, CTUI_IMG_ICON_CALL_PRESSED, NULL);
}

static void __detail_voice_call_unpressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	p_retm_if(NULL == data, "data is NULL");
	elm_icon_file_set(data, CTUI_IMG_ICON_CALL, NULL);
}

static void __detail_msg_ug_pressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	p_retm_if(NULL == data, "data is NULL");
	elm_icon_file_set(data, CTUI_IMG_ICON_SMS_PRESSED, NULL);
}

static void __detail_msg_ug_unpressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	p_retm_if(NULL == data, "data is NULL");
	elm_icon_file_set(data, CTUI_IMG_ICON_SMS, NULL);
}

Evas_Object *ct_detail_gl_number_util_icon_get(void *data, Evas_Object *obj, const char *part)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	char *fullpath;
	const char *number;
	contacts_record_h default_nubmer = data;
	ct_detail_data *cdetail_d;
	Evas_Object *btn;
	Evas_Object *icon;
	Evas_Smart_Cb clicked_fn = NULL;
	Evas_Smart_Cb pressed_fn = NULL;
	Evas_Smart_Cb unpressed_fn = NULL;

	p_retv_if(NULL == data, NULL);

	cdetail_d = evas_object_data_get(obj, "cdetail_d");
	p_retvm_if(NULL == cdetail_d, NULL, "evas_object_data_get() return NULL");
	if (0 == strcmp(part, "elm.icon.1")) {
		fullpath = CTUI_IMG_ICON_CALL;
		clicked_fn = ct_detail_voice_call_cb;
		pressed_fn = __detail_voice_call_pressed_cb;
		unpressed_fn = __detail_voice_call_unpressed_cb;
	}
	else if (0 == strcmp(part, "elm.icon.2")) {
		fullpath = CTUI_IMG_ICON_SMS;
		clicked_fn = ct_detail_msg_ug_cb;
		pressed_fn = __detail_msg_ug_pressed_cb;
		unpressed_fn = __detail_msg_ug_unpressed_cb;
	}
	else {
		return NULL;
	}


	err = contacts_record_get_str_p(default_nubmer, _contacts_number.number, (char**)&number);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

	btn = elm_button_add(obj);
	p_retvm_if(NULL == btn, NULL, "elm_button_add() return NULL");
	elm_object_style_set(btn, "detail.circle/empty");

	icon = elm_icon_add(btn);
	p_retvm_if(NULL == icon, NULL, "phone_create_icon_with_img() return NULL");
	elm_icon_file_set(icon, fullpath, NULL);
	elm_icon_resizable_set(icon, EINA_TRUE, EINA_TRUE);
	elm_object_part_content_set(btn, "elm.swallow.content", icon);
	evas_object_data_set(btn, "cdetail_d", cdetail_d);
	evas_object_smart_callback_add(btn, "clicked", clicked_fn, number);
	evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_DOWN, pressed_fn, icon);
	evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, unpressed_fn, icon);

	return btn;
}

void ct_detail_voice_call_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_detail_data *cdetail_d;

	cdetail_d = evas_object_data_get(obj, "cdetail_d");
	p_retm_if(NULL == cdetail_d, "evas_object_data_get() return NULL");

	if (cdetail_d->popup)
		ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);

	phone_launch_voice_call(cdetail_d->person_id, data);
}

void ct_detail_msg_ug_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_detail_data *cdetail_d;

	cdetail_d = evas_object_data_get(obj, "cdetail_d");
	p_retm_if(NULL == cdetail_d, "evas_object_data_get() return NULL");
	p_retm_if(NULL != cdetail_d->child_ug, "child_ug is not null");

	if (cdetail_d->popup)
		ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);

	cdetail_d->child_ug = ctui_msg_ug(cdetail_d->ug, data, cdetail_d->person_id, NULL, __detail_destroy_child_ug, cdetail_d);
}


void ct_detail_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_detail_data *cdetail_d = data;
	evas_object_del(cdetail_d->popup);
	cdetail_d->popup = NULL;
	elm_object_tree_focus_allow_set(cdetail_d->navi, EINA_TRUE);
}

const char* ct_detail_get_ringtone_display_name(const char *real_data)
{
	const char *data;

	if (NULL == real_data)
		data = T_(CT_GET_TEXT_BASIC, CTTEXT_DEFAULT);
	else {
		data = strrchr(real_data, '/');
		if (NULL == data)
			data = real_data;
		else
			data = data + 1;
	}
	return data;
}

const char* ct_detail_get_vibration_display_name(const char *real_data)
{
	const char *data;

	if (NULL == real_data)
		data = T_(CT_GET_TEXT_BASIC, CTTEXT_DEFAULT);
	else {
		data = strrchr(real_data, '/');
		if (NULL == data)
			data = real_data;
		else
			data = data + 1;
	}
	return data;
}

void ct_detail_get_group_names(contacts_record_h record_contact, char *dest, int dest_len)
{
	PH_FN_CALL;
	int ret;
	int index = 0;
	int group_id;
	char *group_str;
	char temp[PH_TEXT_MAX_LEN];
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	contacts_record_h group_record = NULL;
	dest[0] = '\0';

	while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.group_relation, index++, &record)) {
		err = contacts_record_get_int(record, _contacts_group_relation.group_id, &group_id);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_record_get_int() Failed(%d)", err);
			continue;
		}
		PH_DBG("group id = %d",group_id);
		err = contacts_db_get_record(_contacts_group._uri,group_id, &group_record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);
		if(err == CONTACTS_ERROR_NONE){
			err = contacts_record_get_str_p(group_record, _contacts_group.name, &group_str);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
			PH_DBG("group name = %s",group_str);
			if (group_str && *group_str) {
				if (*dest) {
					ret = snprintf(temp, PH_TEXT_MAX_LEN, "%s, %s", dest, group_str);
					if (ret < dest_len) {
						memcpy(dest, temp, ret + 1);
					}
					else {
						memcpy(dest, temp, dest_len);
						dest[dest_len - 1] = '\0';
					}
				}
				else {
					snprintf(dest, PH_TEXT_MAX_LEN, "%s", group_str);
				}
			}
		}
		err = contacts_record_destroy(group_record,true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	}

	if ('\0' == *dest)
		snprintf(dest, dest_len, "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_NOT_ASSIGNED));
}

void ct_detail_person_get_group_names(Eina_List *contact_list, char *dest, int dest_len)
{
	PH_FN_CALL;
	int ret;
	int index = 0;
	Eina_List *l;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_group_relation = NULL;
	char* group_names = NULL;
	p_retm_if(NULL == contact_list, "contact_list is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		if (NULL == record_contact)
			continue;
		while(CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact,
				_contacts_contact.group_relation, index++, &record_group_relation)) {

			err = contacts_record_get_str_p(record_group_relation, _contacts_group_relation.name, &group_names);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
			PH_DBG("group_names = %s",group_names);
			if (group_names && *group_names) {
				if (*dest) {
					char temp[PH_TEXT_MAX_LEN];
					ret = snprintf(temp, sizeof(temp), "%s, %s", dest, group_names);
					if (0 < ret)
						memcpy(dest, temp, ret+1);
				}
				else {
					snprintf(dest, dest_len, "%s", group_names);
				}
			}
		}
		index = 0;
	}
	if ('\0' == *dest)
		snprintf(dest, dest_len, "%s", T_(CT_GET_TEXT_BASIC, CTTEXT_NOT_ASSIGNED));
}

void ct_detail_refresh_view(ct_detail_data *cdetail_d)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	if (cdetail_d->popup)
		ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);

	if (cdetail_d->person) {
		err = contacts_record_destroy(cdetail_d->person, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
		cdetail_d->person = NULL;
	}
	ctui_free_record_list(cdetail_d->list_contacts);
	cdetail_d->list_contacts = NULL;

	if (cdetail_d->person_id) {
		err = contacts_db_get_record( _contacts_person._uri, cdetail_d->person_id,  &(cdetail_d->person));
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_db_get_record() Failed(%d)", err);
			elm_naviframe_item_pop(cdetail_d->navi);
			return;
		}

		cdetail_d->list_contacts = ctui_person_get_contact_list(cdetail_d->person_id);
	}
	else{
		contacts_list_h list_contacts = NULL;
		int fd = 0;
		int size = 0;
		char buf[CT_VCARD_FILE_MAX_SIZE+1] = {0, };

		fd = open(cdetail_d->vcard_path, O_RDONLY);
		p_retm_if(-1 == fd, "open() return -1");
		size = read(fd, buf, sizeof(buf)-1);
		close(fd);

		err = contacts_vcard_parse_to_contacts(buf, &list_contacts);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_vcard_parse_to_contacts() Failed(%d)", err);
		while (CONTACTS_ERROR_NONE == err) {
			err = contacts_list_get_current_record_p(list_contacts, &(cdetail_d->contact));
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				err = contacts_list_destroy(list_contacts, true);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy(list_contacts) Failed(%d)", err);
				elm_exit();
			}
			cdetail_d->list_contacts = eina_list_append(cdetail_d->list_contacts, cdetail_d->contact);
		}
		err = contacts_list_destroy(list_contacts, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy(list_contacts) Failed(%d)", err);
	}
	ctui_detail_genlist_update(cdetail_d);
}

void ct_detail_contact_changed(const char *view_uri, void *user_data)
{
	PH_FN_CALL;
	int addressbook_id = 0;
	int contact_id = 0;
	int changed_id = 0;
	int changed_type = 0;
	int out_current_version = 0;
	bool changed =  false;
	ct_detail_data *cdetail_d = user_data;
	Eina_List *l;
	contacts_record_h record_contact;
	contacts_list_h out_change_list;
	contacts_error_e err;

	EINA_LIST_FOREACH(cdetail_d->list_contacts, l, record_contact) {
		if (NULL == record_contact) continue;
		err = contacts_record_get_int(record_contact, _contacts_contact.address_book_id, &addressbook_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list() Failed(%d)", err);

		err = contacts_record_get_int(record_contact, _contacts_contact.id, &contact_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list() Failed(%d)", err);

		changed = false;
		err = contacts_db_get_changes_by_version( _contacts_contact_updated_info._uri, addressbook_id, cdetail_d->view_create_version, &out_change_list, &out_current_version);

		while (CONTACTS_ERROR_NONE == err) {
			err = phone_cts_get_int_from_list(out_change_list, _contacts_contact_updated_info.contact_id, &changed_id);
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list(addressbook_id) Failed(%d)", err);
			if (changed_id == contact_id){
				changed = true;
				phone_cts_get_int_from_list(out_change_list, _contacts_contact_updated_info.type, &changed_type);
				p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_int_from_list(type) Failed(%d)", err);
				break;
			}
			err = contacts_list_next(out_change_list);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
		}

		err = contacts_list_destroy(out_change_list, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy(out_change_list) Failed(%d)", err);

		if (changed)
			break;
	}
	cdetail_d->view_create_version = out_current_version;

	if (cdetail_d->updated)
		cdetail_d->updated = false;

	if (CONTACTS_CHANGE_DELETED == changed_type) {
		if (cdetail_d->is_shortcut)
			elm_exit();
		else if (NULL == cdetail_d->prev_view_data && cdetail_d->ug)
			ug_destroy_me(cdetail_d->ug);
		else
			elm_naviframe_item_pop_to(cdetail_d->prev_navi_item);
	}
	else {
		ct_detail_refresh_view(cdetail_d);
	}
}

static inline void detail_append_str_with_linebreak(const char *src, char *dest, int dest_len)
{
	char temp[PH_TEXT_MAX_LEN];
	if (src && '\0' != src[0]) {
		if ('\0' == dest[0])
			snprintf(dest, dest_len, "%s", src);
		else {
			snprintf(temp, sizeof(temp), "%s\n%s", src, dest);
			snprintf(dest, dest_len, "%s", temp);
		}
	}
}

int ct_detail_get_full_addr(contacts_record_h record, char *buf, int buf_len)
{
	const char *code;
	const char *region;
	const char *locality;
	const char *street;
	const char *extended;
	const char *country;
	const char *pobox;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_record_get_str_p(record, _contacts_address.postal_code , (char**)&code);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(postal_code) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.extended , (char**)&extended);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(extend) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.postbox , (char**)&pobox);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(postbox) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.street , (char**)&street);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(street) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.locality , (char**)&locality);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(locality) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.region , (char**)&region);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(region) Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_address.country ,(char**)&country);
	p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list(country) Failed(%d)", err);

	detail_append_str_with_linebreak(country, buf, buf_len);
	detail_append_str_with_linebreak(region, buf, buf_len);
	detail_append_str_with_linebreak(locality, buf, buf_len);
	detail_append_str_with_linebreak(street, buf, buf_len);
	detail_append_str_with_linebreak(pobox, buf, buf_len);
	detail_append_str_with_linebreak(extended, buf, buf_len);
	detail_append_str_with_linebreak(code, buf, buf_len);

	return 0;

}

void ct_detail_mouseup(void *data, Evas *evas, Evas_Object *obj,
		void *event_info)
{
	Evas_Event_Mouse_Up *ev = event_info;
	ct_detail_data *cdetail_d = data;
	cdetail_d->touch_x = ev->canvas.x;
	cdetail_d->touch_y = ev->canvas.y;
}

static void detail_myfile_ringtone_result(ui_gadget_h ug, service_h service, void *data)
{
	char *val = NULL;
	ct_detail_data *cdetail_d = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	p_retm_if(NULL == ug || NULL == data, "ug or data is NULL");

	if (service) {
		service_get_extra_data(service, "result", &val);
		p_retm_if(!val, "return value is NULL");
		p_retm_if(NULL == cdetail_d->person, "person is NULL");

		err = contacts_record_set_str(cdetail_d->person, _contacts_person.ringtone_path, (const char *)val);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);
		err = contacts_db_update_record(cdetail_d->person);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
		ctui_create_contacts_error_popup(cdetail_d->navi, err);

		elm_genlist_item_update(cdetail_d->ringtone_item);
		free(val);
	}
}

void ct_detail_myfile_ringtone(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	struct ug_cbs cbs = {0};

	service_h service;
	char *str = NULL;
	char buf[PH_TEXT_MAX_LEN+1];
	const char *ringtone_path = NULL;
	ct_detail_data *cdetail_d = data;

	err = contacts_record_get_str_p(cdetail_d->person, _contacts_person.ringtone_path, &str);
	if(CONTACTS_ERROR_NONE == err)
		ringtone_path = ct_detail_get_ringtone_display_name(str);

	service_create(&service);
	service_add_extra_data(service, "path", "/opt/share/settings/Ringtones");
	service_add_extra_data(service, "select_type", "SINGLE_FILE");
	service_add_extra_data(service, "file_type", "SOUND");
	if (ringtone_path) {
		snprintf(buf, sizeof(buf), "/opt/share/settings/Ringtones/%s", ringtone_path);
		service_add_extra_data(service, "marked_mode", buf);
	}

	cbs.result_cb = detail_myfile_ringtone_result;
	cbs.destroy_cb = detail_destroy_child_ug;
	cbs.layout_cb = ug_common_layout_cb;
	cbs.priv = cdetail_d;

	cdetail_d->child_ug = ug_create(cdetail_d->ug, MYFILES_UG, UG_MODE_FULLVIEW, service, &cbs);
	p_warn_if(NULL == cdetail_d->child_ug, "ug_create() Failed");

	service_destroy(service);

	if (cdetail_d->popup) {
		evas_object_del(cdetail_d->popup);
		cdetail_d->popup = NULL;
	}
}

static void detail_vibration_result(ui_gadget_h ug, service_h service, void *data)
{
	char *val = NULL;
	ct_detail_data *cdetail_d = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	p_retm_if(NULL == ug || NULL == data, "ug or data is NULL");

	if (service) {
		service_get_extra_data(service, "result", &val);
		p_retm_if(!val, "return value is NULL");
		p_retm_if(NULL == cdetail_d->person, "person is NULL");

		err = contacts_record_set_str(cdetail_d->person, _contacts_person.vibration, (const char *)val);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);
		err = contacts_db_update_record(cdetail_d->person);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
		ctui_create_contacts_error_popup(cdetail_d->navi, err);

		elm_genlist_item_update(cdetail_d->vibration_item);
		free(val);
	}
}

void ct_detail_myfile_vibration(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char *vibration_path = NULL;
	struct ug_cbs cbs = {0};
	service_h service;
	ct_detail_data *cdetail_d = data;

	err = contacts_record_get_str_p(cdetail_d->person, _contacts_person.vibration, &vibration_path);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)",err);

	service_create(&service);
	service_add_extra_data(service, "caller", "org.tizen.phone");
	if (vibration_path) {
		service_add_extra_data(service, "selected", vibration_path);
	}

	cbs.result_cb = detail_vibration_result;
	cbs.destroy_cb = detail_destroy_child_ug;
	cbs.layout_cb = ug_common_layout_cb;
	cbs.priv = cdetail_d;

	cdetail_d->child_ug = ug_create(cdetail_d->ug, VIBRATION_UG, UG_MODE_FULLVIEW, service, &cbs);
	p_warn_if(NULL == cdetail_d->child_ug, "ug_create() Failed");

	service_destroy(service);
	if (cdetail_d->popup) {
		evas_object_del(cdetail_d->popup);
		cdetail_d->popup = NULL;
	}
}

static Evas_Object* detail_list_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_person_list* person_info = data;
	contacts_record_h record_person = NULL;

	p_retvm_if(NULL == person_info, NULL, "person_info is NULL");

	if (0 == strcmp(part, "elm.icon.1")) {
		const char *path;
		Evas_Object *icon;

		if (!person_info->img_loaded) {
			err = contacts_db_get_record( _contacts_person._uri, person_info->id, &record_person);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record(%d) Failed(%d)", person_info->id, err);

			err = contacts_record_get_str(record_person, _contacts_person.image_thumbnail_path, &(person_info->img_path));
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);
			person_info->img_loaded = true;

			err = contacts_record_destroy(record_person, true);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
		}

		path = person_info->img_path;
		if (NULL == path) return NULL;

		icon = elm_icon_add(obj);
		if (icon) {
			elm_icon_prescale_set(icon, CT_LIST_ICON_SIZE);
			elm_icon_file_set(icon, path, NULL);
			evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			evas_object_show(icon);
		}
		return icon;
	}
	else if (0 == strcmp(part, "elm.icon.2")) {
		Evas_Object *icon;
		char *icon_path;
		int addressbook_id = ctui_get_addressbook_id_by_index(person_info->addressbook_id_list, 0);

		if(addressbook_id < 0)
			return NULL;

		icon_path = ctui_get_addressbook_icon_path(addressbook_id);
		if (NULL == icon_path)
			return NULL;

		icon = elm_icon_add(obj);
		p_retvm_if(NULL == icon, NULL, "elm_icon_add() return NULL");

		elm_icon_file_set(icon, icon_path, NULL);
		evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		free(icon_path);
		return icon;
	}
	else if (0 == strcmp(part, "elm.icon.3")) {
		return NULL;
	}
	else if (0 == strcmp(part, "elm.icon.4")) {
		return NULL;
	}
	return NULL;
}

static void detial_list_create_itcs(ct_list_data *clist_d)
{
	clist_d->itcs[CT_LIST_ITEM_NO_NUMBER].item_style = "link_list";
	clist_d->itcs[CT_LIST_ITEM_NO_NUMBER].func.text_get = ct_list_gl_label_get;
	clist_d->itcs[CT_LIST_ITEM_NO_NUMBER].func.content_get = detail_list_gl_icon_get;

	clist_d->itcs[CT_LIST_ITEM].item_style = "link_list";
	clist_d->itcs[CT_LIST_ITEM].func.text_get = ct_list_gl_label_get;
	clist_d->itcs[CT_LIST_ITEM].func.content_get = detail_list_gl_icon_get;

	clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER].item_style = "link_list.tb";
	clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER].func.text_get = ct_list_gl_label_get;
	clist_d->itcs[CT_LIST_ITEM_SEARCH_NO_NUMBER].func.content_get = detail_list_gl_icon_get;

	clist_d->itcs[CT_LIST_ITEM_SEARCH].item_style = "link_list.tb";
	clist_d->itcs[CT_LIST_ITEM_SEARCH].func.text_get = ct_list_gl_label_get;
	clist_d->itcs[CT_LIST_ITEM_SEARCH].func.content_get = detail_list_gl_icon_get;

	clist_d->itcs[CT_LIST_ITEM_COUNT].item_style="1text_center";
	clist_d->itcs[CT_LIST_ITEM_COUNT].func.text_get = ct_list_count_gl_label_get;
}

void ct_detail_create_list_view(ct_detail_data *cdetail_d, Evas_Smart_Cb sel_cb, void *cb_data, int id_caller)
{
	ct_list_data *clist_d;
	char buf[PH_TEXT_MAX_LEN+1];
	Evas_Object *btn;

	clist_d = calloc(1, sizeof(ct_list_data));
	clist_d->list_op = CT_GET_LIST_ALL_PERSON;
	clist_d->selected_person_id = id_caller;
	clist_d->navi = cdetail_d->navi;
	clist_d->win = cdetail_d->win;
	clist_d->ug = cdetail_d->ug;
	clist_d->gl_sel = sel_cb;
	clist_d->gl_sel_data = cb_data;

	ct_list_init_account_data(clist_d);
	ct_list_get_contacts(clist_d, &clist_d->list);
	detial_list_create_itcs(clist_d);
	clist_d->base_layout = ct_list_create_contact_layout(clist_d);

	FREEandSTRDUP(clist_d->title, T_(CT_GET_TEXT_BASIC, CTTEXT_ALL_CONTACTS));
	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);
	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);

	btn = elm_button_add(clist_d->navi);
	elm_object_style_set(btn, "naviframe/back_btn/default");
	evas_object_smart_callback_add(btn, "clicked", phone_view_back_btn_cb, clist_d->navi);
	elm_object_item_part_content_set(clist_d->navi_item, "prev_btn", btn);
}

static void __detail_ug_error_popup_cb(void *data, Evas_Object *obj, void *event_info)
{
	ug_destroy_me(data);
}

int ct_detail_load_vcard_contact(ct_detail_data *cdetail_d, char *path)
{
	int ret;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	FILE *fp;
	char vcard_stream[1024*100];
	PH_DBG("path = %s", path);

	fp = fopen(path, "r");
	if (NULL == fp) {
		ERR("fopen() return NULL");
		elm_exit();
		return -1;
	}
	ret = fread(vcard_stream, 1, sizeof(vcard_stream), fp);
	if (0 < ret && ret < sizeof(vcard_stream)) {
		int fd = 0;
		int size = 0;
		char buf[CT_VCARD_FILE_MAX_SIZE+1] = {0, };
		contacts_record_h record = NULL;
		contacts_list_h list_contacts = NULL;
		vcard_stream[ret] = '\0';

		cdetail_d->vcard_path = strdup(path);

		fd = open(cdetail_d->vcard_path, O_RDONLY);
		if(fd < 0){
			close(fp);
			ERR("vOpen(%s) Failed", cdetail_d->vcard_path);
			return -1;
		}
		size = read(fd, buf, sizeof(buf)-1);
		close(fd);

		err = contacts_vcard_parse_to_contacts(buf, &list_contacts);
		while (CONTACTS_ERROR_NONE == err) {
			contacts_list_get_current_record_p(list_contacts, &record);
			if (NULL == cdetail_d->contact) cdetail_d->contact = record;
			cdetail_d->list_contacts = eina_list_append(cdetail_d->list_contacts, record);
			err = contacts_list_next(list_contacts);
		}
		err = contacts_list_destroy(list_contacts, false);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_vcard_parse_to_contacts() Failed(%d)", err);
			fclose(fp);
			return -1;
		}
	}
	else {
		ERR("vcard is invalid(%d)", ret);
		cdetail_d->popup = phone_show_popup(cdetail_d->win, S_(CT_SYS_POP_ERROR), 1.5);
		evas_object_smart_callback_add(cdetail_d->popup, "timeout", __detail_ug_error_popup_cb, cdetail_d->ug);
	}
	fclose(fp);
	return 0;
}

void ct_detail_person_get_addressbook_name(contacts_record_h record_contact, char *buf, int buf_len)
{
	int addressbook_id;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	const char *display_name;
	char temp[PH_TEXT_MAX_LEN];
	contacts_record_h record_addressbook = NULL;

	buf[0] = '\0';

	err = contacts_record_get_int(record_contact, _contacts_contact.address_book_id, &addressbook_id);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int(address_book_id) Failed(%d)", err);

	err = contacts_db_get_record(_contacts_address_book._uri, addressbook_id, &record_addressbook);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

	err = contacts_record_get_str_p(record_addressbook, _contacts_address_book.name, (char**)&display_name);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

	if (display_name && *display_name) {
		if (*buf) {
			snprintf(temp, sizeof(temp), "%s, %s", buf, display_name);
			snprintf(buf, buf_len, "%s", temp);
		}
		else {
			snprintf(buf, buf_len, "%s", display_name);
		}
	}
	err = contacts_record_destroy(record_addressbook, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
}

static bool __phone_list_is_postal_exist(Eina_List *list, const char *postal_str)
{
	Eina_List *l;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, l, record_temp) {
		char postaladdress_tmp[PH_TEXT_MAX_LEN] = {0,};
		if (NULL == record_temp)
			continue;

		ct_detail_get_full_addr(record_temp, postaladdress_tmp, sizeof(postaladdress_tmp));
		if (0 == strcmp(postaladdress_tmp, postal_str)) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_postal_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index = 0;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Eina_List *l;
	Eina_List *list_postal_record = NULL;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_address = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.address, index++, &record_address)) {
			char postaladdress[PH_TEXT_MAX_LEN] = {0,};
			ct_detail_get_full_addr(record_address, postaladdress, sizeof(postaladdress));
			if(checkduplicate){
				if (__phone_list_is_postal_exist(list_postal_record, postaladdress)) {
					ERR("__phone_list_is_postal_exist() return true number = %s", postaladdress);
					continue;
				}
			}
			err = contacts_record_clone(record_address, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_record_clone() Failed(%d)", err);
				break;
			}
			list_postal_record = eina_list_append(list_postal_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_postal_record;
}

static bool __phone_list_is_messenger_exist(Eina_List *list, const char *messenger)
{
	Eina_List *l;
	char *messenger_temp;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, l, record_temp) {
		if (NULL == record_temp)
			continue;

		err = contacts_record_get_str_p(record_temp, _contacts_messenger.im_id, &messenger_temp);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_str_p() Failed(%d)", err);

		if (NULL == messenger_temp)
			continue;

		if (0 == strcmp(messenger_temp, messenger)) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_messenger_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index = 0;
	char *messenger;
	Eina_List *l;
	Eina_List *list_messenger_record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_messenger = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.messenger, index++, &record_messenger)) {
			err = contacts_record_get_str_p(record_messenger, _contacts_messenger.im_id, &messenger);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			if(checkduplicate){
				if (__phone_list_is_messenger_exist(list_messenger_record, messenger)) {
					ERR("__phone_list_is_email_exist() return true messenger = %s", messenger);
					continue;
				}
			}
			err = contacts_record_clone(record_messenger, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				break;
			}
			list_messenger_record = eina_list_append(list_messenger_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_messenger_record;
}

static bool __phone_list_is_website_exist(Eina_List *list, const char *website)
{
	Eina_List *l;
	char *website_temp;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, l, record_temp) {
		if (NULL == record_temp)
			continue;

		err = contacts_record_get_str_p(record_temp, _contacts_url.url, &website_temp);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_str_p() Failed(%d)", err);

		if (NULL == website_temp)
			continue;

		if (0 == strcmp(website_temp, website)) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_website_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index=0;
	char *website;
	Eina_List *l;
	Eina_List *list_website_record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_url = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.url, index++, &record_url)) {
			err = contacts_record_get_str_p(record_url, _contacts_url.url, &website);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			if (checkduplicate && __phone_list_is_website_exist(list_website_record, website)) {
				ERR("__phone_list_is_email_exist() return true URL = %s", website);
				continue;
			}
			err = contacts_record_clone(record_url, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				break;
			}
			list_website_record = eina_list_append(list_website_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_website_record;
}

static bool __phone_list_is_relationship_exist(Eina_List *list, const char *relationship_name)
{
	Eina_List *cursor;
	char *relationship_name_temp;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, cursor, record_temp) {
		if (NULL == record_temp)
			continue;

		err = contacts_record_get_str_p(record_temp, _contacts_relationship.name, &relationship_name_temp);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_str_p() Failed(%d)", err);

		if (NULL == relationship_name_temp)
			continue;

		if (0 == strcmp(relationship_name_temp, relationship_name)) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_relationship_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index = 0;
	char *relationship_name;
	Eina_List *l;
	Eina_List *list_relationship_record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_relationship = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.relationship, index++, &record_relationship)) {
			err = contacts_record_get_str_p(record_relationship, _contacts_relationship.name, &relationship_name);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			if (checkduplicate && __phone_list_is_relationship_exist(list_relationship_record, relationship_name)) {
				ERR("__phone_list_is_relationship_exist() return true URL = %s", relationship_name);
				continue;
			}
			err = contacts_record_clone(record_relationship, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				break;
			}
			list_relationship_record = eina_list_append(list_relationship_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_relationship_record;
}

static bool __phone_list_is_event_exist(Eina_List *list, int event_date)
{
	Eina_List *l;
	int event_date_temp;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, l, record_temp) {
		if (NULL == record_temp)
			continue;

		err = contacts_record_get_int(record_temp, _contacts_event.date, &event_date_temp);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_int() Failed(%d)", err);

		if (event_date_temp == event_date) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_event_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index = 0;
	int event_date;
	Eina_List *l;
	Eina_List *list_event_record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_event = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.event, index++, &record_event)) {
			err = contacts_record_get_int(record_event, _contacts_event.date, &event_date);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_int_from_list() Failed(%d)", err);
				break;
			}
			if (checkduplicate && __phone_list_is_event_exist(list_event_record, event_date)) {
				ERR("__phone_list_is_email_exist() return true event_date = %d", event_date);
				continue;
			}
			err = contacts_record_clone(record_event, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				break;
			}
			list_event_record = eina_list_append(list_event_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_event_record;
}

static bool __phone_list_is_note_exist(Eina_List *list, const char *note)
{
	Eina_List *l;
	char *note_temp;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_temp = NULL;

	EINA_LIST_FOREACH(list, l, record_temp) {
		if (NULL == record_temp)
			continue;

		err = contacts_record_get_str_p(record_temp, _contacts_note.note, &note_temp);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_str_p() Failed(%d)", err);

		if (NULL == note_temp)
			continue;

		if (0 == strcmp(note_temp, note)) {
			return true;
		}
	}
	return false;
}

Eina_List* ct_detail_person_get_note_list(Eina_List *contact_list)
{
	Eina_Bool checkduplicate = false;
	int index = 0;
	char *note;
	Eina_List *l;
	Eina_List *list_note_record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_note = NULL;
	contacts_record_h record_clone = NULL;

	p_retvm_if(contact_list == NULL, NULL, "Parameter(contact_list) is NULL");

	EINA_LIST_FOREACH(contact_list, l, record_contact) {
		index = 0;
		while (CONTACTS_ERROR_NONE == contacts_record_get_child_record_at_p(record_contact, _contacts_contact.note, index++, &record_note)) {
			err = contacts_record_get_str_p(record_note, _contacts_note.note, &note);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			if (checkduplicate && __phone_list_is_note_exist(list_note_record, note)) {
				ERR("__phone_list_is_email_exist() return true note = %s", note);
				continue;
			}
			err = contacts_record_clone(record_note, &record_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_get_current_record_p() Failed(%d)", err);
				break;
			}
			list_note_record = eina_list_append(list_note_record, record_clone);
		}
		checkduplicate = true;
	}
	return list_note_record;
}

// TODO: Fixme
int ct_detail_get_address_type_index(int type)
{
	switch (type) {
	case CONTACTS_ADDRESS_TYPE_CUSTOM:
		return 1;
	case CONTACTS_ADDRESS_TYPE_HOME:
		return 2;
	case CONTACTS_ADDRESS_TYPE_WORK:
		return 3;
	default:
		return 0;
	}
}

// TODO: Fixme
int ct_detail_get_website_type_index(int type)
{
	switch (type) {
	case CONTACTS_URL_TYPE_CUSTOM:
		return 1;
	case CONTACTS_URL_TYPE_HOME:
		return 2;
	case CONTACTS_URL_TYPE_WORK:
		return 3;
	default:
		return 0;
	}
}

bool ct_detail_person_get_readonly(Eina_List *list_contacts)
{
	PH_FN_CALL;
	int addressbook_id = -1;
	int addressbook_mode = CONTACTS_ADDRESS_BOOK_MODE_READONLY;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Eina_List *l;
	contacts_record_h record_contact = NULL;
	contacts_record_h record_addressbook = NULL;

	p_retvm_if(NULL == list_contacts, true, "list_contacts is NULL");

	EINA_LIST_FOREACH(list_contacts, l, record_contact){
		if (NULL == record_contact)
			continue;

		err = contacts_record_get_int(record_contact, _contacts_contact.address_book_id, &addressbook_id);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_db_get_record(_contacts_address_book._uri, addressbook_id, &record_addressbook);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_db_get_record() Failed(%d)", err);

		err = contacts_record_get_int(record_addressbook, _contacts_address_book.mode, &addressbook_mode);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_record_destroy(record_addressbook, true);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_record_destroy() Failed(%d)", err);

		if (CONTACTS_ADDRESS_BOOK_MODE_READONLY != addressbook_mode)
			return false;
	}
	return true;
}

UDateFormat* ct_detail_init_time_formatter(void)
{
	const char *locale;
	int32_t pattern_capacity;
	UChar pattern[PH_TEXT_SHORT_LEN];
	UChar customSkeleton[PH_TEXT_SHORT_LEN];
	UDateFormat *formatter = NULL;
	UDateTimePatternGenerator generator;
	UErrorCode status = U_ZERO_ERROR;

	u_uastrncpy(customSkeleton, UDAT_YEAR_MONTH_DAY, strlen(UDAT_YEAR_MONTH_DAY));

	locale = uloc_getDefault();
	p_retvm_if(NULL == locale, NULL, "uloc_getDefault() return NULL");

	generator = udatpg_open(locale, &status);
	p_retvm_if(U_FAILURE(status), NULL, "udatpg_open() Failed()");

	pattern_capacity = (int32_t) (sizeof(pattern) / sizeof((pattern)[0]));
	udatpg_getBestPattern(generator, customSkeleton, 6, pattern, pattern_capacity, &status);

	if (!U_FAILURE(status))
		formatter = udat_open(UDAT_IGNORE, UDAT_IGNORE, locale, NULL, -1, pattern, -1, &status);

	udatpg_close(generator);
	return formatter;
}

char* ct_detail_get_datetime_str(UDateFormat* formatter, int year, int mon, int day)
{
	time_t timer;
	struct tm stime;
	char *ret_str = NULL;
	char formatted_str[PH_TEXT_SHORT_LEN];
	int32_t formattedCapacity;
	UChar formatted[PH_TEXT_SHORT_LEN];
	UErrorCode status = U_ZERO_ERROR;

	memset(&stime, 0, sizeof(struct tm));
	stime.tm_year = year-1900;
	stime.tm_mon = mon-1;
	stime.tm_mday = day;
	timer = mktime(&stime);

	formattedCapacity = (int32_t)(sizeof(formatted)/sizeof((formatted)[0]));
	udat_format(formatter, (UDate)timer*1000, formatted, formattedCapacity, NULL, &status);
	p_retvm_if(U_FAILURE(status), NULL, "udat_format() Failed()");

	u_austrcpy(formatted_str, formatted);
	ret_str = g_strdup(formatted_str);

	return ret_str;
}

