/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#ifndef __PHONE_DIALER_H__
#define __PHONE_DIALER_H__

#include "phone-common.h"

#define MAX_DIAL_NUMBER_FONT_SIZE		100
#define MIN_DIAL_NUMBER_FONT_SIZE		67

#define CURSOR_BEGIN    -1
#define CURSOR_END      -2

#define PH_TAG_TEXT_MAX_LEN		(PH_TEXT_MAX_LEN*2)


enum PH_DAILER_TYPES {
	PH_DAILER_NORMAL = 0,
	PH_DAILER_DURING_CALL,
	PH_DAILER_EMERGENCY
};

enum {
	CONTACT_DATA_CHAGNED = 1 << 0,
	LOG_DATA_CHAGNED = 1 << 1,
	SPEEDDIAL_DATA_CHAGNED = 1 << 2,
};

typedef enum {
	MODE_DIAL_NUMBER = 0,
	MODE_COUNTRY_CODE
} ListMode;

typedef enum {
	DASH_REAL = 1,
	DASH_PREVIEW = DASH_REAL
} DashMode;

typedef struct _InputNumber {
	char num;

	struct _InputNumber *prev;
	struct _InputNumber *next;
} InputNumber;

typedef struct {
	bool alive;
	bool success;
	bool completed;
	pthread_t thread;
	Ecore_Pipe *epipe;
} ph_thread_info;

typedef struct {
	Evas_Object *win;
	Evas_Object *layout;
	Evas_Object *entry;
	Evas_Object *navi;
	Evas_Object *genlist;
	Evas_Object *fast_index;
	Evas_Object *count_button;
	Evas_Object *count_layout;
	Evas_Object *popup;
	Ecore_Timer *search_timer;
	Ecore_Timer *dash_timer;
	Ecore_Timer *longpress_timer;
	Ecore_Timer *update_timer;
	Ecore_Timer *slide_timer;
	Ecore_Job *job;
	Ecore_Idler *search_idler;
	Ecore_Event_Handler *key_press;
	Ecore_Event_Handler *key_release;
	char *longpress_source;
	char *searched_text;
	char* selected_text;
	InputNumber number_head;
	InputNumber *number_cursor;
	int cur_mcc;
	int cur_sim_mcc;
	int cur_cc_index;
	int roaming;
	int entry_font_size;
	int count_button_status;
	int run_mode;
	int compare_N;
	int show_N;
	int searched_speed_id;
	int data_dirty_flag;
	int touch_x;
	int touch_y;
	ListMode list_mode;
	bool is_background;
	bool is_emergency;
	bool cursor_changed;
	bool is_show_items;
	bool has_focus;
	bool initialized;
	bool searching;
	bool loading;
	bool slide;
	ph_thread_info *t_info;	// load contact
	Eina_List *search_list;
	Eina_List *contact_list;
	Eina_List *speeddial_list;
	Eina_List *log_list;
	Eina_List *cc_list;
	Eina_List *cc_list_sorted;
	void *ug;
	void *child_ug;
	Elm_Theme *th;
}ph_dialer_data;

Evas_Object* phone_create_dialer_view(ph_view_info *view_info);
void phone_dialer_set_navi(Evas_Object *layout);
void phone_dialer_check_focus(Evas_Object *base);
void phone_dialer_view_reset(Evas_Object *base);
void phone_dialer_view_language_changed(Evas_Object *layout);
void phone_dialer_view_resume(Evas_Object *base);
void phone_dialer_set_background(Evas_Object *obj, bool is_background);
void phone_dialer_view_set_num(Evas_Object *layout, char *number);
void phone_dialer_view_send_key(Evas_Object *layout);

#endif   /* __PHONE_DIALER_H__ */
