/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <Ecore_X.h>
#include <ui-gadget-module.h>

#include "phone.h"
#include "phone-common.h"

#include "ct-favor.h"
#include "ct-list.h"
#include "ct-detail.h"
#include "ph-favorite-view.h"
#include "ph-favorite-edit-view.h"

#define KEY_FAVOR_DATA	"favor_d"
#define PH_FAVOR_LIST_ICON_SIZE 70

static Elm_Gengrid_Item_Class grid_itc_favor, grid_itc_favor_no_id, grid_itc_mf, grid_itc_mf_no_id, grid_itc_index, grid_itc_mf_index;

enum favor_item_type {
	FAVOR_ITEM_MFC = 0,
	FAVOR_ITEM_FAVOR,
};


static char* favor_get_contact_number(int person_id)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	char *number;

	err = contacts_db_get_record( _contacts_person_number._uri, person_id, &record);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record(%d) Failed(%d)", person_id, err);

	err = contacts_record_get_str(record, _contacts_person_number.number, &number);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);

	err = contacts_record_destroy(record, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

	return number;
}

static contacts_filter_h favor_create_filter_bool(contacts_filter_h filter, const char *view_uri, int property_id, contacts_match_int_flag_e flag, bool match_value)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;

	if (NULL == filter) {
		err = contacts_filter_create(view_uri, &filter);
		p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_filter_create() Failed(%d)", err);
	}
	else {
		err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_operator() Failed(%d)", err);
	}

	err = contacts_filter_add_bool(filter, property_id, match_value);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_add_int() Failed(%d)", err);

	return filter;
}

static void favor_load_favorite_data(ph_favorite_data *favor_d)
{
	int count = 0;
	int ret = 0;
	Eina_List *list = NULL;
	contacts_list_h person_list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_filter_h filter = NULL;
	contacts_filter_h filter2 = NULL;
	contacts_query_h query = NULL;

	err = contacts_query_create(_contacts_person._uri, &query);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

	switch (favor_d->ug_request) {
		//case PH_UG_REQUEST_SELECT_NUMBER:
		//case PH_UG_REQUEST_CHECK_FOR_NUMBER:
	case CT_UG_REQUEST_SELECT_NUMBER:
	case CT_UG_REQUEST_CHECK_FOR_NUMBER:
		do {
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.is_favorite, CONTACTS_MATCH_EQUAL, true);
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_sort(query, _contacts_person.favorite_priority, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &person_list))) break;
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_query_destroy(query);
		contacts_filter_destroy(filter);

		break;
		//case PH_UG_REQUEST_SELECT_EMAIL:
		//case PH_UG_REQUEST_CHECK_FOR_EMAIL:
	case CT_UG_REQUEST_SELECT_EMAIL:
	case CT_UG_REQUEST_CHECK_FOR_EMAIL:
		do {
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.is_favorite, CONTACTS_MATCH_EQUAL, true);
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.has_email, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_sort(query, _contacts_person.favorite_priority, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &person_list))) break;
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_query_destroy(query);
		contacts_filter_destroy(filter);

		break;

	case CT_UG_REQUEST_SELECT_NUMBER_OR_EMAIL:
	case CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL:
		do {
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.is_favorite, CONTACTS_MATCH_EQUAL, true);
			filter2 = favor_create_filter_bool(NULL, _contacts_person._uri, _contacts_person.has_phonenumber, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter2, CONTACTS_FILTER_OPERATOR_OR))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_bool(filter2, _contacts_person.has_email, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_filter(filter, filter2))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_sort(query, _contacts_person.favorite_priority, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &person_list))) break;
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_query_destroy(query);
		contacts_filter_destroy(filter);
		contacts_filter_destroy(filter2);

		break;
	default :
		do {
			filter = favor_create_filter_bool(filter, _contacts_person._uri, _contacts_person.is_favorite, CONTACTS_MATCH_EQUAL, true);
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_query_set_sort(query, _contacts_person.favorite_priority, true))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &person_list))) break;
		} while (0);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);

		contacts_query_destroy(query);
		contacts_filter_destroy(filter);

		break;
	}
	if (CONTACTS_ERROR_NONE != ret) {
		ERR("contacts_db_get_records_with_query() Failed(%d)");
		favor_d->favor_list_count = 0;
		return;
	}

	contacts_db_get_current_version(&favor_d->version);

	while (CONTACTS_ERROR_NONE == err) {
		ph_favor_info *f_info;
		f_info = calloc(1, sizeof(ph_favor_info));

		err = phone_cts_get_int_from_list(person_list, _contacts_person.id, &f_info->person_id);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_int_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		err = phone_cts_get_str_from_list(person_list, _contacts_person.display_name, &f_info->display);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_str_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		SAFE_STRDUP(f_info->display);
		err = phone_cts_get_str_from_list(person_list, _contacts_person.image_thumbnail_path, &f_info->img_path);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_str_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		SAFE_STRDUP(f_info->img_path);

		if (CT_UG_REQUEST_SELECT_NUMBER == favor_d->ug_request ||CT_UG_REQUEST_CHECK_FOR_NUMBER == favor_d->ug_request) {
			f_info->contact_list = ctui_person_get_contact_list(f_info->person_id);
			p_warn_if(NULL == f_info->contact_list, "ctui_person_get_contact_list() return NULL");

			f_info->number_list= ctui_person_get_number_list(f_info->contact_list);
			p_warn_if(NULL == f_info->number_list, "ctui_person_get_contact_list() return NULL");
		}
		else if (CT_UG_REQUEST_SELECT_EMAIL == favor_d->ug_request || CT_UG_REQUEST_CHECK_FOR_EMAIL == favor_d->ug_request) {
			f_info->contact_list = ctui_person_get_contact_list(f_info->person_id);
			p_warn_if(NULL == f_info->contact_list, "ctui_person_get_contact_list() return NULL");

			f_info->email_list= ctui_person_get_email_list(f_info->contact_list);
			p_warn_if(NULL == f_info->email_list, "ctui_person_get_contact_list() return NULL");

		}
		else if (CT_UG_REQUEST_SELECT_NUMBER_OR_EMAIL == favor_d->ug_request || CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == favor_d->ug_request) {
			f_info->contact_list = ctui_person_get_contact_list(f_info->person_id);
			p_warn_if(NULL == f_info->contact_list, "ctui_person_get_contact_list() return NULL");

			f_info->email_list= ctui_person_get_email_list(f_info->contact_list);
			p_warn_if(NULL == f_info->email_list, "ctui_person_get_contact_list() return NULL");

			f_info->number_list= ctui_person_get_number_list(f_info->contact_list);
			p_warn_if(NULL == f_info->number_list, "ctui_person_get_contact_list() return NULL");
		}

		list = eina_list_append(list, (void*)f_info);
		count++;
		err = contacts_list_next(person_list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}
	err = contacts_list_destroy(person_list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

	favor_d->favor_list_count = count;
	favor_d->favor_list = list;
}

static void favor_free_data(Eina_List *list)
{
	Eina_List *l;
	ph_favor_info *f_info;
	p_retm_if(NULL == list, "List is null");

	EINA_LIST_FOREACH(list, l, f_info) {
		if (NULL == f_info)
			continue;
		free(f_info->display);
		free(f_info->img_path);
		free(f_info->number);
		if (f_info->contact_list)
			ctui_free_record_list(f_info->contact_list);

		if (f_info->number_list)
			ctui_free_record_list(f_info->number_list);

		if (f_info->email_list)
			ctui_free_record_list(f_info->email_list);
		free(f_info);
	}
	eina_list_free(list);
}

static void favor_destroy_child_ug(void *data)
{
	ph_favorite_data *favor_d = data;
	favor_d->child_ug = NULL;
}

static void favor_ctxpopup_dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon;
	Evas_Object *btn;
	ph_favorite_data *favor_d = data;
	btn = evas_object_data_get(obj, "expand_btn");
	icon = elm_object_part_content_get(btn, "icon");
	elm_icon_file_set(icon, CTUI_IMG_ICON_EXPAND_OPENED, NULL);
	elm_object_part_content_set(btn, "icon", icon);
	evas_object_del(favor_d->ctx_popup);
	favor_d->ctx_popup = NULL;
}

static void favor_ctxpopup_hide_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon;
	Evas_Object *btn;
	btn = (Evas_Object *)data;
	icon = elm_object_part_content_get(btn, "icon");
	elm_icon_file_set(icon, CTUI_IMG_ICON_EXPAND_OPENED, NULL);
	elm_object_part_content_set(btn, "icon", icon);
}

static void favor_msg_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favor_info *f_info;
	ph_favorite_data *favor_d = data;

	f_info = evas_object_data_get(obj, "f_info");
	p_retm_if(NULL == f_info, "evas_object_data_get() return NULL");

	favor_ctxpopup_dismissed_cb(favor_d, favor_d->ctx_popup, NULL);

	favor_d->child_ug = phone_launch_msg_composer_ug(favor_d->ug, f_info->number, 0,
			favor_destroy_child_ug, favor_d);
}

static void favor_call_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favor_info *f_info;
	ph_favorite_data *favor_d = data;

	f_info = evas_object_data_get(obj, "f_info");
	p_retm_if(NULL == f_info, "evas_object_data_get() return NULL");

	favor_ctxpopup_dismissed_cb(favor_d, favor_d->ctx_popup, NULL);
	phone_launch_voice_call(f_info->person_id, f_info->number);
}

void ph_favor_load_favor_list(ph_favorite_data *favor_d)
{
	p_retm_if(NULL == favor_d, "Parameter is null");

	favor_free_data(favor_d->favor_list);
	favor_d->favor_list = NULL;
	favor_load_favorite_data(favor_d);
}

static void favor_img_crop_result(ui_gadget_h ug, service_h service, void *data)
{
	PH_FN_CALL;
	Eina_List *list_writable_contact;
	ph_favorite_data *favor_d = data;

	p_retm_if(NULL == ug || NULL == data, "ug or data is NULL");

	if (service) {
		char *normal = NULL;
		char *full = NULL;
		Eina_List *l;
		contacts_record_h record = NULL;
		contacts_record_h record_image = NULL;

		service_get_extra_data(service, "crop_image_path", &normal);
		p_retm_if(NULL == normal, "return value is NULL.");
		service_get_extra_data(service, "image_path", &full);
		contacts_error_e err = CONTACTS_ERROR_NONE;

		list_writable_contact = NULL;
		list_writable_contact = ctui_person_get_writable_contact_list(favor_d->selected_person_id);

		if (list_writable_contact) {
			EINA_LIST_FOREACH(list_writable_contact, l, data) {
				record = (contacts_record_h)data;
				if (NULL == record) continue;

				record_image = ctui_get_record_image(record);
				if (NULL == record_image) {
					ERR("ctui_get_record_image() Failed(%d)", err);
					break;
				}

				err = contacts_record_set_str(record_image, _contacts_image.path, normal);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

				err = contacts_db_update_record(record);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
				ctui_create_contacts_error_popup(favor_d->navi, err);
			}
			ctui_free_record_list(list_writable_contact);
		}

		free(normal);
		free(full);
	}
}

static void favor_popup_response_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	if (PH_UG_REQUEST_SET_WITH_RINGTONE == favor_d->ug_request)
		ug_destroy_me(favor_d->ug);
}

static void favor_value_list_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d;
	Elm_Object_Item *item;
	ph_favor_info *f_info;

	favor_d = evas_object_data_get(data, "favor_d");
	p_retm_if(NULL == favor_d, "evas_object_data_get() return NULL");

	evas_object_del(favor_d->popup);
	favor_d->popup = NULL;

	if (NULL!= obj) {
		item = evas_object_data_get(data, "item");
		p_retm_if(NULL == item, "evas_object_data_get() return NULL");

		f_info = elm_object_item_data_get(item);
		p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

		f_info->checked = EINA_FALSE;
		favor_d->checked_cnt--;
		elm_genlist_item_update(item);
	}
}

static void favor_del_response_cb(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ph_favor_info *f_info = data;
	ph_favorite_data *favor_d;
	contacts_record_h record = NULL;

	favor_d = evas_object_data_get(obj, KEY_FAVOR_DATA);

	evas_object_del(favor_d->popup);
	favor_d->popup = NULL;

	err = contacts_db_get_record(_contacts_person._uri, f_info->person_id, &record);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed");

	err = contacts_record_set_bool(record, _contacts_person.is_favorite, false);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_bool() Failed(%d)", err);

	err = contacts_db_update_record(record);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);

	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_record_set_bool() Failed(%d)", err);
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
	}
	else
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
}

static void favor_mf_del_response_cb(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ph_favor_info *f_info = data;
	ph_favorite_data *favor_d;

	favor_d = evas_object_data_get(obj, KEY_FAVOR_DATA);

	evas_object_del(favor_d->popup);
	favor_d->popup = NULL;

	err = contacts_person_reset_usage(f_info->person_id, CONTACTS_USAGE_STAT_TYPE_OUTGOING_CALL);

	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_person_reset_usage() Failed(%d)", err);
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
	}
	else
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
	ph_favor_data_lazy_update(NULL, favor_d);
}

static void favor_delete_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	ph_favor_info *f_info;
	Evas_Object *btn;
	char buf[PH_TEXT_MAX_LEN] = {0};

	f_info = evas_object_data_get(obj, "f_info");
	p_retm_if(NULL == f_info, "evas_object_data_get() return NULL");

	favor_ctxpopup_dismissed_cb(favor_d, favor_d->ctx_popup, NULL);

	snprintf(buf, sizeof(buf), "%s?", S_(PH_SYS_SK_DELETE));
	btn = phone_create_confirm_popup(&favor_d->popup, favor_d->win, buf, favor_del_response_cb, f_info);
	evas_object_data_set(btn, KEY_FAVOR_DATA, favor_d);
}

static void favor_mf_delete_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	ph_favor_info *f_info;
	Evas_Object *btn;
	char buf[PH_TEXT_MAX_LEN] = {0};

	f_info = evas_object_data_get(obj, "f_info");
	p_retm_if(NULL == f_info, "evas_object_data_get() return NULL");

	favor_ctxpopup_dismissed_cb(favor_d, favor_d->ctx_popup, NULL);

	snprintf(buf, sizeof(buf), "%s?", S_(PH_SYS_SK_DELETE));
	btn = phone_create_confirm_popup(&favor_d->popup, favor_d->win, buf, favor_mf_del_response_cb, f_info);
	evas_object_data_set(btn, KEY_FAVOR_DATA, favor_d);
}

static Evas_Object *favor_create_button(Evas_Object *box, char *fullpath, ph_favor_info *f_info)
{
	const int PH_FAVOR_BTN_WIDTH = 160;
	const int PH_FAVOR_BTN_HEIGHT = 74;
	const int PH_FAVOR_1BTN_WIDTH = 326;
	Evas_Object *btn;
	double scale;
	scale = elm_config_scale_get();
	btn = phone_create_list_sweep_btn(box, fullpath);
	p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn is failed");

	if(strcmp(fullpath, PH_IMG_ICON_DELETE) == 0)
		elm_object_style_set(btn, "sweep/delete");
	evas_object_data_set(btn, "f_info", f_info);
	if (f_info->number) {
		evas_object_size_hint_min_set(btn, (int)(PH_FAVOR_BTN_WIDTH*scale), (int)(PH_FAVOR_BTN_HEIGHT*scale));
		evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, 0.5);
	}
	else {
		evas_object_size_hint_min_set(btn, (int)(PH_FAVOR_1BTN_WIDTH*scale), (int)(PH_FAVOR_BTN_HEIGHT*scale));
		evas_object_size_hint_align_set(btn, 0.5, 0.5);
	}
	evas_object_show(btn);
	elm_box_pack_end(box, btn);
	return btn;
}

static void __favor_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	if (favor_d->popup) {
		evas_object_del(favor_d->popup);
		favor_d->popup = NULL;
	}

}



static void favor_call_btn_pressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon = NULL;
	p_retm_if(NULL == data, "data is NULL");
	icon = elm_object_part_content_get(data, "icon");
	elm_icon_file_set(icon, PH_IMG_ICON_CALL_PRESS, NULL);
}

static void favor_call_btn_unpressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon = NULL;
	p_retm_if(NULL == data, "data is NULL");
	icon = elm_object_part_content_get(data, "icon");
	elm_icon_file_set(icon, PH_IMG_ICON_CALL, NULL);
}

static void favor_msg_btn_pressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon = NULL;
	p_retm_if(NULL == data, "data is NULL");
	icon = elm_object_part_content_get(data, "icon");
	elm_icon_file_set(icon, PH_IMG_ICON_MESSAGE_PRESS, NULL);
}

static void favor_msg_btn_unpressed_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Object *icon = NULL;
	p_retm_if(NULL == data, "data is NULL");
	icon = elm_object_part_content_get(data, "icon");
	elm_icon_file_set(icon, PH_IMG_ICON_MESSAGE, NULL);
}

static Evas_Object *favor_create_popup_button(Evas_Object *parent, ph_favorite_data *favor_d, ph_favor_info *f_info, int item_type)
{
	Evas_Object *btn1, *btn2, *btn3, *btn4;
	Evas_Object *box;

	box = elm_box_add(parent);
	elm_box_padding_set(box, 16, 0);

	if (f_info->number) {
		elm_box_horizontal_set(box, EINA_TRUE);
		btn1 = favor_create_button(box, PH_IMG_ICON_CALL, f_info);
		evas_object_smart_callback_add(btn1, "clicked", favor_call_btn_cb, favor_d);
		evas_object_event_callback_add(btn1, EVAS_CALLBACK_MOUSE_DOWN, favor_call_btn_pressed_cb, btn1);
		evas_object_event_callback_add(btn1, EVAS_CALLBACK_MOUSE_UP, favor_call_btn_unpressed_cb, btn1);

		btn2 = favor_create_button(box, PH_IMG_ICON_MESSAGE, f_info);
		evas_object_smart_callback_add(btn2, "clicked", favor_msg_btn_cb, favor_d);
		evas_object_event_callback_add(btn2, EVAS_CALLBACK_MOUSE_DOWN, favor_msg_btn_pressed_cb, btn2);
		evas_object_event_callback_add(btn2, EVAS_CALLBACK_MOUSE_UP, favor_msg_btn_unpressed_cb, btn2);

		btn4 = favor_create_button(box, PH_IMG_ICON_DELETE, f_info);
		if (FAVOR_ITEM_MFC == item_type)
			evas_object_smart_callback_add(btn4, "clicked", favor_mf_delete_btn_cb, favor_d);
		else if (FAVOR_ITEM_FAVOR == item_type)
			evas_object_smart_callback_add(btn4, "clicked", favor_delete_btn_cb, favor_d);

	}
	else {
		btn4 = favor_create_button(box, PH_IMG_ICON_DELETE, f_info);

		if (FAVOR_ITEM_MFC == item_type)
			evas_object_smart_callback_add(btn4, "clicked", favor_mf_delete_btn_cb, favor_d);
		else if (FAVOR_ITEM_FAVOR == item_type)
			evas_object_smart_callback_add(btn4, "clicked", favor_delete_btn_cb, favor_d);

	}

	return box;
}

static void favor_expand_btn_clicked_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	const char *file_icon;
	const char *file_group;
	const int PH_CTX_POPUP_HEIGHT = 112;
	Evas_Object *ctx_popup;
	Evas_Object *box;
	Evas_Object *icon;
	Evas_Object *layout;
	ph_favor_info *f_info;
	ph_favorite_data *favor_d = data;
	Ecore_X_Window win;
	int win_width;
	int item_type;
	double scale;

	icon = elm_object_part_content_get(obj, "icon");

	elm_icon_file_get(icon, &file_icon, &file_group);
	p_retm_if(NULL == file_icon, "elm_icon_file_set() return NULL");

	if (0 == strcmp(file_icon, CTUI_IMG_ICON_EXPAND_CLOSED)) { // OPENED
		elm_icon_file_set(icon, CTUI_IMG_ICON_EXPAND_OPENED, NULL);
		elm_object_part_content_set(obj, "icon", icon);
	}
	else {
		win = ecore_x_window_root_get(ecore_x_window_focus_get());
		ecore_x_window_size_get(win, &win_width, NULL);

		scale = elm_config_scale_get();
		elm_icon_file_set(icon, CTUI_IMG_ICON_EXPAND_CLOSED, NULL);
		elm_object_part_content_set(obj, "icon", icon);

		if (favor_d->ctx_popup) {
			evas_object_del(favor_d->ctx_popup);
			favor_d->ctx_popup = NULL;
		}

		ctx_popup = elm_ctxpopup_add(favor_d->navi);
		evas_object_size_hint_min_set(ctx_popup, win_width, (int)(PH_CTX_POPUP_HEIGHT*scale));
		elm_ctxpopup_horizontal_set(ctx_popup, EINA_TRUE);
		evas_object_data_set(ctx_popup, "expand_btn", obj);
		favor_d->ctx_popup = ctx_popup;

		elm_ctxpopup_direction_priority_set(ctx_popup, ELM_CTXPOPUP_DIRECTION_DOWN, ELM_CTXPOPUP_DIRECTION_UP,
				ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_RIGHT);

		evas_object_smart_callback_add(ctx_popup, "dismissed", favor_ctxpopup_dismissed_cb, favor_d);
		evas_object_event_callback_add(ctx_popup, EVAS_CALLBACK_HIDE, favor_ctxpopup_hide_cb, obj);

		f_info = evas_object_data_get(obj, "f_info");
		p_retm_if(NULL == f_info, "evas_object_data_get() return NULL");
		item_type = (int)evas_object_data_get(obj, "item_type");

		f_info->number = SAFE_STRDUP(favor_get_contact_number(f_info->person_id));
		layout = elm_object_part_content_get(favor_d->layout, "contents");

		box = favor_create_popup_button(layout, favor_d, f_info, item_type);
		evas_object_show(box);
		elm_object_content_set(ctx_popup, box);
		phone_move_ctxpopup(favor_d->ctx_popup, obj);
		evas_object_show(ctx_popup);
	}
}


static void favor_nums_popup_sel(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d = evas_object_data_get(obj, "favor_d");
	int contact_id = (int)evas_object_data_get(obj, "person_id");

	if (PH_UG_REQUEST_CHECK_FOR_NUMBER == favor_d->ug_request || PH_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == favor_d->ug_request) {
		Elm_Object_Item *item;
		ph_favor_info *f_info;

		item = evas_object_data_get(obj, "item");
		p_retm_if(NULL == item, "evas_object_data_get() return NULL");

		f_info = elm_object_item_data_get(item);
		p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

		f_info->id_ret = (int)data;
		f_info->id_ret_type = 1;
		ph_favor_edit_set_selection_info(favor_d);
		favor_value_list_popup_hide_cb(obj, NULL, NULL);
	}
	else if (PH_UG_REQUEST_LAUNCH_VOICECALL == favor_d->ug_request) {
		char *num_info = (char *)elm_object_item_text_get(event_info);
		char *number;
		strtok(num_info, " ");
		number = strtok(NULL, " ");
		favor_value_list_popup_hide_cb(obj, NULL, NULL);
		phone_launch_voice_call(contact_id, number);
	}
	else {
		phone_ug_return_id(favor_d->ug, contact_id, (int)data, 0);
		ug_destroy_me(favor_d->ug);
	}
}

static inline void favor_handle_select_email(ph_favorite_data *favor_d, ph_favor_info *f_info)
{
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	int email_id;

	if (f_info->email_list) {
			if (1 == eina_list_count(f_info->email_list)) {
				record = (contacts_record_h)f_info->email_list->data;
				err = contacts_record_get_int(record, _contacts_email.id, &email_id);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
				phone_ug_return_id(favor_d->ug, f_info->person_id, 0, email_id);
				ug_destroy_me(favor_d->ug);
			}
		else
			ph_favor_create_value_list_popup(favor_d, f_info);
	}
	else
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);
}

static void favor_emails_popup_sel(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = evas_object_data_get(obj, "favor_d");
	int contact_id = (int)evas_object_data_get(obj, "person_id");

	if (CT_UG_REQUEST_CHECK_FOR_EMAIL == favor_d->ug_request || PH_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == favor_d->ug_request) {
		Elm_Object_Item *item;
		ph_favor_info *f_info;

		item = evas_object_data_get(obj, "item");
		p_retm_if(NULL == item, "evas_object_data_get() return NULL");

		f_info = elm_object_item_data_get(item);
		p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

		f_info->id_ret = (int)data;
		f_info->id_ret_type = 2;
		ph_favor_edit_set_selection_info(favor_d);
		favor_value_list_popup_hide_cb(obj, NULL, NULL);
	}
	else {
		phone_ug_return_id(favor_d->ug, contact_id, 0, (int)data);
		ug_destroy_me(favor_d->ug);
	}
}

static inline void favor_handle_select_number(ph_favorite_data *favor_d, ph_favor_info *f_info)
{
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	char *number_str_p = NULL;
	int num_id;

	if (f_info->number_list) {
		if (1 == eina_list_count(f_info->number_list)) {
			record = (contacts_record_h)f_info->number_list->data;
			if (PH_UG_REQUEST_LAUNCH_VOICECALL == favor_d->ug_request) {
				err = contacts_record_get_str_p(record, _contacts_number.number, &number_str_p);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
				phone_launch_voice_call(f_info->person_id, number_str_p);
			}
			else {
				err = contacts_record_get_int(record, _contacts_number.id, &num_id);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
				phone_ug_return_id(favor_d->ug, f_info->person_id, num_id, 0);
				ug_destroy_me(favor_d->ug);
			}
		}
		else
			ph_favor_create_value_list_popup(favor_d, f_info);
	}
	else
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_NUMBER), 1.5);
}

static inline void favor_handle_select_number_or_email(ph_favorite_data *favor_d, ph_favor_info *f_info)
{
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	int num_id;
	int email_id;

	int number_list_count = 0;
	int email_list_count = 0;

	if (f_info->number_list)
		number_list_count = eina_list_count(f_info->number_list);
	if (f_info->email_list)
		email_list_count = eina_list_count(f_info->email_list);

	if (number_list_count == 0 && email_list_count == 0) {
		phone_show_popup(favor_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_NUMBER), 1.5);
		ug_destroy_me(favor_d->ug);
	}
	else if (number_list_count == 1 && email_list_count == 0) {
		record = (contacts_record_h)f_info->number_list->data;
		err = contacts_record_get_int(record, _contacts_number.id, &num_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		phone_ug_return_id(favor_d->ug, f_info->person_id, num_id, 0);
		ug_destroy_me(favor_d->ug);
	}
	else if (number_list_count == 0 && email_list_count == 1) {
		record = (contacts_record_h)f_info->email_list->data;
		err = contacts_record_get_int(record, _contacts_email.id, &email_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		phone_ug_return_id(favor_d->ug, f_info->person_id, 0, email_id);
		ug_destroy_me(favor_d->ug);
	}
	else {
		ph_favor_create_value_list_popup(favor_d, f_info);
	}
}

Evas_Object* ph_favor_create_value_list_popup(ph_favorite_data *favor_d, ph_favor_info *f_info)
{
	Evas_Object *popup;
	Evas_Object *list = NULL;
	Evas_Object *btn;

	popup = elm_popup_add(favor_d->navi);
	elm_object_style_set(popup, "min_liststyle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (f_info->number_list)
		list = ctui_list_create_values_list(_contacts_number._uri, popup, list, f_info->number_list, favor_nums_popup_sel);

	if (f_info->email_list)
		list = ctui_list_create_values_list(_contacts_email._uri, popup, list, f_info->email_list, favor_emails_popup_sel);

	elm_object_content_set(popup, list);
	evas_object_data_set(list, "favor_d", favor_d);
	evas_object_data_set(list, "person_id", (void *)f_info->person_id);

	btn = elm_button_add(popup);
	elm_object_text_set(btn, S_(PH_SYS_POP_CLOSE));
	elm_object_part_content_set(popup, "button1", btn);
	elm_object_style_set(btn, "popup_button/default");
	evas_object_smart_callback_add(btn, "clicked", favor_value_list_popup_hide_cb, list);

	favor_d->popup = popup;
	evas_object_show(popup);
	return list;
}

static void favor_destory_child_ug(ui_gadget_h ug, void *priv)
{
	ph_favorite_data *favor_d = priv;
	p_retm_if(NULL == ug, "ug is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == priv, "priv is NULL");
	favor_d->child_ug = NULL;
	ug_destroy_me(favor_d->ug);

}

static void favor_ug_check(ph_favorite_data *favor_d, ph_favor_info *f_info)
{
	Evas_Object *popup;
	ph_view_info *view_info;
	struct ug_cbs cbs = {0};
	const char *err_msg;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;


	switch (favor_d->ug_request) {
	case PH_UG_REQUEST_SELECT:
		phone_ug_return_id(favor_d->ug, f_info->person_id, 0, 0);
		ug_destroy_me(favor_d->ug);
		break;
	case PH_UG_REQUEST_SELECT_EMAIL:
		favor_handle_select_email(favor_d, f_info);
		break;
	case PH_UG_REQUEST_SELECT_NUMBER:
	case PH_UG_REQUEST_LAUNCH_VOICECALL:
		//FIXME : send selection info to caller
		favor_handle_select_number(favor_d, f_info);
		break;
	case PH_UG_REQUEST_SELECT_NUMBER_OR_EMAIL:
		favor_handle_select_number_or_email(favor_d, f_info);
		break;
	case PH_UG_REQUEST_SET_WITH_NUM:
	case PH_UG_REQUEST_SET_WITH_EMAIL:
	case PH_UG_REQUEST_SET_WITH_WEB:
		view_info = calloc(1, sizeof(ph_view_info));
		phone_set_view_info(view_info, favor_d->win, favor_d->navi, favor_d->ug, favor_d->ug_request, favor_d->ug_cnt_max, favor_d->ug_data);
		ctui_list_update_with_data(view_info, favor_d, f_info->person_id, -1, -1);
		free(view_info);
		break;
	case PH_UG_REQUEST_SET_WITH_IMG:
		favor_d->selected_person_id = f_info->person_id;

		cbs.result_cb = favor_img_crop_result;
		cbs.destroy_cb = favor_destory_child_ug;
		cbs.priv = favor_d;
		favor_d->child_ug = phone_image_crop_ug(favor_d->ug, favor_d->ug_data, &cbs);
		break;
	case PH_UG_REQUEST_SET_WITH_RINGTONE:
		err = contacts_db_get_record(_contacts_person._uri, f_info->person_id, &record);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed");

		err = contacts_record_set_str(record, _contacts_person.ringtone_path, favor_d->ug_data);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed");

		err = contacts_db_update_record(record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);

		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_db_update_record() Failed(%d)", err);
			ctui_create_contacts_error_popup(favor_d->navi, err);
		}
		else {
			err_msg = S_(CT_SYS_POP_SUCCESS);
			popup = phone_show_popup(favor_d->navi, err_msg, 1.5);
			evas_object_smart_callback_add(popup, "timeout", favor_popup_response_cb, favor_d);
		}
		break;
	default:
		ctui_create_detail_view_from_phone(favor_d->win, favor_d->navi, favor_d->ug, f_info->person_id);
		break;
	}

}

static void favor_item_sel(void *data, Evas_Object *obj, void *event_info)
{
	ph_favor_info *f_info;
	ph_favorite_data *favor_d = data;
	Elm_Object_Item *item = event_info;

	elm_gengrid_item_selected_set(item, EINA_FALSE);

	f_info = elm_object_item_data_get(item);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");

	if (PH_FAVOR_MODE_EDIT != favor_d->list_mode)
		favor_ug_check(favor_d, f_info);
}

#define N 20

static Eina_Bool favor_append_item_part(void *data)
{
	int count;
	Eina_List *l;
	ph_favor_info *fd;
	ph_favorite_data *favor_d = data;

	count = 0;
	l = eina_list_nth_list(favor_d->favor_list, favor_d->readed_n);
	EINA_LIST_FOREACH(l, l, fd) {
		if (N == count) break;
		if (!fd) continue;
		if (fd->img_path)
			fd->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_favor, fd, favor_item_sel, favor_d);
		else
			fd->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_favor_no_id, fd, favor_item_sel, favor_d);
		count ++;
	}
	if (N <= count) {
		favor_d->readed_n += N;
		return ECORE_CALLBACK_RENEW;
	}
	else {
		favor_d->list_idler = NULL;
		return ECORE_CALLBACK_CANCEL;
	}
}

void ph_favor_fill_favor_gengrid(ph_favorite_data *favor_d)
{
	PH_FN_CALL;
	int count;
	Eina_List *l;
	ph_favor_info *f_info;
	p_retm_if(NULL == favor_d->favor_gengrid, "Parameter is null");

	if (favor_d->list_idler) {
		ecore_idler_del(favor_d->list_idler);
		favor_d->list_idler = NULL;
	}

	if (favor_d->favor_gengrid)
		elm_gengrid_clear(favor_d->favor_gengrid);

	favor_d->readed_n = 0;
	count = 0;
	if (favor_d->favor_list && PH_FAVOR_MODE_DEFAULT == favor_d->list_mode)
		elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_index, T_(PH_GET_TEXT_BASIC, PHTEXT_FAVORITES), NULL, NULL);

	EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
		if (N == count) break;
		if (!f_info) continue;
		if (f_info->img_path && *f_info->img_path)
			f_info->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_favor, f_info, favor_item_sel, favor_d);
		else
			f_info->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_favor_no_id, f_info, favor_item_sel, favor_d);

		if (favor_d->list_mode == PH_FAVOR_MODE_EDIT)
			elm_object_item_signal_emit(f_info->item, "favorites_theme/edit_no_effect", "favorites_theme");
		count++;
	}

	if (N == count) {
		favor_d->readed_n = N;
		favor_d->list_idler = ecore_idler_add(favor_append_item_part, favor_d);
	}
}

void ph_favor_fill_mf_gengrid(ph_favorite_data *favor_d)
{
	PH_FN_CALL;
	Eina_List *l;
	ph_favor_info *f_info;
	p_retm_if(NULL == favor_d->favor_gengrid, "Parameter is null");

	if (favor_d->mf_list) {
		elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_mf_index, T_(CT_GET_TEXT_BASIC, CTTEXT_MOST_FREQUENT_CONTACTS), NULL, NULL);

		EINA_LIST_FOREACH(favor_d->mf_list, l, f_info) {
			if (!f_info) continue;
			if (f_info->img_path && *f_info->img_path)
				f_info->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_mf, f_info, favor_item_sel, favor_d);
			else
				f_info->item = elm_gengrid_item_append(favor_d->favor_gengrid, &grid_itc_mf_no_id, f_info, favor_item_sel, favor_d);
		}
	}
}

void ph_favor_update_button_status(ph_favorite_data *favor_d)
{
	p_retm_if(NULL == favor_d || NULL == favor_d->navi_button, "Navi button is null");

	if (favor_d->checked_cnt == 0 && PH_FAVOR_MODE_EDIT == favor_d->list_mode)
		elm_object_disabled_set(favor_d->navi_button, EINA_TRUE);
	else if (favor_d->favor_list_count == 0)
		elm_object_disabled_set(favor_d->navi_button, EINA_TRUE);
	else
		elm_object_disabled_set(favor_d->navi_button, EINA_FALSE);
}

static char *favor_label_get(void *data, Evas_Object *obj, const char *part)
{
	ph_favor_info * f_info = data;
	char buf[PATH_MAX];

	if (0 == strcmp(part, "elm.text")) {
		if (f_info->display && *f_info->display)
			snprintf(buf, sizeof(buf), "%s", f_info->display);
		else {
			if (!f_info->number_loaded) {
				f_info->number = SAFE_STRDUP(favor_get_contact_number(f_info->person_id));
				f_info->number_loaded = true;
			}

			if (!f_info->number || !*f_info->number)
				return strdup(S_(PH_SYS_BODY_NONAME));
			else
				snprintf(buf, sizeof(buf), "%s", f_info->number);
		}
		return strdup(buf);
	}
	return NULL;
}

static char *favor_mf_label_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[PATH_MAX];
	ph_favor_info *f_info = data;

	if (0 == strcmp(part, "elm.text")) {
		if (f_info->display && *f_info->display)
			snprintf(buf, sizeof(buf), "%s", f_info->display);
		else {
			if (!f_info->number_loaded) {
				f_info->number = SAFE_STRDUP(favor_get_contact_number(f_info->person_id));
				f_info->number_loaded = true;
			}

			if (!f_info->number || !*f_info->number)
				return strdup(S_(PH_SYS_BODY_NONAME));
			else
				snprintf(buf, sizeof(buf), "%s", f_info->number);
		}
		return strdup(buf);
	}
	return NULL;
}

static Evas_Object *favor_icon_get(void *data, Evas_Object *obj, const char *part)
{
	ph_favor_info * f_info = data;
	ph_favorite_data *favor_d = evas_object_data_get(obj, KEY_FAVOR_DATA);
	if (0 == strcmp(part, "elm.swallow.icon")) {
		Evas_Object *ic;
		char buf[PATH_MAX]={0};
		if (f_info->img_path && *f_info->img_path) {
			snprintf(buf, sizeof(buf), "%s", f_info->img_path);

			ic = elm_icon_add(obj);
			p_retvm_if(NULL == ic, NULL, "elm_icon_add is failed");

			elm_icon_file_set(ic, buf, NULL);
			evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			return ic;
		}
	}
	else if (0 == strcmp(part, "elm.swallow.expand.icon") && favor_d->list_mode == PH_FAVOR_MODE_DEFAULT) {
		Evas_Object *exp_icon;
		Evas_Object *btn;
		btn = elm_button_add(obj);
		evas_object_propagate_events_set(btn , EINA_FALSE);
		evas_object_data_set(btn, "f_info", f_info);
		evas_object_data_set(btn, "item_type", (void *)FAVOR_ITEM_FAVOR);
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, favor_expand_btn_clicked_cb, favor_d);
		elm_object_style_set(btn, "transparency");
		exp_icon = elm_icon_add(btn);
		elm_icon_file_set(exp_icon, CTUI_IMG_ICON_EXPAND_OPENED, NULL);
		elm_object_part_content_set(btn, "icon", exp_icon);
		evas_object_show(btn);
		return btn;
	}
	else if (0 == strcmp(part, "elm.swallow.end") && favor_d->list_mode == PH_FAVOR_MODE_EDIT) {
		Evas_Object *check;
		check = elm_check_add(obj);
		elm_object_style_set(check, "grid");
		elm_check_state_pointer_set(check, &(f_info->checked));
		evas_object_data_set(check, KEY_FAVOR_DATA, favor_d);
		evas_object_smart_callback_add(check, "changed", ph_favor_edit_check_changed_cb, f_info);
		return check;
	}
	return NULL;
}

static char* favor_group_label_get(void *data, Evas_Object *obj, const char *part)
{
	PH_FN_CALL;
	char* title = data;
	if (title && *title)
		return strdup((const char *)data);

	return NULL;
}

static Evas_Object *favor_mf_icon_get(void *data, Evas_Object *obj, const char *part)
{
	ph_favor_info *f_info = data;
	ph_favorite_data *favor_d = evas_object_data_get(obj, KEY_FAVOR_DATA);
	if (0 == strcmp(part, "elm.swallow.icon")) {
		Evas_Object *ic;
		char buf[PATH_MAX];

		if (f_info->img_path && *f_info->img_path) {
			snprintf(buf, sizeof(buf), "%s", f_info->img_path);
			ic = elm_icon_add(obj);
			elm_icon_file_set(ic, buf, NULL);
			evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			return ic;
		}
	}
	else if (0 == strcmp(part, "elm.swallow.expand.icon")) {
		Evas_Object *exp_icon;
		Evas_Object *btn;
		btn = elm_button_add(obj);
		elm_object_style_set(btn, "transparency");
		exp_icon = elm_icon_add(btn);
		elm_icon_file_set(exp_icon, CTUI_IMG_ICON_EXPAND_OPENED, NULL);
		elm_object_part_content_set(btn, "icon", exp_icon);
		evas_object_propagate_events_set(btn , EINA_FALSE);
		evas_object_data_set(btn, "f_info", f_info);
		evas_object_data_set(btn, "item_type", (void *)FAVOR_ITEM_MFC);
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, favor_expand_btn_clicked_cb, favor_d);
		evas_object_show(btn);
		return btn;
	}

	return NULL;
}

static void favor_grid_longpress(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	favor_d->longpressed = (PH_FAVOR_MODE_EDIT == favor_d->list_mode ? EINA_TRUE : EINA_FALSE);
}

static void favor_grid_move(void *data, Evas_Object *obj, void *event_info)
{
	int ori_favor_id = -1;
	int front_favor_id = -1;
	int back_favor_id = -1;
	ph_favor_info *f_info;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Elm_Object_Item *prev_item;
	Elm_Object_Item *next_item;
	Elm_Object_Item *it = event_info;
	ph_favorite_data *favor_d = (ph_favorite_data *)data;

	favor_d->favor_move = false;
	f_info = elm_object_item_data_get(it);
	p_retm_if(NULL == f_info, "elm_object_item_data_get() return NULL");
	ori_favor_id = f_info->person_id;

	prev_item = elm_gengrid_item_prev_get(it);
	if (prev_item) {
		f_info = elm_object_item_data_get(prev_item);
		if (f_info)
			front_favor_id = f_info->person_id;
	}

	next_item = elm_gengrid_item_next_get(it);
	if (next_item) {
		f_info = elm_object_item_data_get(next_item);
		if (f_info)
			back_favor_id = f_info->person_id;
	}

	p_ret_if(-1 == back_favor_id && -1 == front_favor_id);
	if (-1 == back_favor_id)
		back_favor_id = 0;
	if (-1 == front_favor_id)
		front_favor_id = 0;

	PH_DBG("ori %d front %d back %d ", ori_favor_id, front_favor_id, back_favor_id);

	if (-1 != ori_favor_id && -1 != front_favor_id && -1 != back_favor_id) {
		err = contacts_person_set_favorite_order(ori_favor_id, front_favor_id, back_favor_id);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_person_set_favorite_order() Failed(%d)", err);
		favor_d->favor_move = true;
	}
}

static void favor_edit_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	p_retm_if(NULL == favor_d, "parameter(favor_d) is NULL");

	if (favor_d->popup)
		__favor_popup_hide_cb(favor_d, NULL, NULL);

	phone_favor_edit_mode_start(favor_d);
}

static void favor_done_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	int person_count = 0;
	int number_count = 0;
	int email_count = 0;
	int *result_list_number = NULL;
	int *result_list_email = NULL;
	int *result_list_person = NULL;
	Eina_List *l;
	ph_favor_info *f_info;
	ph_favorite_data *favor_d = data;
	p_retm_if(NULL == favor_d, "parameter(favor_d) is NULL");

	if (PH_UG_REQUEST_CHECK_FOR_NUMBER == favor_d->ug_request) {
		result_list_number = calloc(favor_d->checked_cnt, sizeof(int));
		EINA_LIST_FOREACH(favor_d->mf_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (1 != f_info->id_ret_type) {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
			result_list_number[number_count++] = f_info->id_ret;
		}
		EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (1 != f_info->id_ret_type) {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
			result_list_number[number_count++] = f_info->id_ret;
		}
		phone_list_ug_return_ids(favor_d->ug, result_list_number, number_count, CT_UG_BUNDLE_RESULT_NUMBER_ID_LIST);
		free(result_list_number);
	}
	else if (PH_UG_REQUEST_CHECK_FOR_EMAIL == favor_d->ug_request) {
		result_list_email = calloc(favor_d->checked_cnt, sizeof(int));
		EINA_LIST_FOREACH(favor_d->mf_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (2 != f_info->id_ret_type) {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
			result_list_email[email_count++] = f_info->id_ret;
		}
		EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (2 != f_info->id_ret_type) {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
			result_list_email[email_count++] = f_info->id_ret;
		}
		phone_list_ug_return_ids(favor_d->ug, result_list_email, email_count, CT_UG_BUNDLE_RESULT_EMAIL_ID_LIST);
		free(result_list_email);
	}
	else if (PH_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == favor_d->ug_request) {
		result_list_number = calloc(favor_d->checked_cnt, sizeof(int));
		result_list_email = calloc(favor_d->checked_cnt, sizeof(int));

		EINA_LIST_FOREACH(favor_d->mf_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (1 == f_info->id_ret_type)
				result_list_number[number_count++] = f_info->id_ret;
			else if (2 == f_info->id_ret_type)
				result_list_email[email_count++] = f_info->id_ret;
			else {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
		}

		EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
			if (NULL == f_info || !f_info->checked) continue;
			if (1 == f_info->id_ret_type)
				result_list_number[number_count++] = f_info->id_ret;
			else if (2 == f_info->id_ret_type)
				result_list_email[email_count++] = f_info->id_ret;
			else {
				ERR("Invalid f_info->id_ret_type(%d)", f_info->id_ret_type);
				continue;
			}
		}
		phone_list_ug_return_number_email_ids(favor_d->ug, result_list_number, number_count, result_list_email, email_count);
		free(result_list_number);
		free(result_list_email);
	}
	else {
		result_list_person = calloc(favor_d->checked_cnt, sizeof(int));
		EINA_LIST_FOREACH(favor_d->mf_list, l, f_info) {
			result_list_person[person_count++] = f_info->person_id;
		}
		EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
			result_list_person[person_count++] = f_info->person_id;
		}
		phone_list_ug_return_ids(favor_d->ug, result_list_person, person_count, CT_UG_BUNDLE_RESULT_PERSON_ID_LIST);
		free(result_list_person);
	}

	ug_destroy_me(favor_d->ug);
}

static Eina_List* favor_get_contact(Eina_List *favor_list)
{
	int person_id;
	Eina_List *result = NULL;
	Eina_List *cursor;
	ct_person_list *person_info;
	ph_favor_info *f_info;
	int is_exist = false;
	contacts_list_h person_list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_db_get_all_records(_contacts_person._uri, 0,0, &person_list);
	p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_db_get_all_records...() Failed(%d)", err);


	while (CONTACTS_ERROR_NONE == err) {
		is_exist = false;

		err = phone_cts_get_int_from_list(person_list, _contacts_person.id, &person_id);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_int_from_list() Failed(%d)", err);
			break;
		}

		EINA_LIST_FOREACH(favor_list, cursor, f_info) {
			if (!f_info)
				continue;
			if (f_info->person_id == person_id) {
				is_exist = true;
				break;
			}
		}
		if (false == is_exist) {
			char *addressbook_ids;
			person_info = calloc(1, sizeof(ct_person_list));
			if (NULL == person_info) {
				ERR("calloc() return NULL");
				break;
			}
			person_info->id = person_id;
			err = phone_cts_get_str_from_list(person_list, _contacts_person.display_name, &person_info->display);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			SAFE_STRDUP(person_info->display);
			err = phone_cts_get_int_from_list(person_list, _contacts_person.link_count, &person_info->link_count);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_int_from_list() Failed(%d)", err);
				break;
			}
			err = phone_cts_get_str_from_list(person_list, _contacts_person.display_name_index, &(person_info->display_index));
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_str_from_list() Failed(%d)", err);
				break;
			}
			err = phone_cts_get_str_from_list(person_list, _contacts_person_contact.addressbook_ids, &addressbook_ids);
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);
			person_info->addressbook_id_list = ctui_get_addressbook_id_list(addressbook_ids, " ");

			err = phone_cts_get_str_from_list(person_list, _contacts_person.status, &(person_info->status_str));
			p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);

			result = eina_list_append(result, person_info);
		}
		err = contacts_list_next(person_list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}
	return result;
}

static void favor_add_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	Eina_List *ct_list;
	evas_object_data_set(favor_d->navi, "navi_it_pop", favor_d->navi_item);
	ct_list = favor_get_contact(favor_d->favor_list);
	ctui_create_favorite_list_select_view(favor_d->win, favor_d->navi, ct_list);
}

static int favor_get_mf_list(contacts_list_h *list)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_filter_h filter = NULL;
	contacts_query_h query = NULL;

	err = contacts_filter_create(_contacts_person_usage._uri, &filter);
	p_retvm_if(CONTACTS_ERROR_NONE != err, err, "contactcs_filter_create() Failed(%d)", err);

	do {
		if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_int(filter, _contacts_person_usage.times_used, CONTACTS_MATCH_GREATER_THAN_OR_EQUAL, 1))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_query_create(_contacts_person_usage._uri, &query))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_query_set_sort(query, _contacts_person_usage.times_used, false))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, list))) break;

		contacts_filter_destroy(filter);
		contacts_query_destroy(query);

		return CONTACTS_ERROR_NONE;
	} while (0);

	contacts_filter_destroy(filter);
	contacts_query_destroy(query);

	return err;
}
void ph_favor_load_mf_list(void *data)
{
	PH_FN_CALL;
	int count = 0;
	Eina_List *list = NULL;
	ph_favorite_data *favor_d = data;
	contacts_list_h mf_list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	favor_free_data(favor_d->mf_list);
	favor_d->mf_list = NULL;
	favor_d->mf_list_count = 0;

	err = favor_get_mf_list(&mf_list);
	p_retm_if(CONTACTS_ERROR_NONE != err, "favor_get_mf_list() Failed()");

	while (CONTACTS_ERROR_NONE == err) {
		ph_favor_info *f_info;
		bool is_favorite;

		if (3 <= count)
			break;

		f_info = calloc(1, sizeof(ph_favor_info));
		if (NULL == f_info) {
			ERR("calloc() return NULL");
			break;
		}

		err = phone_cts_get_bool_from_list(mf_list, _contacts_person_usage.is_favorite, &is_favorite);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_bool_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		if (true == is_favorite) {
			err = contacts_list_next(mf_list);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_next() Failed(%d)", err);
				free(f_info);
				break;
			}
			continue;
		}
		err = phone_cts_get_int_from_list(mf_list, _contacts_person_usage.person_id, &f_info->person_id);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_int_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		err = phone_cts_get_str_from_list(mf_list, _contacts_person_usage.display_name, &f_info->display);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_str_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		err = phone_cts_get_str_from_list(mf_list, _contacts_person_usage.image_thumbnail_path, &f_info->img_path);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("phone_cts_get_str_from_list() Failed(%d)", err);
			free(f_info);
			break;
		}
		if (CT_UG_REQUEST_SELECT_NUMBER == favor_d->ug_request) {
			f_info->contact_list = ctui_person_get_contact_list(f_info->person_id);
			p_warn_if(NULL == f_info->contact_list, "ctui_person_get_contact_list() return NULL");
			f_info->number_list= ctui_person_get_number_list(f_info->contact_list);
			p_warn_if(NULL == f_info->number_list, "ctui_person_get_contact_list() return NULL");
		}
		else if (CT_UG_REQUEST_SELECT_EMAIL == favor_d->ug_request) {
			f_info->contact_list = ctui_person_get_contact_list(f_info->person_id);
			p_warn_if(NULL == f_info->contact_list, "ctui_person_get_contact_list() return NULL");
			f_info->email_list= ctui_person_get_email_list(f_info->contact_list);
			p_warn_if(NULL == f_info->email_list, "ctui_person_get_contact_list() return NULL");

		}

		list = eina_list_append(list, (void*)f_info);
		count ++;
		err = contacts_list_next(mf_list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}
	err = contacts_list_destroy(mf_list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

	favor_d->mf_list = list;
	favor_d->mf_list_count = count;
}

static void favor_create_favorite(ph_favorite_data *favor_d)
{
	PH_FN_CALL;
	Evas_Object *gengrid;
	int wx = 0;
	int hy = 0;

	favor_d->box = elm_box_add(favor_d->layout);
	evas_object_size_hint_weight_set(favor_d->box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(favor_d->layout, "contents", favor_d->box);
	evas_object_show(favor_d->box);

	gengrid = elm_gengrid_add(favor_d->box);
	elm_object_theme_set(gengrid, favor_d->th);
	evas_object_size_hint_weight_set(gengrid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(gengrid, EVAS_HINT_FILL, EVAS_HINT_FILL);

	ecore_x_window_size_get(ecore_x_window_root_get(ecore_x_window_focus_get()), &wx, &hy);

	elm_gengrid_item_size_set(gengrid, wx/3, wx/3);
	elm_gengrid_align_set(gengrid, 0.0, 0.0);
	elm_gengrid_horizontal_set(gengrid, EINA_FALSE);
	elm_gengrid_multi_select_set(gengrid, EINA_TRUE);

	evas_object_smart_callback_add(gengrid, "moved", favor_grid_move, favor_d);
	evas_object_smart_callback_add(gengrid, "longpressed", favor_grid_longpress, favor_d);

	favor_d->favor_gengrid = gengrid;
	elm_gengrid_reorder_mode_set(favor_d->favor_gengrid, EINA_FALSE);
	evas_object_data_set(gengrid, KEY_FAVOR_DATA, favor_d);
	evas_object_show(favor_d->favor_gengrid);
	elm_box_pack_end(favor_d->box, favor_d->favor_gengrid);
}

static void favor_fill_layout(ph_favorite_data *favor_d)
{
	if (favor_d->ctx_popup) {
		evas_object_del(favor_d->ctx_popup);
		favor_d->ctx_popup = NULL;
	}

	if ( 0 == favor_d->favor_list_count &&  0 < favor_d->mf_list_count && 0 == ctui_get_mfc_option())
	{
		if (NULL == favor_d->favor_gengrid)
			favor_create_favorite(favor_d);
		else
			elm_gengrid_clear(favor_d->favor_gengrid);
		if (favor_d->list_mode == CT_LIST_MODE_DEFAULT)
			ph_favor_fill_mf_gengrid(favor_d);
	}
	else if (0 < favor_d->favor_list_count || (0 < favor_d->mf_list_count && 0 == ctui_get_mfc_option())) {
		if (NULL == favor_d->favor_gengrid)
			favor_create_favorite(favor_d);
		else
			elm_gengrid_clear(favor_d->favor_gengrid);
		ph_favor_fill_favor_gengrid(favor_d);
		if (0 == ctui_get_mfc_option() && favor_d->list_mode == CT_LIST_MODE_DEFAULT)
			ph_favor_fill_mf_gengrid(favor_d);
	}
	else if (0 == favor_d->favor_list_count && (0 == favor_d->mf_list_count || 1 == ctui_get_mfc_option())) {
		if (favor_d->favor_gengrid) {
			elm_gengrid_clear(favor_d->favor_gengrid);
			evas_object_del(favor_d->favor_gengrid);
			evas_object_del(favor_d->box);
			favor_d->favor_gengrid = NULL;
			favor_d->box = NULL;
		}
		Evas_Object *base;
		base = phone_create_nocontents(favor_d->layout, T_(PH_GET_TEXT_BASIC, PHTEXT_NO_FAVORITES));
		p_retm_if(NULL == base, "elm_layout_add() return NULL");
		elm_object_part_content_set(favor_d->layout, "contents", base);
		evas_object_show(base);
	}
}

static Eina_Bool favor_data_changed_cb(void *data)
{
	ph_favorite_data *favor_d = data;
	p_retvm_if(NULL == favor_d, ECORE_CALLBACK_CANCEL, "favor_d is null");

	if (favor_d->update_timer_favor) {
		ecore_timer_del(favor_d->update_timer_favor);
		favor_d->update_timer_favor = NULL;
	}

	ph_favor_load_mf_list(favor_d);
	ph_favor_load_favor_list(favor_d);
	favor_fill_layout(favor_d);
	ph_favor_update_button_status(favor_d);

	return ECORE_CALLBACK_CANCEL;
}

void ph_favor_data_lazy_update(const char *view_uri, void *data)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d = data;

	if (favor_d->updating)
		return;

	if (favor_d->update_timer_favor) {
		ecore_timer_del(favor_d->update_timer_favor);
		favor_d->update_timer_favor = NULL;
	}

	PH_DBG("favor_d->is_background=%d",favor_d->is_background);

	if (favor_d->is_background)
		favor_d->update_timer_favor = ecore_timer_add(3.0, favor_data_changed_cb, favor_d);
	else
		favor_data_changed_cb(favor_d);
}

static Eina_Bool favor_log_data_changed_cb(void *data)
{
	ph_favorite_data *favor_d = data;
	p_retvm_if(NULL == favor_d, ECORE_CALLBACK_CANCEL, "favor_d is null");

	if (favor_d->update_timer_log) {
		ecore_timer_del(favor_d->update_timer_log);
		favor_d->update_timer_log = NULL;
	}

	ph_favor_load_mf_list(favor_d);
	favor_fill_layout(favor_d);
	ph_favor_update_button_status(favor_d);

	return ECORE_CALLBACK_CANCEL;
}

static void favor_log_data_lazy_update(const char *view_uri, void *data)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d = data;

	if (favor_d->update_timer_log) {
		ecore_timer_del(favor_d->update_timer_log);
		favor_d->update_timer_log = NULL;
	}

	PH_DBG("favor_d->is_background=%d",favor_d->is_background);

	if (favor_d->is_background)
		favor_d->update_timer_log = ecore_timer_add(3.0, favor_log_data_changed_cb, favor_d);
	else
		favor_log_data_changed_cb(favor_d);
}

static int favor_get_person_id_by_contact_id(int contact_id)
{
	int person_id = 0;
	contacts_record_h record = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_db_get_record(_contacts_contact._uri, contact_id, &record);
	p_retvm_if(CONTACTS_ERROR_NONE != err, err, "contacts_db_get_record() Failed()");

	err = contacts_record_get_int(record, _contacts_contact.person_id, &person_id);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_record_get_int() Failed(%d)", err);
		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	}

	err = contacts_record_destroy(record, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
	return person_id;
}

static ph_favor_info* favor_find_info_by_contact_id(ph_favorite_data *favor_d, int contact_id)
{
	int person_id;
	ph_favor_info *f_info;
	Eina_List *l;

	person_id = favor_get_person_id_by_contact_id(contact_id);
	EINA_LIST_FOREACH(favor_d->favor_list, l, f_info) {
		if (!f_info) continue;
		if (f_info->person_id == person_id)
			return f_info;
	}
	return NULL;
}

static int favor_set_updated_data(ph_favorite_data *favor_d, int ab_id)
{
	int contact_id;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_list_h list = NULL;
	int ver;
	int type;
	contacts_record_h record = NULL;

	err = contacts_db_get_changes_by_version(_contacts_contact_updated_info._uri, ab_id, favor_d->version, &list, &ver);
	p_retvm_if(CONTACTS_ERROR_NONE != err, FALSE, "contacts_db_get_changes_by_version() Failed(%d)", err);

	if (!list)
		return TRUE;

	while (CONTACTS_ERROR_NONE == err) {
		ph_favor_info *f_info;
		err = phone_cts_get_int_from_list(list, _contacts_contact_updated_info.contact_id, &contact_id);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_int_from_list() Failed(%d)", err);
				break;
			}
		err = phone_cts_get_int_from_list(list, _contacts_contact_updated_info.type, &type);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_int_from_list() Failed(%d)", err);
				break;
			}

		if (favor_d->version < ver)
			favor_d->version = ver;

		switch (type) {
		case CONTACTS_CHANGE_UPDATED :
			f_info = favor_find_info_by_contact_id(favor_d, contact_id);

			if (f_info) {
				favor_d->is_mf_changed = true;
				err = contacts_db_get_record(_contacts_person._uri, f_info->person_id, &record);
				if (CONTACTS_ERROR_NONE != err) {
					ERR("contacts_db_get_record() is failed");
					f_info->img_path = NULL;
					break;
				}
				else {
					free(f_info->display);
					err = contacts_record_get_str(record, _contacts_person.display_name, &f_info->display);
					if (CONTACTS_ERROR_NONE != err) {
						ERR("contacts_record_get_str() Failed(%d)", err);
						err = contacts_record_destroy(record, true);
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
						break;
					}
					f_info->number_loaded = false;
					free(f_info->number);
					f_info->number = NULL;
					free(f_info->img_path);
					err = contacts_record_get_str(record, _contacts_person.image_thumbnail_path, &f_info->img_path);
					if (CONTACTS_ERROR_NONE != err) {
						ERR("contacts_record_get_str() Failed(%d)", err);
						err = contacts_record_destroy(record, true);
						p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
						break;
					}
				}
				if (f_info->img_path && *f_info->img_path)
					elm_gengrid_item_item_class_update(f_info->item, &grid_itc_favor);
				else
					elm_gengrid_item_item_class_update(f_info->item, &grid_itc_favor_no_id);
			}
			err = contacts_record_destroy(record, true);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
			break;
		case CONTACTS_CHANGE_DELETED:
			favor_free_data(favor_d->favor_list);
			favor_d->favor_list = NULL;
			favor_load_favorite_data(favor_d);
			break;
		case CONTACTS_CHANGE_INSERTED :
		default :
			break;
		}
		err = contacts_list_next(list);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
	}
	phone_cts_free_list(list);
	return TRUE;
}

static Eina_Bool favor_contact_data_changed_cb(void *data)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d = data;
	contacts_list_h ab_list = NULL;
	int ab_id;
	int ret;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retvm_if(NULL == favor_d, ECORE_CALLBACK_CANCEL, "favor_d is null");

	favor_d->is_mf_changed = false;

	if (favor_d->update_timer_contact) {
		ecore_timer_del(favor_d->update_timer_contact);
		favor_d->update_timer_contact = NULL;
	}

	ret = favor_set_updated_data(favor_d, 0);
	if (!ret) {
		ERR("favor_set_updated_data is failed(%d)", ret);
		return ECORE_CALLBACK_CANCEL;
	}

	err = contacts_db_get_all_records(_contacts_address_book._uri, 0, 0, &ab_list);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_db_get_all_records is failed(%d)", err);
		return ECORE_CALLBACK_CANCEL;
	}

	while (CONTACTS_ERROR_NONE == err) {
		err = phone_cts_get_int_from_list(ab_list, _contacts_address_book.id, &ab_id);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("phone_cts_get_int_from_list() Failed(%d)", err);
				break;
			}
		ret = favor_set_updated_data(favor_d, ab_id);
		if (!ret) {
			ERR("favor_set_updated_data is failed");
			break;
		}
		err = contacts_list_next(ab_list);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
	}
	phone_cts_free_list(ab_list);

	ph_favor_load_mf_list(favor_d);
	favor_fill_layout(favor_d);

	ph_favor_update_button_status(favor_d);
	return ECORE_CALLBACK_CANCEL;
	PH_FN_END;
}

static void favor_view_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	ph_favorite_data *favor_d = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retm_if(NULL == favor_d, "favor_d is null");

	if (favor_d->p_info) {
		free(favor_d->p_info);
	}
	// TODO: Fixme - person_changed

	err = contacts_db_remove_changed_cb(_contacts_person._uri, ph_favor_data_lazy_update, favor_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_remove_changed_cb() Failed(%d)", err);

	err = contacts_db_remove_changed_cb(_contacts_phone_log._uri, favor_log_data_lazy_update, favor_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_remove_changed_cb() Failed(%d)", err);


	if (favor_d->th) {
		elm_theme_extension_del(favor_d->th, FAVORITES_THEME_EDJ);
		elm_theme_extension_del(favor_d->th, CONTACT_BUTTON_THEME_EDJ);
		elm_theme_free(favor_d->th);
	}

	if (favor_d->favor_list)
		favor_free_data(favor_d->favor_list);

	if (favor_d->mf_list)
		favor_free_data(favor_d->mf_list);

	if (favor_d->popup)
		evas_object_del(favor_d->popup);

	if (favor_d->update_timer_contact)
		ecore_timer_del(favor_d->update_timer_contact);

	if (favor_d->update_timer_log)
		ecore_timer_del(favor_d->update_timer_log);

	if (favor_d->update_timer_favor)
		ecore_timer_del(favor_d->update_timer_favor);

	free(favor_d);
}

static void favor_set_gengrid_item_style()
{
	grid_itc_favor.item_style = "gridtext_favorites";
	grid_itc_favor.func.text_get = favor_label_get;
	grid_itc_favor.func.content_get = favor_icon_get;
	grid_itc_favor.func.state_get = NULL;
	grid_itc_favor.func.del = NULL;

	grid_itc_mf.item_style = "gridtext_favorites";
	grid_itc_mf.func.text_get = favor_mf_label_get;
	grid_itc_mf.func.content_get = favor_mf_icon_get;
	grid_itc_mf.func.state_get = NULL;
	grid_itc_mf.func.del = NULL;

	grid_itc_favor_no_id.item_style = "gridtext_favorites_no_id";
	grid_itc_favor_no_id.func.text_get = favor_label_get;
	grid_itc_favor_no_id.func.content_get = favor_icon_get;
	grid_itc_favor_no_id.func.state_get = NULL;
	grid_itc_favor_no_id.func.del = NULL;

	grid_itc_mf_no_id.item_style = "gridtext_favorites_no_id";
	grid_itc_mf_no_id.func.text_get = favor_mf_label_get;
	grid_itc_mf_no_id.func.content_get = favor_mf_icon_get;
	grid_itc_mf_no_id.func.state_get = NULL;
	grid_itc_mf_no_id.func.del = NULL;

	grid_itc_index.item_style = "group_index";
	grid_itc_index.func.text_get = favor_group_label_get;

	grid_itc_mf_index.item_style = "group_index";
	grid_itc_mf_index.func.text_get = favor_group_label_get;
}

static int favor_create_layout(ph_favorite_data *favor_d, Evas_Object *parent)
{
	PH_FN_CALL;
	Evas_Object *base;
	Evas_Object *layout;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	favor_d->navi = parent;
	base = phone_create_base_layout(favor_d->navi, false);

	layout = elm_layout_add(base);
	elm_object_part_content_set(base, "elm.swallow.content", layout);

	p_retvm_if(NULL == layout, -1, "elm_layout_add return NULL");
	elm_layout_file_set(layout, SELECTION_INFO_EDJ, "selectioninfo_layout");

	favor_d->layout = layout;
	evas_object_show(layout);

	favor_set_gengrid_item_style();

	if (0 == ctui_get_mfc_option()) {
		ph_favor_load_mf_list(favor_d);
	}

	ph_favor_load_favor_list(favor_d);
	favor_fill_layout(favor_d);

	evas_object_event_callback_add(favor_d->layout, EVAS_CALLBACK_DEL,
			favor_view_delete_cb, favor_d);

	// TODO: Fixme - person_changed
	err = contacts_db_add_changed_cb(_contacts_person._uri, ph_favor_data_lazy_update, favor_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);

	err = contacts_db_add_changed_cb(_contacts_phone_log._uri, favor_log_data_lazy_update, favor_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);

	return 0;
}

static void favor_more_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d  = data;
	Evas_Object *popup = NULL;
	int x, y, w, h;

	if (favor_d->popup)
		__favor_popup_hide_cb(favor_d, NULL, NULL);

	popup = elm_ctxpopup_add(favor_d->navi);
	p_retm_if(NULL == popup, "elm_ctxpopup_add() return NULL");

	elm_ctxpopup_item_append(popup, S_(PH_SYS_SK_EDIT), NULL, favor_edit_btn_cb, favor_d);

	elm_ctxpopup_direction_priority_set(popup, ELM_CTXPOPUP_DIRECTION_DOWN, ELM_CTXPOPUP_DIRECTION_UP, -1, -1);

	evas_object_smart_callback_add(popup, "dismissed", __favor_popup_hide_cb, favor_d);

	evas_object_geometry_get(obj, &x, &y, &w, &h);

	evas_object_move(popup, x+(w/2), y+(h/2));
	evas_object_show(popup);

	favor_d->popup = popup;

}

API void phone_favor_set_navi(Evas_Object *layout, int mode)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d = evas_object_data_get(layout, KEY_FAVOR_DATA);
	Elm_Object_Item *navi_item;
	Evas_Object *l_btn;

	p_retm_if(NULL == favor_d, "favor data is NULL");
	navi_item = elm_naviframe_bottom_item_get(favor_d->navi);
	p_retm_if(NULL == navi_item, "navi item is NULL");

	phone_navi_item_clear(navi_item);
	if (favor_d->ug) {
		if (mode == PH_FAVOR_MODE_EDIT) {
			favor_d->navi_button = phone_create_toolbar_btn(favor_d->navi, S_(CT_SYS_SK_DONE), favor_done_btn_cb, favor_d);
			elm_object_item_part_content_set(navi_item, "toolbar_button1", favor_d->navi_button);
			elm_object_style_set(favor_d->navi_button, "naviframe/toolbar/left");

			l_btn = phone_create_toolbar_btn(favor_d->navi, S_(PH_SYS_SK_CANCEL), phone_ug_destroy_me_cb, favor_d->ug);
			elm_object_item_part_content_set(navi_item, "toolbar_button2", l_btn);
			elm_object_style_set(l_btn, "naviframe/toolbar/right");
		}
		else {
			l_btn = phone_create_toolbar_btn(favor_d->navi, S_(PH_SYS_SK_CANCEL), phone_ug_destroy_me_cb, favor_d->ug);
			elm_object_item_part_content_set(navi_item, "toolbar_button1", l_btn);
		}
	}
	else {
		if (mode == PH_FAVOR_MODE_DEFAULT) {
			Evas_Object *add_button = NULL;

			favor_d->navi_button = ctui_create_more_btn(favor_d->navi, favor_more_btn_clicked_cb, favor_d);
			elm_object_item_part_content_set(favor_d->navi_item, "toolbar_more_btn", favor_d->navi_button);

			add_button = phone_create_toolbar_btn(favor_d->navi, S_(CT_SYS_SK_ADD), favor_add_btn_cb, favor_d);
			elm_object_item_part_content_set(navi_item, "toolbar_button1", add_button);

			l_btn = elm_button_add(favor_d->navi);
			evas_object_smart_callback_add(l_btn, "clicked", phone_back_btn_clicked_cb, favor_d->win);
			elm_object_style_set(l_btn, "naviframe/back_btn/default");
			elm_object_item_part_content_set(favor_d->navi_item, "prev_btn", l_btn);
		}
		else { // edit mode
			favor_d->navi_button = phone_create_toolbar_btn(favor_d->navi, S_(PH_SYS_SK_DELETE), ph_favor_edit_del_btn_cb, favor_d);
			elm_object_item_part_content_set(navi_item, "toolbar_button1", favor_d->navi_button);
			elm_object_style_set(favor_d->navi_button, "naviframe/toolbar/left");

			l_btn = phone_create_toolbar_btn(favor_d->navi, S_(PH_SYS_SK_CANCEL), ph_favor_edit_cancel_cb, favor_d);
			elm_object_item_part_content_set(navi_item, "toolbar_button2", l_btn);
			elm_object_style_set(l_btn, "naviframe/toolbar/right");
		}
	}
	ph_favor_update_button_status(favor_d);
}

Evas_Object* phone_favor_create_view(ph_view_info *view_info)
{
	int ret;
	Elm_Theme *th;
	ph_favorite_data *favor_d = calloc(1, sizeof(ph_favorite_data));
	p_retvm_if(NULL == favor_d, NULL, "Fail to calloc ph_favorite_data");
	favor_d->ug = view_info->ug;
	favor_d->win = view_info->win;
	favor_d->navi_item = elm_naviframe_bottom_item_get(view_info->navi);
	favor_d->ug_request = view_info->ug_request;
	favor_d->ug_data = view_info->ug_data;
	favor_d->ug_cnt_max = view_info->ug_cnt_max;
	favor_d->is_background = true;

	th = elm_theme_new();
	elm_theme_ref_set(th, NULL);
	elm_theme_extension_add(th, FAVORITES_THEME_EDJ);
	elm_theme_extension_add(th, CONTACT_BUTTON_THEME_EDJ);

	favor_d->th = th;

	ret = favor_create_layout(favor_d, view_info->navi);
	if (ret < 0) {
		free(favor_d);
		return NULL;
	}

	evas_object_data_set(favor_d->layout, KEY_FAVOR_DATA, favor_d);
	return favor_d->layout;
}

API void phone_favor_view_language_changed(Evas_Object *base)
{
	ph_favorite_data *favor_d;
	favor_d = evas_object_data_get(base, KEY_FAVOR_DATA);
	p_retm_if(NULL == favor_d, "favor_d is null");
	favor_data_changed_cb(favor_d);

	if (PH_FAVOR_MODE_EDIT == favor_d->list_mode)
		ph_favor_edit_set_selection_info(favor_d);
}

void phone_favor_view_reset(Evas_Object *base)
{
	PH_FN_CALL;
	ph_favorite_data *favor_d;
	Elm_Object_Item *item;

	favor_d = evas_object_data_get(base, KEY_FAVOR_DATA);
	p_retm_if(NULL == favor_d, "favor_d is null");

	if (favor_d->child_ug) {
		ug_destroy(favor_d->child_ug);
		favor_d->child_ug = NULL;
	}

	if (PH_UG_REQUEST_CHECK != favor_d->ug_request &&
			PH_UG_REQUEST_CHECK_FOR_NUMBER != favor_d->ug_request &&
			PH_UG_REQUEST_CHECK_FOR_EMAIL != favor_d->ug_request)
		if (PH_FAVOR_MODE_EDIT == favor_d->list_mode)
			ph_favor_edit_mode_end(favor_d, false);

	item = elm_naviframe_bottom_item_get(favor_d->navi);
	if (elm_naviframe_top_item_get(favor_d->navi) != item)
		elm_naviframe_item_pop_to(item);

	favor_d->navi_button = NULL;
}

void phone_favor_set_background(Evas_Object *obj, bool is_background)
{
	ph_favorite_data *favor_d;
	favor_d = evas_object_data_get(obj, KEY_FAVOR_DATA);
	p_retm_if(NULL == favor_d, "favor data is null");

	favor_d->is_background = is_background;

	if (favor_d->update_timer_contact) {
		ecore_timer_del(favor_d->update_timer_contact);
		favor_d->update_timer_contact = NULL;
		favor_contact_data_changed_cb(favor_d);
	}

	if (favor_d->update_timer_favor) {
		ecore_timer_del(favor_d->update_timer_favor);
		favor_d->update_timer_favor = NULL;
		favor_data_changed_cb(favor_d);
	}

	if (favor_d->update_timer_log) {
		ecore_timer_del(favor_d->update_timer_log);
		favor_d->update_timer_log = NULL;
		favor_log_data_changed_cb(favor_d);
	}

	if (true == is_background)
		favor_d->mfc_option = ctui_get_mfc_option();
	else
		favor_fill_layout(favor_d);
}

