/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <errno.h>
#include <fcntl.h>
#include <ui-gadget-module.h>
#include "phone.h"
#include "ct-detail.h"
#include "ct-list.h"
#include "ct-list-contact-view.h"
#include "ct-list-utils.h"

int ct_list_parse_bundle(service_h service, ct_list_data* clist_d)
{
	char* val = NULL;

	if (service) {
		service_get_extra_data(service, CT_UG_BUNDLE_TYPE, &val);
		if (val) clist_d->ug_request = atoi(val);
		PH_DBG("type = %d", clist_d->ug_request);
		free(val);

		switch (clist_d->ug_request) {
		case CT_UG_REQUEST_SELECT_GROUP:
		case CT_UG_REQUEST_SELECT_GROUP_ADD_TO_HOME:
			return 0;
		case CT_UG_REQUEST_LIST:
		case CT_UG_REQUEST_LIST_WITH_BACK:
		case CT_UG_REQUEST_SELECT_PERSON:
		case CT_UG_REQUEST_SHARE_CONTACT_INFO:
		case CT_UG_REQUEST_SELECT_VCARD:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}

			break;
		case CT_UG_REQUEST_SELECT_NUMBER:
		case CT_UG_REQUEST_SELECT_CONTACT_ADD_TO_HOME:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER;
			}
			break;
		case CT_UG_REQUEST_SELECT_EMAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_EMAIL;
			}
			break;
		case CT_UG_REQUEST_SELECT_NUMBER_OR_EMAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL;
			}
			break;
		case CT_UG_REQUEST_UPDATE_CONTACT:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			service_get_extra_data(service, CT_UG_BUNDLE_NUM, &clist_d->ug_data);
			if(clist_d->ug_data)
				break;
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &clist_d->ug_data);
			if(clist_d->ug_data)
				break;
			service_get_extra_data(service, CT_UG_BUNDLE_WEB, &clist_d->ug_data);
			if(clist_d->ug_data)
				break;
			break;
		case CT_UG_REQUEST_UPDATE_WITH_NUM:
			service_get_extra_data(service, CT_UG_BUNDLE_NUM, &clist_d->ug_data);
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->base_ab = atoi(val);
				free(val);
			}
			clist_d->list_op = CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID;
			break;
		case CT_UG_REQUEST_UPDATE_WITH_EMAIL:
			service_get_extra_data(service, CT_UG_BUNDLE_EMAIL, &clist_d->ug_data);
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->base_ab = atoi(val);
				free(val);
			}
			clist_d->list_op = CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID;
			break;
		case CT_UG_REQUEST_UPDATE_WITH_WEB:
			service_get_extra_data(service, CT_UG_BUNDLE_WEB, &clist_d->ug_data);
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->base_ab = atoi(val);
				free(val);
			}
			clist_d->list_op = CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID;
			break;
		case CT_UG_REQUEST_CHECK:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				service_get_extra_data(service, CT_UG_BUNDLE_MAX, &val);
				if (val) {
					clist_d->cnt_max_check = atoi(val);
					free(val);
				}
			}
			break;
		case CT_UG_REQUEST_CHECK_FOR_NUMBER:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if (val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER;
				service_get_extra_data(service, CT_UG_BUNDLE_MAX, &val);
				if (val)
					clist_d->cnt_max_check = atoi(val);
				free(val);
			}
			break;
		case CT_UG_REQUEST_CHECK_FOR_EMAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if(val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_EMAIL;
				service_get_extra_data(service, CT_UG_BUNDLE_MAX, &val);
				if (val)
					clist_d->cnt_max_check = atoi(val);
				free(val);
			}
			break;
		case CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL:
			val = NULL;
			service_get_extra_data(service, CT_UG_BUNDLE_ACCOUNTID, &val);
			if (val) {
				clist_d->list_op = CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID;
				clist_d->base_ab = atoi(val);
				free(val);
			}
			else {
				clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL;
				service_get_extra_data(service, CT_UG_BUNDLE_MAX, &val);
				if (val)
					clist_d->cnt_max_check = atoi(val);
				free(val);
			}
			break;
			break;
		case PH_UG_REQUEST_SET_WITH_IMG:
		case PH_UG_REQUEST_SET_WITH_RINGTONE:
		case CT_UG_REQUEST_SAVE_IMG:
		case CT_UG_REQUEST_SAVE_RINGTONE:
			service_get_extra_data(service, CT_UG_BUNDLE_PATH, &clist_d->ug_data);
			if(CT_UG_REQUEST_SAVE_IMG == clist_d->ug_request)
				clist_d->list_op = CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID;
			if (-1 == access(clist_d->ug_data, R_OK)) {
				ERR("Invalid filepath(%s)", clist_d->ug_data);
				return -2;
			}
			break;
		default:
			ERR("Invalid request(%d)", clist_d->ug_request);
			return -1;
		}
	}
	return 0;
}

static void list_img_crop_result_for_add(ui_gadget_h ug,service_h service, void *data)
{
	contacts_record_h record_image = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_detail_data *cdetail_d = data;

	p_retm_if(NULL == ug || NULL == data, "ug or data is NULL");

	if (service) {
		char *normal = NULL;
		char *full = NULL;

		service_get_extra_data(service, "crop_image_path", &normal);
		p_retm_if(!normal, "return value is NULL.\n");

		service_get_extra_data(service, "image_path", &full);
		record_image = ctui_get_record_image(cdetail_d->contact);

		err = contacts_record_set_str(record_image, _contacts_image.path, normal);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

		free(normal);
		free(full);
	}
	cdetail_d->only_input = true;
	ctui_create_input_view(cdetail_d);
}

static void list_destory_detail_child_ug(ui_gadget_h ug, void *priv)
{
	ct_detail_data *cdetail_d = priv;
	p_retm_if(NULL == ug, "ug is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == priv, "priv is NULL");
	cdetail_d->child_ug = NULL;
}

void ct_list_destroy_child_ug(void *data)
{
	ct_list_data *clist_d = (ct_list_data*)data;
	clist_d->child_ug = NULL;
}

API void ctui_list_update_with_data(ph_view_info *view_info, void *data, int person_id, int grp_id, int ab_id)
{
	PH_FN_CALL;
	ct_detail_data *cdetail_d;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	struct ug_cbs cbs = {0};

	cdetail_d = calloc(1, sizeof(ct_detail_data));
	p_retm_if(!cdetail_d, "cdetail_d is NULL, Not enough memory");

	cdetail_d->win = view_info->win;
	cdetail_d->navi = view_info->navi;
	cdetail_d->ug = view_info->ug;
	cdetail_d->prev_navi_item = elm_naviframe_top_item_get(view_info->navi);
	cdetail_d->prev_view_data = data;
	cdetail_d->person_id = person_id;
	cdetail_d->base_grp = grp_id;
	cdetail_d->base_ab = ab_id;
	cdetail_d->ug_request = view_info->ug_request;

	if (person_id) {
		Eina_List *list_contact;
		err = contacts_db_get_record(_contacts_person._uri, person_id, &(cdetail_d->person));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

		list_contact = ctui_person_get_writable_contact_list(person_id);
		if (NULL == list_contact) {
			err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);
		}
		else {
			err = contacts_record_clone((contacts_record_h)list_contact->data, &(cdetail_d->contact));
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_clone() Failed(%d)", err);
			ctui_free_record_list(list_contact);
		}
	}

	if (NULL == cdetail_d->contact) {
		err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);
	}

	switch (view_info->ug_request) {
		//case PH_UG_REQUEST_SET_WITH_NUM:
	case CT_UG_REQUEST_UPDATE_WITH_NUM:
		err = contacts_record_create(_contacts_number._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_number.number, view_info->ug_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

		err = contacts_record_set_int(record, _contacts_number.type, CONTACTS_NUMBER_TYPE_CELL);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

		err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.number, record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
		break;
		//case PH_UG_REQUEST_SET_WITH_EMAIL:
	case CT_UG_REQUEST_UPDATE_WITH_EMAIL:
		err = contacts_record_create(_contacts_email._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_email.email, view_info->ug_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

		err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.email, record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
		break;
		//case PH_UG_REQUEST_SET_WITH_WEB:
	case CT_UG_REQUEST_UPDATE_WITH_WEB:
		err = contacts_record_create(_contacts_url._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_url.url, view_info->ug_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

		err = contacts_record_add_child_record(cdetail_d->contact, _contacts_contact.url, record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_add_child_record() Failed(%d)", err);
		break;
	case CT_UG_REQUEST_UPDATE_CONTACT:
		break;
	case PH_UG_REQUEST_SET_WITH_IMG:
	case CT_UG_REQUEST_SAVE_IMG: //only handle for add
		cbs.result_cb = list_img_crop_result_for_add;
		cbs.destroy_cb = list_destory_detail_child_ug;
		cbs.priv = cdetail_d;
		cdetail_d->child_ug = phone_image_crop_ug(cdetail_d->ug, view_info->ug_data, &cbs);
		return;
	case PH_UG_REQUEST_SET_WITH_RINGTONE:
	case CT_UG_REQUEST_SAVE_RINGTONE: //only handle for add
		err = contacts_record_set_str(cdetail_d->contact, _contacts_contact.ringtone_path, view_info->ug_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);
		break;
	default:
		ERR("Invalid type(%d)", view_info->ug_request);
		break;
	}
	cdetail_d->only_input = true;
	ctui_create_input_view(cdetail_d);
	PH_FN_END;
}

void ct_list_call(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d;
	ct_person_list *person_info = data;
	p_retm_if(NULL == person_info, "parameter(person_info) is NULL");
	p_retm_if(NULL == person_info->number || '\0' == *person_info->number, "number is NULL");
	phone_launch_voice_call(person_info->id, person_info->number);

	clist_d = (ct_list_data *)evas_object_data_get(obj, "clist_d");
	if(clist_d)
		ctui_list_set_unsweep(clist_d);
}

void ct_list_msg(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d;
	ct_person_list *person_info = data;

	p_retm_if(NULL == data, "parameter(person_info) is NULL");
	p_retm_if(NULL == person_info->number || '\0' == *person_info->number, "number is NULL");

	clist_d = (ct_list_data *)evas_object_data_get(obj, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	clist_d->child_ug = ctui_msg_ug(clist_d->ug, person_info->number, person_info->id, NULL,
			ct_list_destroy_child_ug, clist_d);

	ctui_list_set_unsweep(clist_d);
}

void ct_list_popup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d;

	clist_d = evas_object_data_get(data, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get(clist_d) return NULL");
	evas_object_del(clist_d->popup);
	clist_d->popup = NULL;
}

static void list_sweep_delete_contact(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_list_data *clist_d;
	contacts_record_h record = NULL;
	int contact_id =0;

	PH_DBG("##### [TSP_TEST] list->sweep delete start#####");

	clist_d = evas_object_data_get(obj, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");
	evas_object_del(clist_d->popup);
	clist_d->popup = NULL;

	evas_object_data_set(clist_d->navi, "ct_list_update_ui", (void *)1);

	err = contacts_db_get_record(_contacts_person._uri,(int)data,&record);
	p_retm_if(err != CONTACTS_ERROR_NONE, "contacts_db_get_record() fail");
	err = contacts_record_get_int(record,_contacts_person.display_contact_id,&contact_id);
	p_retm_if(err != CONTACTS_ERROR_NONE, "contacts_record_get_int() fail");
	err = contacts_db_delete_record(_contacts_person._uri, (int)data);
	if (CONTACTS_ERROR_NONE == err)
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
	else
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
}


void ct_list_sweep_del_cb(void *data, Evas_Object *obj, void *event_info)
{
	int id;
	ct_list_data *clist_d;
	Evas_Object *btn;

	clist_d = evas_object_data_get(obj, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	id = (int)data;
	btn = phone_create_confirm_popup(&clist_d->popup, clist_d->win, S_(PH_SYS_POP_DELETE_Q), list_sweep_delete_contact, (void *)id);
	evas_object_data_set(btn,"clist_d", clist_d);

	ctui_list_set_unsweep(clist_d);
}

void ct_list_get_title(ct_list_data *clist_d, int list_op, int id_ab, int id_grp)
{
	char *title = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	char* group_system_id = NULL;

	// TODO: Fixme
	switch(list_op) {
	case CT_GET_LIST_ALL_PERSON:
	case CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID:
	case CT_GET_LIST_ALL_PERSON_HAD_EMAIL:
	case CT_GET_LIST_ALL_PERSON_HAD_NUMBER:
		{
			char buf[PH_TEXT_SHORT_LEN];
			snprintf(buf, sizeof(buf), "%s (%d)", T_(CT_GET_TEXT_BASIC, CTTEXT_ALL_CONTACTS), clist_d->cnt_list_total);
			clist_d->title = strdup(buf);
		}
		break;
	case CT_GET_LIST_PERSON_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID:
		if (0 < id_ab) {
			char temp[PH_TEXT_MAX_LEN];
			temp[0] = '\0';
			ctui_get_addressbook_name(id_ab, temp, sizeof(temp));
			if (*temp)
				title = strdup(temp);
		}
		else
			title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
		clist_d->sub_title = title;
		clist_d->title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_ALL_CONTACTS));
		break;
	case CT_GET_LIST_PERSON_OF_GROUP_ID:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_GROUP_ID:
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_GROUP_ID:
		err = contacts_db_get_record(_contacts_group._uri, id_grp, &record);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_record_get() Failed(%d)", err);

		err = contacts_record_get_str_p(record, _contacts_group.system_id, &group_system_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		if (group_system_id != NULL) {
			if (strcmp(group_system_id,"friends")==0)
				clist_d->title = SAFE_STRDUP(T_(CT_GET_TEXT_BASIC, CTTEXT_FRIENDS));
			else if (strcmp(group_system_id,"family")==0)
				clist_d->title = SAFE_STRDUP(T_(CT_GET_TEXT_BASIC, CTTEXT_FAMILY));
			else if (strcmp(group_system_id,"coworkers")==0)
				clist_d->title = SAFE_STRDUP(T_(CT_GET_TEXT_BASIC, CTTEXT_CO_WORKERS));
			else
				clist_d->title = SAFE_STRDUP(T_(CT_GET_TEXT_ERR, CTTEXT_NONAME));
		}else{
			err = contacts_record_get_str(record, _contacts_group.name, &(clist_d->title));
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);
		}
		err = contacts_record_destroy(record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

		if(0 < id_ab) {
			char temp[PH_TEXT_MAX_LEN];
			temp[0] = '\0';
			ctui_get_addressbook_name(id_ab, temp, sizeof(temp));
			if (*temp)
				title = strdup(temp);
		}
		else
			title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
		clist_d->sub_title = title;
		break;
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_EMAIL_OF_ADDRBOOK_ID:
		clist_d->title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_NOT_ASSIGNED));
		if(0 < id_ab) {
			char temp[PH_TEXT_MAX_LEN];
			temp[0] = '\0';
			ctui_get_addressbook_name(id_ab, temp, sizeof(temp));
			if (*temp)
				title = strdup(temp);
		}
		else
			title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_PHONE));
		clist_d->sub_title = title;
		break;

	}
}

static void __list_add_contacts_to_favorite(ct_list_data *clist_d);

static void __list_add_contacts_to_favorite_cb(int err, void *data)
{
	ct_list_data *clist_d = data;
	ph_progress_info *p_info = NULL;

	p_retm_if(NULL == clist_d, "Invalid parameter(clist_d is NULL)");
	p_info = clist_d->p_info;
	p_retm_if(NULL == p_info, "Invalid parameter(p_info is NULL)");

	if (p_info->update_list) {
		contacts_list_destroy(p_info->update_list, true);
		p_info->update_list = NULL;
	}

	if (CONTACTS_ERROR_NONE != err || p_info->alive == false) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
		free(p_info);
		clist_d->p_info = NULL;
		clist_d->updating = false;
		elm_naviframe_item_pop_to(evas_object_data_get(clist_d->navi, "navi_it_pop"));
		return;
	}
	else if (p_info->cnt == p_info->cnt_checked_total) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(CT_SYS_POP_SUCCESS));
		free(p_info);
		clist_d->p_info = NULL;
		elm_naviframe_item_pop_to(evas_object_data_get(clist_d->navi, "navi_it_pop"));
		return;
	}
	else {
		__list_add_contacts_to_favorite(clist_d);
	}

}

static void __list_add_contacts_to_favorite(ct_list_data *clist_d)
{
	PH_FN_CALL;
	const int LIST_BULK_LENGTH = 50;
	int count = 0;
	int base_count = 0;
	double value = 0.0;
	char percent_total[PH_TEXT_MAX_LEN+1] = {0, };
	char percent[PH_TEXT_MAX_LEN+1] = {0, };
	ph_progress_info *p_info = NULL;
	contacts_record_h record = NULL;
	contacts_list_h list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Eina_List *l;
	ct_person_list *person_info;

	p_retm_if(NULL == clist_d, "parameter(clist_d) is NULL");
	p_info = clist_d->p_info;
	p_retm_if(NULL == p_info, "parameter(p_info) is NULL");

	err = contacts_list_create(&list);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_list_create() Failed(%d)", err);

	EINA_LIST_FOREACH(clist_d->list, l, person_info) {
		if (NULL == person_info) continue;
		if (person_info->checked) {
			if (base_count < p_info->cnt) {
				base_count++;
				continue;
			}

			err = contacts_db_get_record(_contacts_person._uri, person_info->id, &record);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_db_get_record() Failed(%d)", err);
				p_info->result = FALSE;
				p_info->completed = true;
				break;
			}
			err = contacts_record_set_bool(record, _contacts_person.is_favorite, true);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_record_set_bool() Failed(%d)", err);
				p_info->result = FALSE;
				p_info->completed = true;
				break;
			}

			err = contacts_list_add(list, record);

			count++;
			if (LIST_BULK_LENGTH == count || (count + p_info->cnt) == p_info->cnt_checked_total) {
				p_info->cnt += count;
				break;
			}
		}
	}
	value = (double)p_info->cnt / (double)p_info->cnt_checked_total;
	elm_progressbar_value_set(p_info->progressbar, value);
	snprintf(percent, sizeof(percent), "%d%%", (int)(100.0 * (double)p_info->cnt/(double)p_info->cnt_checked_total));
	snprintf(percent_total, sizeof(percent_total), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", percent_total);

	p_info->update_list = list;
	contacts_db_update_records_async(list, __list_add_contacts_to_favorite_cb, clist_d);
}


void ct_list_add_contacts_to_favorite(Evas_Object *win, Evas_Object *navi, int cnt)
{
	ct_list_data *clist_d;
	ph_progress_info *p_info;

	clist_d = evas_object_data_get(navi, "favor_list_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get return NULL");

	p_retm_if(clist_d->p_info, "thread is running");

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc return NULL");

	clist_d->p_info = p_info;
	p_info->cnt_checked_total = cnt;
	p_info->alive = true;
	p_info->result = TRUE;
	p_info->popup = phone_progressbar_popup(win, p_info, T_(CT_GET_TEXT_BASIC, CTTEXT_ADDING_ING));
	__list_add_contacts_to_favorite(clist_d);
}

static void __list_add_group_relation(int group_id, int person_id, contacts_list_h list_result)
{
	contacts_record_h record_group = NULL;
	contacts_list_h list = NULL;
	contacts_filter_h filter = NULL;
	contacts_query_h query = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	p_retm_if(group_id <= 0, "Invalid parameter(group_id is %d)", group_id);
	p_retm_if(person_id <= 0, "Invalid parameter(person_id is %d)", person_id);
	p_retm_if(NULL == list_result, "Invalid parameter(list is NULL)");

	do {
		int address_book_id = 0;
		if (CONTACTS_ERROR_NONE != (err = contacts_db_get_record(_contacts_group._uri, group_id, &record_group))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_record_get_int(record_group, _contacts_group.address_book_id, &address_book_id))) break;

		if (CONTACTS_ERROR_NONE != (err = contacts_query_create(_contacts_contact._uri, &query))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_filter_create(_contacts_contact._uri, &filter))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_int(filter, _contacts_contact.address_book_id, CONTACTS_MATCH_EQUAL, address_book_id))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_filter_add_int(filter, _contacts_contact.person_id, CONTACTS_MATCH_EQUAL, person_id))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_query_set_filter(query, filter))) break;
		if (CONTACTS_ERROR_NONE != (err = contacts_db_get_records_with_query(query, 0, 0, &list))) break;

		while (CONTACTS_ERROR_NONE == err) {
			contacts_record_h record = NULL;
			contacts_record_h record_clone = NULL;
			contacts_record_h record_group_relation = NULL;
			if (CONTACTS_ERROR_NONE != (err = contacts_list_get_current_record_p(list, &record))) break;
			if (CONTACTS_ERROR_NONE != (err = contacts_record_clone(record, &record_clone))) {
				ERR("contacts_record_clone() Failed(%d)", err);
				break;
			}
			if (CONTACTS_ERROR_NONE != (err = contacts_record_create(_contacts_group_relation._uri, &record_group_relation))) {
				ERR("contacts_record_create() Failed(%d)", err);
				break;
			}
			contacts_record_set_int(record_group_relation, _contacts_group_relation.group_id, group_id);
			contacts_record_add_child_record(record_clone, _contacts_contact.group_relation, record_group_relation);
			contacts_list_add(list_result, record_clone);

			err = contacts_list_next(list);
		}
		contacts_filter_destroy(filter);
		contacts_query_destroy(query);
		contacts_list_destroy(list, true);
		contacts_record_destroy(record_group, true);
		return;
	}while (0);
	contacts_filter_destroy(filter);
	contacts_query_destroy(query);
	contacts_list_destroy(list, true);
	contacts_record_destroy(record_group, true);
	return;
}

static void __list_add_contacts_to_group(ct_list_data *clist_d);

static void __list_add_contacts_to_group_cb(int err, void *data)
{
	ct_list_data *clist_d = data;
	ph_progress_info *p_info = NULL;

	p_retm_if(NULL == clist_d, "Invalid parameter(clist_d is NULL)");
	p_info = clist_d->p_info;
	p_retm_if(NULL == p_info, "Invalid parameter(p_info is NULL)");

	if (p_info->update_list) {
		contacts_list_destroy(p_info->update_list, true);
		p_info->update_list = NULL;
	}

	if (CONTACTS_ERROR_NONE != err || p_info->alive == false) {
		char *err_msg = NULL;
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		switch (err) {
		case CONTACTS_ERROR_DB:
			err_msg = T_(CT_GET_TEXT_ERR, CTTEXT_ERR_DB_LOCKED);
			break;
		case CONTACTS_ERROR_FILE_NO_SPACE:
			err_msg = S_(CT_SYS_POP_NOT_ENOUGH_MEMORY);
			break;
		default:
			err_msg = S_(PH_SYS_POP_FAILED);
			break;
		}

		phone_show_tickernoti(err_msg);
		free(p_info);
		clist_d->p_info = NULL;
		clist_d->updating = false;
		elm_naviframe_item_pop_to(evas_object_data_get(clist_d->navi, "navi_it_pop"));
		return;
	}
	else if (p_info->cnt == p_info->cnt_checked_total) {
		if (p_info->popup) {
			evas_object_del(p_info->popup);
			p_info->popup = NULL;
		}
		phone_show_tickernoti(S_(CT_SYS_POP_SUCCESS));
		free(p_info);
		clist_d->p_info = NULL;
		elm_naviframe_item_pop_to(evas_object_data_get(clist_d->navi, "navi_it_pop"));
		return;
	}
	else {
		__list_add_contacts_to_group(clist_d);
	}
}


static void __list_add_contacts_to_group(ct_list_data *clist_d)
{
	PH_FN_CALL;
	const int LIST_BULK_LENGTH = 50;
	int count = 0;
	int base_count = 0;
	double value = 0.0;
	char percent_total[PH_TEXT_MAX_LEN+1] = {0, };
	char percent[PH_TEXT_MAX_LEN+1] = {0, };

	contacts_list_h list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ph_progress_info *p_info = NULL;
	Eina_List *l = NULL;
	ct_person_list *person_info = NULL;

	p_retm_if(NULL == clist_d, "parameter(clist_d) is NULL");
	p_info = clist_d->p_info;
	p_retm_if(NULL == p_info, "parameter(p_info) is NULL");

	err = contacts_list_create(&list);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_list_create() Failed(%d)", err);

	EINA_LIST_FOREACH(clist_d->list, l, person_info) {
		if (NULL == person_info) continue;

		if (person_info->checked) {
			if (base_count < p_info->cnt) {
				base_count++;
				continue;
			}

			__list_add_group_relation(clist_d->base_grp, person_info->id, list);

			count++;
			if (LIST_BULK_LENGTH == count || (count + p_info->cnt) == p_info->cnt_checked_total) {
				p_info->cnt += count;
				break;
			}
		}
	}

	value = (double)p_info->cnt / (double)p_info->cnt_checked_total;
	elm_progressbar_value_set(p_info->progressbar, value);
	snprintf(percent, sizeof(percent), "%d%%", (int)(100.0 * (double)p_info->cnt/(double)p_info->cnt_checked_total));
	snprintf(percent_total, sizeof(percent_total), "%d/%d", p_info->cnt, p_info->cnt_checked_total);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext1", percent);
	edje_object_part_text_set(elm_layout_edje_get(p_info->layout), "elm.text.subtext2", percent_total);

	p_info->update_list = list;
	contacts_db_update_records_async(list, __list_add_contacts_to_group_cb, clist_d);
}


void ct_list_add_contacts_to_group(Evas_Object *win, Evas_Object *navi, int cnt)
{
	PH_FN_CALL;
	ct_list_data *clist_d;
	ph_progress_info *p_info;

	clist_d = evas_object_data_get(navi, "gadd_list_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get return NULL");

	p_retm_if(clist_d->p_info, "thread is running");

	p_info = calloc(1, sizeof(ph_progress_info));
	p_retm_if(NULL == p_info, "calloc return NULL");

	clist_d->p_info = p_info;
	p_info->cnt_checked_total = cnt;
	p_info->alive = true;
	p_info->result = TRUE;
	p_info->popup = phone_progressbar_popup(win, p_info, T_(CT_GET_TEXT_BASIC, CTTEXT_ADDING_MEMBERS_TO_GROUP));
	__list_add_contacts_to_group(clist_d);
}

