/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <notification.h>
#include <ui-gadget-module.h>
#include "vconf-keys.h"
#include "phone.h"
#include "phone-log.h"
#include "ph-log-utils.h"
#include "ph-log-main-view.h"
#include "ph-log-detail-view.h"
#include "ph-log-check-view.h"
#include "ct-detail.h"

#define KEY_LOG_DATA	"log_data"
#define NO_CONTENTS_ICON_POSITION 0.35

static Elm_Genlist_Item_Class itc, itc_one, itc_date, itc_viewby_list;

enum {
	PH_LIST_ALL_CALL_MESSAGE_LOG,
	PH_LIST_MISSED_CALL_LOG,
	PH_LIST_EMAIL_LOG,
	PH_LIST_ALL_CALL_LOG,//more_popup
	PH_LIST_DIALED_CALL_LOG,
	PH_LIST_RECEIVED_CALL_LOG,
	PH_LIST_REJECTED_CALL_LOG,
	PH_LIST_ALL_MESSAGE_LOG,
	PH_LIST_ALL_CALL_MESSAGE_EMAIL_LOG,
};	//log type

enum {
	VIEWBY_POPUP_ALL_LOG = 0,
	VIEWBY_POPUP_ALL_CALL_LOG = 1,
	VIEWBY_POPUP_MISSED_CALL_LOG = 2,
	VIEWBY_POPUP_DIALED_CALL_LOG = 3,
	VIEWBY_POPUP_RECEIVED_CALL_LOG = 4,
	VIEWBY_POPUP_REJECTED_CALL_LOG = 5,
	VIEWBY_POPUP_ALL_MESSAGE_LOG = 6,
};	//set the value with viewby index

static void log_destroy_child_ug(void *data)
{
	ph_log_data *log_d = data;
	log_d->child_ug = NULL;
}

static void log_gl_msg_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_data *log_d = evas_object_data_get(obj, KEY_LOG_DATA);
	log_d->child_ug = phone_launch_msg_composer_ug(log_d->ug, data, 0,
			log_destroy_child_ug, log_d);
}

// today: -1,  yesterday : -2, previous : positive value
static void log_append_separate_all_date_index(ph_log_data *log_d, void *data, int index)
{
	ph_log_info *l_info = data;
	time_t stime;
	struct timeval timeval;
	struct tm *tmp;
	int yday, cur_yday;

	gettimeofday(&timeval, 0);
	tmp = localtime(&timeval.tv_sec);
	cur_yday = tmp->tm_yday;

	stime = l_info->plog_log_time - l_info->plog_duration;
	tmp = localtime(&stime);

	yday = tmp->tm_yday;

	if (yday == cur_yday)
		l_info->group_index = -1;
	else if (yday == (cur_yday-1))
		l_info->group_index = -2;
	else
		l_info->group_index = yday;

	if (yday+1 != log_d->pre_date_type) {
		log_d->pre_date_type = yday+1;
		log_d->git = elm_genlist_item_append(log_d->genlist, &itc_date,
				data, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
		elm_genlist_item_select_mode_set(log_d->git, ELM_OBJECT_SELECT_MODE_NONE);
	}
}

static void __log_update_checked_item(ph_log_data *log_d, ph_log_info *l_selected_info)
{

	Eina_List *l;
	ph_log_info *l_info;
	Eina_Bool		bChecked = l_selected_info->checked;

	EINA_LIST_FOREACH(log_d->log_list, l, l_info) {
		if (!l_info) continue;
		if (l_info->group_base_id == l_selected_info->id)
		{
			l_info->checked = !bChecked;
			//PH_DBG("called id : %d, %d", l_info->id, l_info->group_base_id);
		}
	}

	if (l_selected_info->checked)
		log_d->cnt_checked+=l_selected_info->grouped_count;
	else
		log_d->cnt_checked-=l_selected_info->grouped_count;

}

static void log_gl_item_sel(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	service_h service;
	char buf[16];
	Elm_Object_Item *item = event_info;
	ph_log_info *l_info = data;
	ph_log_data *log_d = evas_object_data_get(obj, KEY_LOG_DATA);
	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (elm_genlist_decorate_mode_get(log_d->genlist)) {
		__log_update_checked_item(log_d, l_info);
		elm_genlist_item_fields_update(event_info, "elm.edit.icon.1", ELM_GENLIST_ITEM_FIELD_CONTENT);
		ph_log_check_update_select_all(log_d);
		ph_log_check_update_selection_info(log_d);
		ph_log_check_update_button_status(log_d);
		return;
	}

	PH_DBG("view_type : %d", log_d->view_type);
	switch (log_d->view_type) {
	case LOG_VIEW_TYPE_UG_LAUNCH_CALL:
		if (l_info->plog_number && *l_info->plog_number)
			phone_launch_voice_call(l_info->plog_ct_id, l_info->plog_number);
		break;
	case LOG_VIEW_TYPE_UG_NUMBER_SELECT:
	case LOG_VIEW_TYPE_UG_EMAIL_SELECT:
	case LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_SELECT:
		service_create(&service);
		snprintf(buf, sizeof(buf), "%d", l_info->id);
		service_add_extra_data(service, PH_UG_BUNDLE_RESULT_PLOG_ID, buf);
		PH_DBG("plog_id : %d %s", l_info->id, buf);
		ug_send_result(log_d->ug, service);
		service_destroy(service);
		ug_destroy_me(log_d->ug);
		break;
	case LOG_VIEW_TYPE_NORMAL:
		ph_create_log_detail_view(log_d, l_info);
		break;
	default:
		ERR("Invalid view type(%d)", log_d->view_type);
		break;
	}
}

#define N 20
static bool log_compare_type(int type, int pre_type)
{
	switch (pre_type) {
	case CONTACTS_PLOG_TYPE_VOICE_INCOMMING:
	case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING:
	case CONTACTS_PLOG_TYPE_MMS_INCOMMING:
	case CONTACTS_PLOG_TYPE_SMS_INCOMMING:
	case CONTACTS_PLOG_TYPE_VOICE_OUTGOING:
	case CONTACTS_PLOG_TYPE_VIDEO_OUTGOING:
	case CONTACTS_PLOG_TYPE_MMS_OUTGOING:
	case CONTACTS_PLOG_TYPE_SMS_OUTGOING:
		switch (type) {
		case CONTACTS_PLOG_TYPE_VOICE_INCOMMING:
		case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING:
		case CONTACTS_PLOG_TYPE_MMS_INCOMMING:
		case CONTACTS_PLOG_TYPE_SMS_INCOMMING:
		case CONTACTS_PLOG_TYPE_VOICE_OUTGOING:
		case CONTACTS_PLOG_TYPE_VIDEO_OUTGOING:
		case CONTACTS_PLOG_TYPE_MMS_OUTGOING:
		case CONTACTS_PLOG_TYPE_SMS_OUTGOING:
			return true;
		default :
			return false;
		}
	case CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN:
	case CONTACTS_PLOG_TYPE_VOICE_INCOMMING_SEEN:
	case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_UNSEEN:
	case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_SEEN:
		switch (type) {
		case CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN:
		case CONTACTS_PLOG_TYPE_VOICE_INCOMMING_SEEN:
		case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_UNSEEN:
		case CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_SEEN:
			return true;
		default :
			return false;
		}
	case CONTACTS_PLOG_TYPE_VOICE_REJECT:
	case CONTACTS_PLOG_TYPE_VIDEO_REJECT:
		switch (type) {
		case CONTACTS_PLOG_TYPE_VOICE_REJECT:
		case CONTACTS_PLOG_TYPE_VIDEO_REJECT:
			return true;
		default :
			return false;
		}
	case CONTACTS_PLOG_TYPE_VOICE_BLOCKED:
	case CONTACTS_PLOG_TYPE_VIDEO_BLOCKED:
	case CONTACTS_PLOG_TYPE_SMS_BLOCKED:
		switch (type) {
		case CONTACTS_PLOG_TYPE_VOICE_BLOCKED:
		case CONTACTS_PLOG_TYPE_VIDEO_BLOCKED:
		case CONTACTS_PLOG_TYPE_SMS_BLOCKED:
			return true;
		default :
			return false;
		}
	default:
		return false;
	}
}

static int log_fill_genlist_sub(ph_log_data * log_d, Eina_List *l, ph_log_info *pre_info)
{
	ph_log_info *l_info;
	int count;
	int cnt_append;
	int pre_type;
	int pre_date;
	int pre_ct_id;
	int last_group_base_id = 0;
	bool diff;
	char *pre_num;
	Elm_Object_Item *item;
	Elm_Object_Item *pre_item;

	if (pre_info) {
		pre_date = pre_info->group_index;
		pre_ct_id = pre_info->plog_ct_id;
		pre_type = pre_info->plog_log_type;
		pre_item = pre_info->item;
		pre_num = SAFE_STRDUP(pre_info->plog_number);
	}
	else {
		pre_date = -3;
		pre_ct_id = -1;
		pre_type = CONTACTS_PLOG_TYPE_NONE;
		pre_item = NULL;
		pre_num = NULL;
	}

	cnt_append = 0;
	count = 0;
	EINA_LIST_FOREACH(l, l, l_info) {
		if (N == cnt_append) break;
		diff = false;
		log_append_separate_all_date_index(log_d, l_info, cnt_append + log_d->readed_n);

		if (l_info->group_index != pre_date)
			diff = true;
		else if (!log_compare_type(l_info->plog_log_type, pre_type))
			diff = true;
		else if (0 < l_info->plog_ct_id && 0 < pre_ct_id && l_info->plog_ct_id != pre_ct_id)
			diff = true;
		else if (l_info->plog_number && *l_info->plog_number) {
			if (!pre_num)
				diff = true;
			else if (strcmp(l_info->plog_number, pre_num))
				diff = true;
		}

		if (diff) {
			if (pre_num)
				free(pre_num);
			if (l_info->plog_number && *l_info->plog_number) {
				item = elm_genlist_item_append(log_d->genlist, &itc, l_info,
						log_d->git, ELM_GENLIST_ITEM_NONE, log_gl_item_sel, l_info);
				pre_num = strdup(l_info->plog_number);
			}
			else {
				item = elm_genlist_item_append(log_d->genlist, &itc, l_info,
						log_d->git, ELM_GENLIST_ITEM_NONE, log_gl_item_sel, l_info);
				pre_num = NULL;
			}
			pre_date = l_info->group_index;
			pre_type = l_info->plog_log_type;
			pre_ct_id = l_info->plog_ct_id;
			pre_item = item;
			l_info->grouped_count = 1;
			pre_info = l_info;
			cnt_append++;
			last_group_base_id = l_info->id;
		}
		else
			pre_info->grouped_count++;

		l_info->group_base_id = last_group_base_id;
		l_info->item = pre_item;
		count++;
	}
	return count;
}

static Eina_Bool log_append_item_part(void *data)
{
	int count;
	Eina_List *l;
	ph_log_info *l_info;
	ph_log_data * log_d = data;

	l = eina_list_nth_list(log_d->log_list, log_d->readed_n-1);
	l_info = l->data;
	count = log_fill_genlist_sub(log_d, l->next, l_info);
	if (N <= count) {
		log_d->readed_n += count;
		return ECORE_CALLBACK_RENEW;
	}
	else {
		log_d->list_idler = NULL;
		return ECORE_CALLBACK_CANCEL;
	}
}

static void log_fill_genlist(ph_log_data * log_d)
{
	int count;

	p_retm_if(NULL == log_d || NULL == log_d->genlist, "Genlist is null");

	if (log_d->list_idler) {
		ecore_idler_del(log_d->list_idler);
		log_d->list_idler = NULL;
	}

	log_d->readed_n = 0;
	count = log_fill_genlist_sub(log_d, log_d->log_list, NULL);

	if (N <= count) {
		log_d->readed_n += count;
		log_d->list_idler = ecore_idler_add(log_append_item_part, log_d);
	}
}

static void log_free_log_list(ph_log_data *log_d)
{
	ph_log_info *l_info;

	if(log_d->log_list) {

		EINA_LIST_FREE(log_d->log_list, l_info) {
			if (NULL == l_info) continue;
			free(l_info->plog_display);
			free(l_info->plog_img_path);
			free(l_info->plog_number);
			free(l_info);
		}
		log_d->log_list = NULL;
	}
}

static unsigned int get_cts_log_list(int type, contacts_list_h* pCts_list)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_filter_h filter = NULL;
	contacts_query_h query = NULL;
	unsigned int record_cnt = 0;

	int log_type_min, log_type_max;
	err = contacts_query_create(_contacts_person_phone_log._uri, &query);
	p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_query_create() Failed(%d)", err);

	err = contacts_filter_create( _contacts_person_phone_log._uri, &filter);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_filter_create() Failed(%d)", err);
		err = contacts_query_destroy(query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
		return NULL;
	}

	switch (type)
	{
	case PH_LIST_MISSED_CALL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN;
		log_type_max = CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_SEEN;
		break;
	case PH_LIST_EMAIL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_EMAIL_RECEIVED;
		log_type_max = CONTACTS_PLOG_TYPE_EMAIL_SENT;
		break;
	case PH_LIST_ALL_CALL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_INCOMMING;
		log_type_max = CONTACTS_PLOG_TYPE_VIDEO_BLOCKED;
		break;
	case PH_LIST_DIALED_CALL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_OUTGOING;
		log_type_max =CONTACTS_PLOG_TYPE_VIDEO_OUTGOING;
		break;
	case PH_LIST_RECEIVED_CALL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_INCOMMING;
		log_type_max =CONTACTS_PLOG_TYPE_VIDEO_INCOMMING;
		break;
	case PH_LIST_REJECTED_CALL_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_REJECT;
		log_type_max = CONTACTS_PLOG_TYPE_VIDEO_REJECT;
		break;
	case PH_LIST_ALL_MESSAGE_LOG:
		log_type_min = CONTACTS_PLOG_TYPE_MMS_INCOMMING;
		log_type_max = CONTACTS_PLOG_TYPE_MMS_BLOCKED;
		break;
	case PH_LIST_ALL_CALL_MESSAGE_LOG:
	default:
		log_type_min = CONTACTS_PLOG_TYPE_VOICE_INCOMMING;
		log_type_max = CONTACTS_PLOG_TYPE_MMS_BLOCKED;
		break;
	}

	if(type == PH_LIST_DIALED_CALL_LOG || type == PH_LIST_RECEIVED_CALL_LOG)
	{
		err = contacts_filter_add_int(filter, _contacts_person_phone_log.log_type, CONTACTS_MATCH_EQUAL, log_type_min);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_int() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}

		err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_OR);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_bool() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}

		err = contacts_filter_add_int(filter, _contacts_person_phone_log.log_type, CONTACTS_MATCH_EQUAL, log_type_max);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_int() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}
	}
	else {
		err = contacts_filter_add_int(filter, _contacts_person_phone_log.log_type, CONTACTS_MATCH_GREATER_THAN_OR_EQUAL, log_type_min);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_int() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}

		err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_AND);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_bool() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}

		err = contacts_filter_add_int(filter, _contacts_person_phone_log.log_type, CONTACTS_MATCH_LESS_THAN_OR_EQUAL, log_type_max);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_add_int() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}
	}
	err = contacts_query_set_filter(query, filter);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_query_set_filter() Failed(%d)", err);
		err = contacts_query_destroy(query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
		err = contacts_filter_destroy(filter);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
		return NULL;
	}

	err = contacts_query_set_sort(query, _contacts_person_phone_log.log_time, false);
	if (CONTACTS_ERROR_NONE != err) {
		ERR("contacts_query_set_filter() Failed(%d)", err);
		err = contacts_query_destroy(query);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
		err = contacts_filter_destroy(filter);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
		return NULL;
	}

	// if output param NULL, just count the records
	if(pCts_list != NULL)
	{
		err = contacts_db_get_records_with_query(query, 0, 0, pCts_list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_db_get_records_with_query() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			err = contacts_filter_destroy(filter);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
			return NULL;
		}
		contacts_list_get_count(*pCts_list, &record_cnt );
	}
	else
	{
		contacts_db_get_count_with_query(query, &record_cnt);
	}


	err = contacts_query_destroy(query);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
	err = contacts_filter_destroy(filter);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);

	PH_DBG("get person_phonelog table records(type:%d, count:%d)", type, record_cnt);
	return record_cnt;

}

static int log_load_data(int type,  ph_log_data* log_d, bool include_unknown)
{
	PH_FN_CALL;
	int count = 0;
	int omit_count = 0;
	contacts_list_h cts_list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	count = (int) get_cts_log_list(type, &cts_list);
	while (CONTACTS_ERROR_NONE == err && count > 0) {
		ph_log_info *l_info = NULL;
		char *number;
		err = phone_cts_get_str_from_list(cts_list, _contacts_person_phone_log.address, &number);
		p_warn_if(CONTACTS_ERROR_NONE != err, "phone_cts_get_str_from_list() Failed(%d)", err);


		if (false == include_unknown && (!number || !*number)) {
			err = contacts_list_next(cts_list);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_next() Failed(%d)", err);
				break;
			}
			omit_count++;
			continue;
		}

		l_info = calloc(1, sizeof(ph_log_info));
		p_retvm_if(l_info == NULL, NULL, "ph_log_info calloc Failed.");

		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.log_id, &l_info->id);
		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.person_id, &l_info->plog_ct_id);
		l_info->plog_number = SAFE_STRDUP(number);
		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.log_time, &l_info->plog_log_time);
		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.log_type, &l_info->plog_log_type);
		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.extra_data1, &l_info->plog_duration);
		err = phone_cts_get_int_from_list(cts_list, _contacts_person_phone_log.address_type, &l_info->plog_num_type);

		err = phone_cts_get_str_from_list(cts_list, _contacts_person_phone_log.display_name, &l_info->plog_display);
		err = phone_cts_get_str_from_list(cts_list, _contacts_person_phone_log.image_thumbnail_path, &l_info->plog_img_path);

		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
		log_d->log_list = eina_list_append(log_d->log_list, (void*)l_info);

		err = contacts_list_next(cts_list);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_list_next() Failed(%d)", err);
			break;
		}
	}
	contacts_list_destroy(cts_list, true);

	return count;
}
static int log_get_log_list_type(int viewby_type)
{
	switch(viewby_type){
		case VIEWBY_ALL_LOG :
			return PH_LIST_ALL_CALL_MESSAGE_LOG;
		case VIEWBY_ALL_CALL:
			return PH_LIST_ALL_CALL_LOG;
		case VIEWBY_MISSED_CALL:
			return PH_LIST_MISSED_CALL_LOG;
		case VIEWBY_DIALED_CALL:
			return PH_LIST_DIALED_CALL_LOG;
		case VIEWBY_RECEIVED_CALL:
			return PH_LIST_RECEIVED_CALL_LOG;
		case VIEWBY_REJECTED_CALL:
			return PH_LIST_REJECTED_CALL_LOG;
		case VIEWBY_ALL_MESSAGE:
			return PH_LIST_ALL_MESSAGE_LOG;
		default:
			return PH_LIST_ALL_CALL_MESSAGE_LOG;
		}
}
static void log_load_list(ph_log_data * log_d)
{
	int count;
	p_retm_if(NULL == log_d, "Parameter is null");
	log_free_log_list(log_d);

	// need to reset all log count
	log_d->all_log_list_count = -1;

	switch(log_d->view_type)
	{
		case LOG_VIEW_TYPE_UG_EMAIL_SELECT:
		case LOG_VIEW_TYPE_UG_EMAIL_CHECK:
			 log_d->log_list_count = log_load_data(PH_LIST_EMAIL_LOG, log_d, true);
			break;
		case LOG_VIEW_TYPE_NORMAL:
		case LOG_VIEW_TYPE_UG_LAUNCH_CALL:
			if(log_get_log_list_type(log_d->viewby_type) == PH_LIST_ALL_CALL_MESSAGE_LOG)
				log_d->all_log_list_count = log_d->log_list_count = log_load_data(PH_LIST_ALL_CALL_MESSAGE_LOG, log_d, true);
			else
				log_d->log_list_count = log_load_data(log_get_log_list_type(log_d->viewby_type), log_d, true);
			break;
		case LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_CHECK:
		case LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_SELECT:
			log_d->log_list_count = log_load_data(PH_LIST_ALL_CALL_MESSAGE_EMAIL_LOG, log_d, true);
 			break;
		case LOG_VIEW_TYPE_UG_NUMBER_CHECK:
		case LOG_VIEW_TYPE_UG_NUMBER_SELECT:
		default:
			log_d->all_log_list_count = log_d->log_list_count = log_load_data(PH_LIST_ALL_CALL_MESSAGE_LOG, log_d, true);
 			break;
	}

	//all_log_count for dim
	if(log_d->all_log_list_count == -1)
		log_d->all_log_list_count = get_cts_log_list(PH_LIST_ALL_CALL_MESSAGE_LOG, NULL);
 }

static void log_update_db_result_cb(int err, void *user_data)
{
	p_warn_if(CONTACTS_ERROR_NONE != err, "log_update_db_result_cb() result : error(%d)", err);
	p_retm_if(user_data == NULL, "log_update_db_result_cb() list handle is NULL.");
	contacts_list_h cts_list = user_data;

	err = contacts_list_destroy(cts_list, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() result : error(%d)", err);

	return;
}
Eina_Bool ph_log_set_log_as_seen(int index, int type)
{
	int logtype = 0;
	contacts_record_h log_record = NULL;
	contacts_list_h cts_list = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_filter_h filter = NULL;
	contacts_query_h query = NULL;
	unsigned int record_cnt = 0;

	p_retvm_if(CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN != type &&
			CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_UNSEEN != type &&
			CONTACTS_PLOG_TYPE_NONE != type,
			false,
			"The type is invalid. It must be CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN"
			" or CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_UNSEEN or none");

	if (0 == index) {
		err = contacts_query_create(_contacts_phone_log._uri, &query);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_query_create() Failed(%d)", err);

		err = contacts_filter_create(_contacts_phone_log._uri, &filter);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_filter_create() Failed(%d)", err);
			err = contacts_query_destroy(query);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
			return false;
		}

		if (CONTACTS_PLOG_TYPE_NONE == type)
		{
			err = contacts_filter_add_int(filter, _contacts_phone_log.log_type, CONTACTS_MATCH_EQUAL, CONTACTS_PLOG_TYPE_VOICE_INCOMMING_UNSEEN);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_filter_add_int() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_OR);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_filter_add_bool() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_filter_add_int(filter, _contacts_phone_log.log_type, CONTACTS_MATCH_EQUAL, CONTACTS_PLOG_TYPE_VIDEO_INCOMMING_UNSEEN);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_filter_add_int() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_query_set_filter(query, filter);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_query_set_filter() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_create() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_db_get_records_with_query(query, 0, 0, &cts_list);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_db_get_records_with_query() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			contacts_list_get_count( cts_list, &record_cnt );
			PH_DBG("get phonelogs table records(index: %d, log_type:%d, count:%d)",index,  type, record_cnt);

			while(CONTACTS_ERROR_NONE == err && record_cnt>0)
			{
				phone_cts_get_int_from_list( cts_list, _contacts_phone_log.log_type, &logtype);
				err = contacts_list_get_current_record_p(cts_list, &log_record);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_get_current_record_p() Failed(%d)", err);
				err = contacts_record_set_int( log_record, _contacts_phone_log.log_type, logtype+1);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_list_next(cts_list);
				if (CONTACTS_ERROR_NONE != err) {
					ERR("contacts_list_next() Failed(%d)", err);
					break;
				}
			}
			if (record_cnt>0)
			{
				err = contacts_db_update_records_async(cts_list, log_update_db_result_cb, cts_list);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_records_async() Failed(%d)", err);
			}
		}
		else
		{
			err = contacts_filter_add_int(filter, _contacts_phone_log.log_type, CONTACTS_MATCH_EQUAL, type);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_filter_add_int() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_query_set_filter(query, filter);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_query_set_filter() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_list_create() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			err = contacts_db_get_records_with_query(query, 0, 0, &cts_list);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_db_get_records_with_query() Failed(%d)", err);
				err = contacts_query_destroy(query);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
				err = contacts_filter_destroy(filter);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
				return false;
			}

			contacts_list_get_count( cts_list, &record_cnt );
			PH_DBG("get phonelogs table records(index: %d, log_type:%d, count:%d)",index,  type, record_cnt);

			while(CONTACTS_ERROR_NONE == err && record_cnt>0)
			{
				err = contacts_list_get_current_record_p(cts_list, &log_record);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_get_current_record_p() Failed(%d)", err);
				err = contacts_record_set_int( log_record, _contacts_phone_log.log_type, type+1);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

				err = contacts_list_next(cts_list);
				if (CONTACTS_ERROR_NONE != err) {
					ERR("contacts_list_next() Failed(%d)", err);
					break;
				}
			}
			if (record_cnt>0)
			{
				err = contacts_db_update_records_async(cts_list, log_update_db_result_cb, cts_list);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_records_async() Failed(%d)", err);
			}
		}
	}
	else {
		PH_DBG("get phonelogs table record(index: %d, log_type:%d)",index,  type);
		err = contacts_db_get_record( _contacts_phone_log._uri, index, &log_record);
		p_retvm_if(CONTACTS_ERROR_NONE != err, false, "contacts_db_get_record() Failed(%d)", err);

		err = contacts_record_get_int( log_record, _contacts_phone_log.log_type, &logtype );
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
		err = contacts_record_set_int( log_record, _contacts_phone_log.log_type, logtype+1);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
		err = contacts_db_update_record(log_record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
		err = contacts_record_destroy(log_record, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

		return true;
	}

	err = contacts_query_destroy(query);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
	err = contacts_filter_destroy(filter);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);

	return true;
}

Eina_Bool ph_log_missed_call_count_reset(void *data)
{
	ph_log_data * log_d = data;

	notification_delete_group_by_group_id(VOICECALL_PKG, 0, 1001);
	notification_delete_group_by_group_id(VIDEOCALL_PKG, 0, 1001);

	ph_log_set_log_as_seen(0, CONTACTS_PLOG_TYPE_NONE);

	log_d->timer = NULL;
	return ECORE_CALLBACK_CANCEL;
}

API void phone_log_set_background(Evas_Object *obj, bool is_background)
{
	ph_log_data *log_d;
	log_d = evas_object_data_get(obj, KEY_LOG_DATA);
	p_retm_if(NULL == log_d, "log data is null");

	log_d->is_background = is_background;
	if (log_d->update_timer) {
		ecore_timer_del(log_d->update_timer);
		log_d->update_timer = NULL;
		ph_log_data_changed_cb(log_d);
	}
}

API void phone_log_missed_info_update(void *layout)
{
	ph_log_data *log_d;
	log_d = evas_object_data_get(layout, KEY_LOG_DATA);
	p_retm_if(NULL == log_d , "log data is null");

	if(elm_naviframe_top_item_get(log_d->navi) == elm_naviframe_bottom_item_get(log_d->navi)) {
		if (LOG_VIEW_TYPE_NORMAL == log_d->view_type) {
			if (log_d->timer)
				ecore_timer_del(log_d->timer);
			log_d->timer = ecore_timer_add(1.0, ph_log_missed_call_count_reset, (void*)log_d);
		}
	}
}

static void log_delete_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_data * log_d = data;
	phone_log_check_mode_start(log_d);
}

static void log_sweep_delete_reponse_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ph_log_info *l_info = data;
	ph_log_data *log_d = evas_object_data_get(obj, KEY_LOG_DATA);
	p_retm_if(NULL == log_d, "log data is NULL");
	evas_object_del(log_d->popup);
	log_d->popup = NULL;

	err = ph_log_util_del_log_by_id(log_d->log_list, l_info);
	if (CONTACTS_ERROR_NONE != err) {
		phone_show_tickernoti(S_(PH_SYS_POP_FAILED));
	}
	else {
		phone_show_tickernoti(S_(PH_SYS_POP_DELETED));
	}
}

static void log_gl_sweep_del_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_info *l_info = data;
	ph_log_data *log_d = evas_object_data_get(obj, KEY_LOG_DATA);
	char buf[PH_TEXT_MAX_LEN] = {0};
	Evas_Object *btn;
	snprintf(buf, sizeof(buf), "%s?", S_(PH_SYS_SK_DELETE));
	btn = phone_create_confirm_popup(&log_d->popup, log_d->win, buf, log_sweep_delete_reponse_cb, (void *)l_info);
	evas_object_data_set(btn, KEY_LOG_DATA, log_d);
}

static void log_gl_sweep_right(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_log_data *log_d = data;
	if (!event_info) return;
	elm_genlist_item_decorate_mode_set(event_info, "slide", EINA_TRUE);
	elm_genlist_item_select_mode_set(log_d->item_last_sweep, ELM_OBJECT_SELECT_MODE_NONE);
	log_d->item_last_sweep = event_info;
}

static void log_gl_sweep_left(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_log_data *log_d = data;
	if (!event_info) return;
	elm_genlist_item_decorate_mode_set(event_info, "slide", EINA_FALSE);
	elm_genlist_item_select_mode_set(event_info, ELM_OBJECT_SELECT_MODE_DEFAULT);
	log_d->item_last_sweep = NULL;
}

static void log_gl_sweep_up_down(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_data *log_d = data;
	Elm_Object_Item *item;
	if (!obj) return;
	item = (Elm_Object_Item *)elm_genlist_decorated_item_get(obj);
	if (item) {
		elm_genlist_item_decorate_mode_set(item, "slide", EINA_FALSE);
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DEFAULT);
		log_d->item_last_sweep = NULL;
	}
}

static char* log_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[PH_TEXT_MAX_LEN]={0};
	ph_log_info *l_info = data;
	if (0 == strcmp(part, "elm.text.1") || 0 == strcmp(part, "elm.slide.text.1")) {
		if (NULL == l_info->plog_number || '\0' == l_info->plog_number[0]) {
			snprintf(buf, sizeof(buf), "%s (%d)", S_(PH_SYS_BODY_UNKNOWN), l_info->grouped_count);
			return strdup(buf);
		}
		else if (l_info->plog_display && *l_info->plog_display) {
			if (1 < l_info->grouped_count)
				snprintf(buf, sizeof(buf), "%s (%d)", l_info->plog_display, l_info->grouped_count);
			else
				snprintf(buf, sizeof(buf), "%s", l_info->plog_display);
		}
		else {
			if (1 < l_info->grouped_count)
				snprintf(buf, sizeof(buf), "%s (%d)", l_info->plog_number, l_info->grouped_count);
			else
				snprintf(buf, sizeof(buf), "%s", l_info->plog_number);
		}
		return strdup(buf);
	}
	else if ((strcmp(part, "elm.text.2") == 0)) {
		ph_log_data * log_d;
		char *time;
		log_d = evas_object_data_get(obj, KEY_LOG_DATA);
		time = ph_log_util_get_formatted_date(log_d->formatter, l_info->plog_log_time-l_info->plog_duration, PH_LOG_TIME_ONLY);
		if (l_info->plog_display && *l_info->plog_display)
			snprintf(buf, sizeof(buf), "%s %s", l_info->plog_number, time);
		else
			snprintf(buf, sizeof(buf), "%s", time);
		free(time);
		return strdup(buf);
	}
	else
		return NULL;
}

static void log_gl_call_btn_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_log_info * l_info = data;
	p_retm_if(NULL == l_info, "parameter(l_info) is NULL");
	p_retm_if(NULL == l_info->plog_number|| '\0' == *l_info->plog_number, "number is NULL");
	phone_launch_voice_call(l_info->plog_ct_id, l_info->plog_number);
}

static void log_list_ctxpopup_hide_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_data *log_d = data;
	evas_object_del(log_d->popup);
	log_d->popup = NULL;
}

void ct_list_destroy_child_ug(void *data)
{
	ph_log_data *log_d = data;
	log_d->child_ug = NULL;


}



static Evas_Object *log_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[PATH_MAX];
	ph_log_info * l_info = data;
	ph_log_data * log_d;
	Evas_Object *btn = NULL;

	buf[0] = '\0';
	log_d = evas_object_data_get(obj, KEY_LOG_DATA);

	if (0 == strcmp(part, "elm.icon.2")) {
		char *icon_type = phone_detail_log_get_icon_type(l_info->plog_log_type);
		Evas_Object *ic;
		if (icon_type) {
			snprintf(buf, sizeof(buf), "%s", icon_type);
			free(icon_type);

			ic = elm_icon_add(obj);
			p_retvm_if(NULL == ic, NULL, "elm_icon_add is failed");
			elm_icon_file_set(ic, buf, NULL);
			evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			return ic;
		}
	}
	else if (0 == strcmp(part, "elm.icon.1")) {
		char *icon_path = phone_detail_log_get_icon_path_with_action(l_info->plog_log_type);
		if (icon_path) {
			Evas_Object *ic;
			snprintf(buf, sizeof(buf), "%s", icon_path);
			free(icon_path);

			ic = elm_icon_add(obj);
			p_retvm_if(NULL == ic, NULL, "elm_icon_add is failed");
			elm_icon_file_set(ic, buf, NULL);
			evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
			return ic;
		}
	}
	else if (0 == strcmp(part, "elm.icon.3")) {
		Evas_Object *ic;
		if (LOG_VIEW_TYPE_NORMAL != log_d->view_type && LOG_VIEW_TYPE_UG_LAUNCH_CALL != log_d->view_type)
			return NULL;

		if (NULL == l_info->plog_number || '\0' == l_info->plog_number[0])
			return NULL;

		btn = elm_button_add(obj);
		ic = elm_icon_add(btn);
		p_retvm_if(NULL == ic, NULL, "elm_icon_add is failed");
		elm_icon_file_set(ic, PH_IMG_ICON_PHONE, NULL);
		evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		elm_object_part_content_set(btn, "icon", ic);
		evas_object_propagate_events_set(btn, EINA_FALSE);
		evas_object_smart_callback_add(btn, "clicked", log_gl_call_btn_cb, l_info);
		return btn;
	}
	else if (0 == strcmp(part, "elm.slide.swallow.1")) {
		if (l_info->plog_number && *l_info->plog_number) {
			btn = phone_create_list_sweep_btn(obj, PH_IMG_ICON_CALL);
			p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn() return NULL");

			evas_object_data_set(btn, KEY_LOG_DATA, log_d);
			evas_object_smart_callback_add(btn, "clicked", log_gl_call_btn_cb, l_info);
			return btn;
		}
		else {
			btn = phone_create_list_sweep_btn(obj, PH_IMG_ICON_CALL_DIM);
			p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn is failed");
			evas_object_data_set(btn, KEY_LOG_DATA, log_d);
			elm_object_disabled_set(btn, true);
			return btn;

		}
	}
	else if (0 == strcmp(part, "elm.slide.swallow.2")) {
		if (l_info->plog_number && *l_info->plog_number) {
			btn = phone_create_list_sweep_btn(obj, PH_IMG_ICON_MESSAGE);
			p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn is failed");
			evas_object_data_set(btn, KEY_LOG_DATA, log_d);
			evas_object_smart_callback_add(btn, "clicked", log_gl_msg_btn_cb, l_info->plog_number);
			return btn;
		}
		else {
			btn = phone_create_list_sweep_btn(obj, PH_IMG_ICON_MESSAGE_DIM);
			p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn is failed");
			evas_object_data_set(btn, KEY_LOG_DATA, log_d);
			elm_object_disabled_set(btn, true);
			return btn;
		}

	}

	else if (0 == strcmp(part, "elm.slide.swallow.3")) {
			btn = phone_create_list_sweep_btn(obj, PH_IMG_ICON_DELETE);
			p_retvm_if(NULL == btn, NULL, "phone_create_list_sweep_btn() return NULL");

			evas_object_data_set(btn, KEY_LOG_DATA, log_d);
			evas_object_smart_callback_add(btn, "clicked", log_gl_sweep_del_btn_cb, l_info);
			return btn;
	}

	else if (0 == strcmp(part, "elm.edit.icon.1")) {
		Evas_Object *check;
		check = elm_check_add(obj);
		p_retvm_if(NULL == check, NULL, "elm_check_add() return NULL");
		elm_check_state_pointer_set(check, &l_info->checked);
		evas_object_smart_callback_add(check, "changed", ph_log_check_check_changed_cb, log_d);
		return check;
	}
	return NULL;
}

static const char *log_get_day_in_week(ph_log_info *l_info)
{
	time_t stime;
	struct tm *tmp;

	stime = l_info->plog_log_time - l_info->plog_duration;
	tmp = localtime(&stime);

	switch(tmp->tm_wday) {
	case 0:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_SUNDAY);
	case 1:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_MONDAY);
	case 2:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_TUESDAY);
	case 3:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_WEDNESDAY);
	case 4:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_THURSDAY);
	case 5:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_FRIDAY);
	case 6:
		return T_(PH_GET_TEXT_DAY, PHTEXT_BODY_SATURDAY);
	}
	return NULL;
}

static char* log_gl_sep_date_label_get(void *data, Evas_Object *obj, const char *part)
{
	ph_log_info *l_info = data;
	const char *when;
	int index = l_info->group_index;

	if (0 == strcmp(part, "elm.text")) {
		char *date_temp;
		ph_log_data *log_d;

		if (-1 == index)
			when = S_(PH_SYS_BODY_TODAY);
		else if (-2 == index)
			when = S_(PH_SYS_BODY_YESTERDAY);
		else
			when = log_get_day_in_week(l_info);

		log_d = evas_object_data_get(obj, KEY_LOG_DATA);
		date_temp = phone_detail_log_get_date(log_d->formatter, l_info->plog_log_time-l_info->plog_duration);
		if (date_temp) {
			char buf[PATH_MAX]={0};
			snprintf(buf, sizeof(buf), " %s (%s)", when, date_temp);
			free(date_temp);
			return strdup(buf);
		}
		else
			return strdup(when);
	}
	else
		return NULL;
}

static void log_cancel_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	ph_log_data *log_d = data;
	ph_log_check_mode_end(log_d, true);
}

static Evas_Object* log_create_genlist(ph_log_data * log_d)
{
	PH_FN_CALL;
	Evas_Object *genlist = elm_genlist_add(log_d->box);
	p_retvm_if(NULL == genlist, NULL, "elm_genlist_add() return NULL");

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	evas_object_show(genlist);
	evas_object_data_set(genlist, KEY_LOG_DATA, log_d);

	if (!log_d->ug) {
		evas_object_smart_callback_add(genlist, "drag,start,right", log_gl_sweep_right, log_d);
		evas_object_smart_callback_add(genlist, "drag,start,left", log_gl_sweep_left, log_d);
		evas_object_smart_callback_add(genlist, "drag,start,up", log_gl_sweep_up_down, log_d);
		evas_object_smart_callback_add(genlist, "drag,start,down", log_gl_sweep_up_down, log_d);
	}
	elm_genlist_block_count_set(genlist, 10);
	return genlist;
}

static Evas_Object* log_fill_layout(void *data)
{
	ph_log_data *log_d = data;
	if (0 < log_d->log_list_count) {
		if (NULL == log_d->genlist)
			log_d->genlist = log_create_genlist(log_d);
		else {
			elm_genlist_clear(log_d->genlist);
			log_d->pre_date_type = 0;
		}
		log_fill_genlist(log_d);
		return log_d->genlist;
	}
	else {
		Evas_Object *noc;
		noc = phone_create_nocontents(log_d->box, T_(PH_GET_TEXT_BASIC, PHTEXT_NO_LOGS));

		p_retvm_if(NULL == noc, NULL, "phone_create_nocontents() return NULL");
		evas_object_size_hint_weight_set(noc, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(noc, EVAS_HINT_FILL, NO_CONTENTS_ICON_POSITION);
		evas_object_show(noc);

		return noc;
	}
}

Eina_Bool ph_log_data_changed_cb(void *data)
{
	PH_FN_CALL;
	Evas_Object *content;
	ph_log_data *log_d = data;

	if (log_d->update_timer) {
		ecore_timer_del(log_d->update_timer);
		log_d->update_timer = NULL;
	}

	if (log_d->list_idler) {
		ecore_idler_del(log_d->list_idler);
		log_d->list_idler = NULL;
	}
	log_load_list(log_d);

	elm_box_unpack(log_d->box, log_d->content);
	evas_object_del(log_d->content);
	log_d->content = NULL;
	if (log_d->genlist) {
		log_d->genlist = NULL;
		log_d->pre_date_type = 0;
	}

	content = log_fill_layout(log_d);
	p_retvm_if(NULL == content, ECORE_CALLBACK_CANCEL, "log_fill_layout() return NULL");
	log_d->content = content;

	elm_box_pack_end(log_d->box, content);
	if (LOG_VIEW_TYPE_UG_NUMBER_CHECK == log_d->view_type
			|| LOG_VIEW_TYPE_NORMAL_CHECK == log_d->view_type
			|| LOG_VIEW_TYPE_UG_EMAIL_CHECK == log_d->view_type
			|| LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_CHECK == log_d->view_type)
		phone_log_check_mode_start(log_d);
	ph_log_check_update_button_status(log_d);
	if (log_d->logdetail_update) {
		ph_log_detail_update(log_d);
		log_d->logdetail_update = false;
	}

	return ECORE_CALLBACK_CANCEL;
}

static void log_data_lazy_update(const char* view_uri, void* data)
{
	PH_FN_CALL;
	ph_log_data *log_d = data;
	if (log_d->update_timer) {
		ecore_timer_del(log_d->update_timer);
		log_d->update_timer = NULL;
	}

	if (log_d->is_background)
		log_d->update_timer = ecore_timer_add(3.0, ph_log_data_changed_cb, log_d);
	else
		ph_log_data_changed_cb(log_d);

	PH_FN_END;
}

static void log_data_updated(const char* view_uri, void* data)
{
	PH_FN_CALL;
	ph_log_data *log_d = data;
	log_d->logdetail_update = true;
	if (log_d->p_info == NULL)
		log_data_lazy_update(_contacts_phone_log._uri, log_d);
}

static void log_view_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_log_data *log_d = data;
	p_retm_if(NULL == log_d, "log_d is null");

	contacts_error_e err = CONTACTS_ERROR_NONE;

	err = contacts_db_remove_changed_cb(_contacts_person._uri, log_data_lazy_update, log_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_remove_changed_cb() Failed(%d)", err);
	err = contacts_db_remove_changed_cb(_contacts_phone_log._uri, log_data_updated, log_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_remove_changed_cb() Failed(%d)", err);

	vconf_ignore_key_changed(VCONFKEY_SYSTEM_TIME_CHANGED, ph_log_timezone_changed);
	evas_object_data_del(log_d->navi, KEY_LOG_DATA);

	log_free_log_list(log_d);

	if (log_d->list_idler)
		ecore_idler_del(log_d->list_idler);

	if (log_d->popup)
		evas_object_del(log_d->popup);

	if (log_d->p_info) {
		log_d->p_info->alive = false;
		pthread_join(log_d->p_info->thread, NULL);
		ecore_timer_del(log_d->p_info->timer);
		free(log_d->p_info);
	}

	if (log_d->timer)
		ecore_timer_del(log_d->timer);

	if (log_d->update_timer)
		ecore_timer_del(log_d->update_timer);

	phone_detail_log_delete_date_format(log_d->formatter);

	free(log_d);
	PH_FN_END;
}
static Evas_Object* viewby_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *radio;
	viewby_list_item *popup_i = (viewby_list_item*)data;
	Evas_Object *radio_main = NULL;

	if (!strcmp(part, "elm.icon"))
	{
		radio = elm_radio_add(obj);
		p_retvm_if(NULL == radio, NULL, "elm_radio_add() return NULL");

		if (0 == popup_i->index) {
			evas_object_data_set(obj, "radio_main", radio);
		}
		else {
			radio_main = evas_object_data_get(obj, "radio_main");
			p_retvm_if(NULL == radio_main, NULL, "evas_object_data_get() return NULL");
			elm_radio_group_add(radio, radio_main);
		}

		elm_radio_state_value_set(radio, popup_i->index);
		if (radio_main && popup_i->state_index == popup_i->index)
			elm_radio_value_set(radio_main, popup_i->state_index);

		evas_object_size_hint_weight_set(radio, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(radio, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_propagate_events_set(radio, EINA_TRUE);

		return radio;

	}

	return NULL;

}


static char *viewby_gl_text_get(void *data, Evas_Object *obj, const char *part)
{
	viewby_list_item *popup_i = (viewby_list_item*)data;

	if (!strcmp(part, "elm.text"))
	{
		switch(popup_i->index) {
			case VIEWBY_POPUP_ALL_LOG:
				return strdup("All logs");
				break;
			case VIEWBY_POPUP_ALL_CALL_LOG:
				return strdup("All calls");
				break;
			case VIEWBY_POPUP_MISSED_CALL_LOG:
				return strdup("Missed calls");
				break;
			case VIEWBY_POPUP_DIALED_CALL_LOG:
				return strdup("Dialed calls");
				break;
			case VIEWBY_POPUP_RECEIVED_CALL_LOG:
				return strdup("Received calls");
				break;
			case VIEWBY_POPUP_REJECTED_CALL_LOG:
				return strdup("Rejected calls");
				break;
			case VIEWBY_POPUP_ALL_MESSAGE_LOG:
				return strdup("All messages");
				break;
			default:
				return strdup("Error");
				break;
		}
	}
	else
		return NULL;
}

static Eina_Bool viewby_gl_state_get(void *data, Evas_Object *obj, const char *part)
{
	return EINA_FALSE;
}

static void viewby_gl_del(void *data, Evas_Object *obj)
{
	return;
}

static void viewby_gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;

	viewby_list_item *popup_i;
	Elm_Object_Item *item = (Elm_Object_Item *) event_info;
	ph_log_data *log_d =  data;
	Evas_Object * delete_button = NULL;
	p_retm_if(NULL == log_d, "log_d is NULL");
	p_retm_if(NULL == item, "item is null");

	popup_i = (viewby_list_item *) elm_object_item_data_get(item);
	popup_i->state_index = popup_i->index;
	elm_genlist_item_selected_set(item, EINA_FALSE);
	elm_genlist_item_update(item);

	switch(popup_i->index)
	{
		case VIEWBY_POPUP_ALL_LOG:
			log_d->viewby_type = VIEWBY_ALL_LOG;
			break;
		case VIEWBY_POPUP_ALL_CALL_LOG:
			log_d->viewby_type = VIEWBY_ALL_CALL;
			break;
		case VIEWBY_POPUP_MISSED_CALL_LOG:
			log_d->viewby_type = VIEWBY_MISSED_CALL;
			break;
		case VIEWBY_POPUP_DIALED_CALL_LOG:
			log_d->viewby_type = VIEWBY_DIALED_CALL;
			break;
		case VIEWBY_POPUP_RECEIVED_CALL_LOG:
			log_d->viewby_type = VIEWBY_RECEIVED_CALL;
			break;
		case VIEWBY_POPUP_REJECTED_CALL_LOG:
			log_d->viewby_type = VIEWBY_REJECTED_CALL;
			break;
		case VIEWBY_POPUP_ALL_MESSAGE_LOG:
			log_d->viewby_type = VIEWBY_ALL_MESSAGE;
			break;
		default:
			log_d->viewby_type = VIEWBY_ALL_LOG;
	}

	log_load_list(log_d);

	log_list_ctxpopup_hide_cb(log_d, NULL, NULL);

	delete_button = elm_object_item_part_content_get(log_d->navi_item, "toolbar_button1");
	if(0 == log_d->log_list_count)
	{
		elm_object_disabled_set(delete_button, EINA_TRUE);
	}
	else
	{
		elm_object_disabled_set(delete_button, EINA_FALSE);
	}

}

static void log_set_genlist_item_style()
{
	itc.item_style = "2text.2icon";
	itc.decorate_item_style= "mode/slide3";
	itc.decorate_all_item_style = "edit_default";
	itc.func.text_get = log_gl_label_get;
	itc.func.content_get = log_gl_icon_get;
	itc.func.state_get = NULL;

	itc_one.item_style = "2text.2icon";
	itc_one.decorate_item_style = "mode/slide";
	itc_one.decorate_all_item_style = "edit_default";
	itc_one.func.text_get = log_gl_label_get;
	itc_one.func.content_get = log_gl_icon_get;
	itc_one.func.state_get = NULL;

	itc_date.item_style ="grouptitle";
	itc_date.func.text_get = log_gl_sep_date_label_get;

	itc_viewby_list.item_style = "1text.1icon.2";
	itc_viewby_list.func.text_get = viewby_gl_text_get;
	itc_viewby_list.func.content_get = viewby_gl_icon_get;
	itc_viewby_list.func.state_get = viewby_gl_state_get;
	itc_viewby_list.func.del = viewby_gl_del;
}

static void log_genlist_update(Evas_Object *genlist)
{
	PH_FN_CALL;
	Elm_Object_Item *item;
	p_retm_if(NULL == genlist, "Genlist is null");

	item = elm_genlist_first_item_get(genlist);
	while (item) {
		elm_genlist_item_update(item);
		item = elm_genlist_item_next_get(item);
	}
}

API void phone_log_views_update(Evas_Object *navi)
{
	ph_log_data *log_d;
	ph_log_detail_data *logdetail_d;

	log_d = evas_object_data_get(navi, KEY_LOG_DATA);
	if (log_d) {
		phone_detail_log_generate_date_format(log_d->formatter);
		log_genlist_update(log_d->genlist);
	}

	logdetail_d = evas_object_data_get(navi, "logdetail_d");
	if (logdetail_d)
		log_genlist_update(logdetail_d->genlist);
}

void ph_log_timezone_changed(keynode_t* key, void *data)
{
	PH_FN_CALL;
	phone_log_views_update(data);
}

API void phone_log_view_resume(Evas_Object *navi)
{
	PH_FN_CALL;
	ph_log_detail_data *logdetail_d;

	phone_log_missed_info_update(navi);

}

API void phone_log_view_reset(Evas_Object *navi)
{
	PH_FN_CALL;
	ph_log_data *log_d;
	Elm_Object_Item *item;

	log_d = evas_object_data_get(navi, KEY_LOG_DATA);
	p_retm_if(NULL == log_d, "log_d is null");

	if (log_d->child_ug) {
		ug_destroy(log_d->child_ug);
		log_d->child_ug = NULL;
	}

	item = elm_naviframe_bottom_item_get(log_d->navi);
	if (elm_naviframe_top_item_get(log_d->navi) != item)
		elm_naviframe_item_pop_to(item);
	else if (LOG_VIEW_TYPE_NORMAL_CHECK == log_d->view_type)
		ph_log_check_mode_end(log_d, false);
	else if (log_d->item_last_sweep) {
		elm_genlist_item_decorate_mode_set(log_d->item_last_sweep, "slide", EINA_FALSE);
		elm_genlist_item_select_mode_set(log_d->item_last_sweep, ELM_OBJECT_SELECT_MODE_DEFAULT);
		log_d->item_last_sweep = NULL;
	}
	log_d->ctrl_item = NULL;
}

API void phone_viewby_type_set(Evas_Object *layout, int viewby_type)
{
	ph_log_data *log_d;
	log_d = evas_object_data_get(layout, KEY_LOG_DATA);
	p_retm_if(NULL == log_d, "log data is NULL");
	log_d->viewby_type = viewby_type;
}

static void viewby_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;

	Evas_Object *popup;
	Evas_Object *genlist;
	ph_log_data *log_d  = data;
	int i;
	viewby_list_item *popup_i;
	Elm_Object_Item *item;

	if(log_d->popup)
		log_list_ctxpopup_hide_cb(log_d, NULL, NULL);

	popup = elm_popup_add(log_d->navi);
 	log_d->popup = popup;
	elm_object_style_set(popup, "menustyle");
	elm_object_part_text_set(popup, "title,text", T_(CT_GET_TEXT_BASIC, CTTEXT_VIEW));
	genlist = elm_genlist_add(popup);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	Evas_Object *cancel_btn = elm_button_add(popup);
	elm_object_style_set(cancel_btn, "popup_button/default");
	elm_object_text_set(cancel_btn,  S_(PH_SYS_SK_CANCEL));
	elm_object_part_content_set(popup, "button1", cancel_btn);
	evas_object_smart_callback_add(cancel_btn, "clicked", log_list_ctxpopup_hide_cb, log_d);

	for(i = 0; i < VIEWBY_MAX; i++)
	{
		popup_i = (viewby_list_item*)calloc(1, sizeof(viewby_list_item));
		popup_i->index = i;
		popup_i->state_index = log_d->viewby_type;
		item = elm_genlist_item_append(genlist, &itc_viewby_list, (void *)popup_i, NULL, ELM_GENLIST_ITEM_NONE, viewby_gl_sel, log_d);
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_ALWAYS);
	}

	elm_object_content_set(popup, genlist);
	evas_object_show(popup);
}

static void more_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	PH_FN_CALL;
	ph_log_data *log_d  = data;
	Evas_Object *popup = NULL;
	Elm_Object_Item* popup_item;

	log_list_ctxpopup_hide_cb(log_d, NULL, NULL);

	popup = elm_ctxpopup_add(log_d->navi);
	p_retm_if(NULL == popup, "elm_ctxpopup_add() return NULL");

	popup_item = elm_ctxpopup_item_append(popup, T_(CT_GET_TEXT_BASIC, CTTEXT_VIEW), NULL, viewby_clicked_cb, log_d);

	if(log_d->all_log_list_count == 0)
	{
		elm_object_item_disabled_set (popup_item, true);
	}

	elm_ctxpopup_direction_priority_set(popup, ELM_CTXPOPUP_DIRECTION_DOWN, ELM_CTXPOPUP_DIRECTION_UP, -1, -1);

	evas_object_smart_callback_add(popup, "dismissed", log_list_ctxpopup_hide_cb, log_d);

	int x, y, w, h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

	evas_object_move(popup, x+(w/2), y+(h/2));
	evas_object_show(popup);

	log_d->popup = popup;

}

API void phone_log_set_navi(Evas_Object *layout)
{
	PH_FN_CALL;
	ph_log_data *log_d;
	Evas_Object *back_btn;
	Elm_Object_Item *navi_it;
	Evas_Object * more_button = NULL;

	log_d = evas_object_data_get(layout, KEY_LOG_DATA);
	p_retm_if(NULL == log_d, "log data is NULL");

	navi_it = elm_naviframe_bottom_item_get(log_d->navi);
	p_retm_if(NULL == navi_it, "navi item is NULL");

	if(log_d->navi_item){
		phone_navi_item_clear(log_d->navi_item);
	}

	back_btn = elm_button_add(log_d->navi);
	elm_object_style_set(back_btn, "naviframe/back_btn/default");
	elm_object_item_part_content_set(log_d->navi_item, "prev_btn", back_btn);

	switch (log_d->view_type) {
	case LOG_VIEW_TYPE_NORMAL:
	case LOG_VIEW_TYPE_UG_LAUNCH_CALL:

//more button
	more_button = ctui_create_more_btn(log_d->navi, more_btn_clicked_cb, log_d);
	elm_object_item_part_content_set(log_d->navi_item, "toolbar_more_btn", more_button);

//delete button
		if (log_d->view_type == LOG_VIEW_TYPE_NORMAL)
		{
			Evas_Object * delete_button = NULL;

			delete_button = phone_create_toolbar_btn(log_d->navi, S_(PH_SYS_SK_DELETE), log_delete_btn_cb, log_d);
			elm_object_item_part_content_set(navi_it, "toolbar_button1", delete_button);

			if(0 == log_d->log_list_count) {
				elm_object_disabled_set(delete_button, EINA_TRUE);
			}
			else {
				elm_object_disabled_set(delete_button, EINA_FALSE);
			}
		}

//back button
		evas_object_smart_callback_add(back_btn, "clicked", phone_back_btn_clicked_cb, log_d->win);

		break;
	case LOG_VIEW_TYPE_NORMAL_CHECK:
	{
		log_d->ctrl_item =  phone_create_toolbar_btn(log_d->navi, S_(PH_SYS_SK_DELETE), ph_log_check_del_btn_cb, log_d);
		elm_object_item_part_content_set(navi_it, "toolbar_button1", log_d->ctrl_item);

		if(0 == log_d->cnt_checked) {
			elm_object_disabled_set(log_d->ctrl_item, EINA_TRUE);
		}
		else {
			elm_object_disabled_set(log_d->ctrl_item, EINA_FALSE);
		}

		evas_object_smart_callback_add(back_btn, "clicked", log_cancel_clicked_cb, log_d);
	}
		break;

	case LOG_VIEW_TYPE_UG_NUMBER_CHECK:
	case LOG_VIEW_TYPE_UG_EMAIL_CHECK:
	case LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_CHECK:
	{
		Evas_Object * done_btn = NULL;
		log_d->ctrl_item =  phone_create_toolbar_btn(log_d->navi, S_(CT_SYS_SK_DONE), ph_log_check_done_btn_cb, log_d);
		elm_object_item_part_content_set(navi_it, "toolbar_button1", log_d->ctrl_item);

	}
		break;
	case LOG_VIEW_TYPE_UG_NUMBER_SELECT:
	case LOG_VIEW_TYPE_UG_EMAIL_SELECT:
	case LOG_VIEW_TYPE_UG_NUMBER_OR_EMAIL_SELECT:
	default:
		break;
	}

	if (log_d->ug)
		evas_object_smart_callback_add(back_btn, "clicked", phone_ug_destroy_me_cb, log_d->ug);


	if(log_d->ctrl_item)
		ph_log_check_update_button_status(log_d);
}

int ph_log_create_content(ph_log_data *log_d)
{
	PH_FN_CALL;
	int ret;
	Evas_Object *box;
	Evas_Object *content;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	log_d->layout = elm_layout_add(log_d->navi);
	p_retvm_if(NULL == log_d->layout, -1, "elm_layout_add() return NULL");
	elm_layout_file_set(log_d->layout, SELECTION_INFO_EDJ, "selectioninfo_layout");
	box = elm_box_add(log_d->layout);
	elm_object_part_content_set(log_d->layout, "contents", box);
	p_retvm_if(NULL == box, -1, "elm_box_add() return NULL");
	log_d->box = box;

	log_set_genlist_item_style();
	log_load_list(log_d);
	content = log_fill_layout(log_d);
	p_retvm_if(NULL == content, -1, "log_fill_layout() return NULL");
	elm_box_pack_end(box, content);
	log_d->content = content;

	evas_object_event_callback_add(box, EVAS_CALLBACK_DEL,
			log_view_delete_cb, log_d);

	err = contacts_db_add_changed_cb(_contacts_person._uri, log_data_lazy_update, log_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);
	err = contacts_db_add_changed_cb(_contacts_phone_log._uri, log_data_updated, log_d);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_add_changed_cb() Failed(%d)", err);

	evas_object_data_set(log_d->navi, KEY_LOG_DATA, log_d);

	vconf_notify_key_changed(VCONFKEY_SYSTEM_TIME_CHANGED, ph_log_timezone_changed, log_d->navi);

	return 0;
}

API void phone_log_view_language_changed(Evas_Object *navi)
{
	// TODO: Fixme
}

